/*
 * bqfs.i.c
 * Copyright (C) 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <bqfx-config.h>

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include <bqfx-bits.h>
#include <bqfx-defs.h>
#include <bqfx-inter.h>
#include <bqfx-lines.h>
#include <bqfx-names.h>
#include <bqfx-types.h>
#include <kfs.h>
#include <kls.h>

#define false(e)			0
#define true(e)				1

#define beta(___s, ___n) \
    (___s) = (char *) (___s) - ((___n) << 1)

#define dana(___s, ___n) \
    (___s) = (char *) (___s) + ((___n) << 1)

#define posh(___s, ___n) \
    (___s) = (char *) (___s) - (___n)

#define road(___s, ___n) \
    ((char *) (___s) + ((___n) << 1))

#define rule(___s, ___n, ___t) \
    ((char *) (___s) + (___n) * (___t))

#define sans(___s, ___n) \
    ((char *) (___s) + (___n))

#define skip(___s, ___n) \
    (___s) = (char *) (___s) + (___n)

#undef Q
#define Q(e)				((integral_q) (e))

#define bqfset(bqfset)			((struct bqfset_type *) (bqfset))

#define rush(a, b, c)			_libx1f4l2_mood_bqfset((a), (b), (c))

static int land(void *, void *, void *, unsigned, unsigned, unsigned *,
		unsigned, unsigned);
static int last(void *, void *, void *, unsigned, void *, unsigned, unsigned *,
		unsigned, unsigned);
static int lend(void *, void *, unsigned *, unsigned, void *, void *, void *,
		unsigned);
static int lest(void *, void *, unsigned, unsigned, void *, void *, unsigned);
static int lift(void *, void *, unsigned, void **);
static int near(void *, void *, integral_q, unsigned, unsigned, void *, void *,
		unsigned *, unsigned);
static int norm(void *, unsigned);
static int over(void *, void *, integral_q, unsigned, unsigned, void *, void *,
		unsigned, unsigned);
static int push(void *, void *, unsigned);
static int quit(void *, void *, integral_q, unsigned);
static int reel(void *, void *, unsigned, unsigned *, void *, void *);
static int roll(void *, void *, unsigned, unsigned, void *, void *);
static int rift(void *, void *, unsigned, void **);
static int sand(void *, void *, unsigned, unsigned, void *, unsigned, unsigned,
		unsigned);
static int sort(void *, void *, unsigned, unsigned, void *, unsigned,
		unsigned);
static int tank(void *, void *, unsigned, void *, unsigned, unsigned);
static int toll(void *, void *, unsigned, void *, unsigned);

static int
land(void *bqfset, void *data, void *club, unsigned fold, unsigned call,
     unsigned *slip, unsigned just, unsigned size)
{
    int (*move)(void *, void *, unsigned);
    integral_q news;
    unsigned i = integral_q_last;
    void *node;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) club;

    node = rule(club, size, integral_q_bits);

    while (1 < fold) {
	if (news & Q(1) << i) {
	    posh(node, size);
	    move(data, node, 1);
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	    fold -= 2;
	} else {
	    beta(node, size);
	    move(data, node, 1);
	    fold--;
	}
	{
	    posh(data, just);
	}

	i -= 2;
    }
    if (fold) {
	if (news & Q(1) << i) {
	    posh(node, size);
	} else {
	    beta(node, size);
	}
	{
	    move(data, node, 1);
	}
    } else {
    }

    if (call < integral_q_line) {
	*slip = i << 1 | fold;
    } else {
	tank(bqfset, club, i << 1 | fold, club, size << 1, size);
    }

    return 0;
}


static int
last(void *bqfset, void *data, void *tier, unsigned fold, void *turn,
     unsigned call, unsigned *slip, unsigned just, unsigned size)
{
    int (*move)(void *, void *, unsigned);
    integral_q news;
    unsigned i = 3;
    void *node;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) tier;

    node = tier;

    if (1) {
	if (news & Q(1) << 1) {
	    skip(node, size);
	    if (fold) {
		skip(data, just);
		move(data, node, 1);
		fold--;
	    } else {
		move(turn, node, 1);
		turn = NULL;
	    }
	    skip(node, size);
	} else {
	    dana(node, size);
	}
    }
    while (1 < fold) {
	skip(data, just);
	if (news & Q(1) << i) {
	    move(data, node, 1);
	    skip(data, just);
	    skip(node, size);
	    move(data, node, 1);
	    skip(node, size);
	    fold -= 2;
	} else {
	    move(data, node, 1);
	    dana(node, size);
	    fold--;
	}

	i += 2;
    }
    if (fold) {
	skip(data, just);
	move(data, node, 1);
    }

    if (call < integral_q_line) {
	*slip = i << 1 | fold;
    } else {
	data = tier;

	if (fold) {
	    if (news & Q(1) << i) {
		skip(node, size);
		if (turn) {
		    move(turn, node, 1);
		    turn = NULL;
		} else {
		    dana(data, size);
		    move(data, node, 1);
		}
		skip(node, size);
	    } else {
		dana(node, size);
	    }
	} else {
	    i -= 2;
	}

	if (turn) {
	    i += 2;

	    if (news & Q(1) << i) {
		move(turn, node, 1);
		skip(node, size);
		dana(data, size);
		move(data, node, 1);
		skip(node, size);
	    } else {
		move(turn, node, 1);
		dana(node, size);
	    }
	}

	/*
	 * if -turn-, it may be that no data needs transferred.
	 */
	while (node != road(data, size)) {
	    i += 2;

	    if (news & Q(1) << i) {
		dana(data, size);
		move(data, node, 1);
		skip(node, size);
		dana(data, size);
		move(data, node, 1);
		skip(node, size);
	    } else {
		dana(data, size);
		move(data, node, 1);
		dana(node, size);
	    }
	}

	*((integral_q *) tier) = news & ~Q(1) << i | Lx55555555;
    }

    return 0;
}


static int
lend(void *bqfset, void *club, unsigned *i, unsigned fold, void *side,
     void *turn, void *file, unsigned size)
{
    int (*move)(void *, void *, unsigned);
    unsigned ever;
    void *data;

    move = bqfset(bqfset)->link_f.move;

    ever = integral_q_line - fold;

    if (~*i) {
	sand(bqfset, club, *i, fold, club, integral_q_bits, size << 1, size);

	data = rule(club, size, integral_q_bits - 2 - (fold << 1));
    } else {
	data = rule(side, size, integral_q_bits - 2 - (fold << 1));

	last(bqfset, data, club, fold, NULL, 0, i, size << 1, size);

	club = side;
    }

    move(data, turn, 1);

    /*
     * may need no transfer to -club- at this time, in which case -land- is
     * only called to retrieve the separator data.
     */

    land(bqfset, beta(data, size), file, ever, integral_q_last, NULL,
	 size << 1, size);

    move(turn, club, 1);

    *((integral_q *) club) = Lx55555555;

    return 0;
}


static int
lest(void *bqfset, void *tier, unsigned i, unsigned fold, void *turn,
     void *file, unsigned size)
{
    int (*move)(void *, void *, unsigned);
    unsigned deck;
    void *data;

    move = bqfset(bqfset)->link_f.move;

    /*
     * -fold- had been tested less than integral_q_line.
     */

    deck = integral_q_line - fold;

    sort(bqfset, tier, i, fold, tier, size << 1, size);

    data = rule(tier, size, 2 + (fold << 1));

    move(data, turn, 1);

    last(bqfset, data, file, deck - 1, turn, integral_q_last, NULL, size << 1,
	 size);

    *((integral_q *) tier) = Lx55555555;

    return 0;
}


static int
lift(void *bqfset, void *data, unsigned size, void **plus)
{
    int status;

    do {
	status = bqfset(bqfset)->link_m.link
	    (bqfset(bqfset)->link_m.data, plus, integral_q_bits * size);
	if (status) {
	    status = LINK_ERROR;
	} else {
	    MD(data, size)[0] = *plus;
	}
    } while (0);

    return status;
}


static int
near(void *bqdata, void *data, integral_q news, unsigned pass, unsigned mind,
     void *turn, void *node, unsigned *i, unsigned deck)
{
    int (*move)(void *, void *, unsigned), status = 0;
    unsigned even, size;
    void *file;

    move = bqfset(bqdata)->link_f.move;

    size = bqfset(bqdata)->link_a.fpnews.link_v.size;

    file = MD(data, size)[pass - 1];

    news = *(integral_q *) file;

    bq_q_bitcount(even, ~news);

    if (pass == 1) {
	even = integral_q_bits - even;

	move(turn, sans(data, size), 1);

	status = norm(bqdata, even + deck);
	if (status) {
	} else {
	    data = bqfset(bqdata)->link_a.fpnews.node;

	    quit(bqdata, file, news, size);

	    bqfset(bqdata)->link_m.free(bqfset(bqdata)->link_m.data, file);

	    data = rule(data, even, size);

	    move(data, turn, 1);

	    if (mind) {
		last(bqdata, data, node, deck, NULL, 0, i, size, size);
	    } else {
		sand(bqdata, node, *i, deck, data, 1 + deck, size, size);
	    }

	    bqfset(bqdata)->link_a.fpnews.link_v.rate = 0;

	    if (mind) {
	    } else {
		bqfset(bqdata)->link_m.free(bqfset(bqdata)->link_m.data, node);
	    }
	}
    } else {
	if (even < deck + 1) {
	    void *side;

	    if (mind) {
		status = rift(bqdata, data, size, &side);
		if (status) {
		} else {
		    *i = ~0;
		}
	    } else {
		side = NULL;
	    }

	    lend(bqdata, node, i, deck, side, rule(data, size, pass), file,
		 size);
	} else {
	    if (mind) {
		*i = ~0;
	    }

	    reel(bqdata, file, deck, i, rule(data, size, pass), node);

	    *(integral_q *) data = pass - 1;

	    if (mind) {
	    } else {
		bqfset(bqdata)->link_m.free(bqfset(bqdata)->link_m.data, node);
	    }
	}
    }

    return status;
}


static int
norm(void *bqfset, unsigned fold)
{
    int status = 0;
#if __MICRO_FSETS__
    unsigned fail, sans, size;
    void *node;
#endif				/* __MICRO_FSETS__ */

#if __MICRO_FSETS__
    size = bqfset(bqfset)->link_a.fpnews.link_v.size;
#endif				/* __MICRO_FSETS__ */

#if __MICRO_FSETS__
    node = bqfset(bqfset)->link_a.fpnews.node;
#endif				/* __MICRO_FSETS__ */

#if __MICRO_FSETS__
    fail = fold < 3 ? 3 : fold;
    sans = integral_q_bits_and_half;
    for (; fail < sans; sans >>= 1);
    sans <<= 1;
    if (sans == integral_q_bits_and_half) {
	sans *= (size + SIZEOF_integral_q);
    } else {
	sans *= size;
    }
#endif				/* __MICRO_FSETS__ */

#if __MICRO_FSETS__
    /*
     * resizing no matter whether is required.  and to minimum 6.
     */
    status = bqfset(bqfset)->link_m.mode
	(bqfset(bqfset)->link_m.data, &node, sans);
    if (status) {
	if (bqfset(bqfset)->link_a.fpnews.link_v.rate) {
	    *((integral_q *) node) = fold;

	    status = 0;
	} else {
	    status = MODE_ERROR;
	}
    } else {
#endif				/* __MICRO_FSETS__ */
	*((integral_q *) node) = fold;

#if __MICRO_FSETS__
	bqfset(bqfset)->link_a.fpnews.node = node;
    }
#endif				/* __MICRO_FSETS__ */

    return status;
}


static int
over(void *bqtext, void *text, integral_q news, unsigned pass11, unsigned mind,
     void *turn, void *node, unsigned i, unsigned ever)
{
    int (*move)(void *, void *, unsigned), status = 0;
    unsigned deck, size;
    void *file;

    move = bqfset(bqtext)->link_f.move;

    size = bqfset(bqtext)->link_a.fpnews.link_v.size;

    file = MD(text, size)[1];

    news = *(integral_q *) file;

    bq_q_bitcount(deck, ~news);

    if (mind == 1) {
	deck = integral_q_bits - deck;

	move(turn, sans(text, size), 1);

	status = norm(bqtext, deck + ever);
	if (status) {
	} else {
	    void *into;

	    text = bqfset(bqtext)->link_a.fpnews.node;

	    into = rule(text, ever + 1, size);

	    bqfset(bqtext)->link_a.fpnews.node = into;

	    quit(bqtext, file, news, size);

	    bqfset(bqtext)->link_m.free(bqfset(bqtext)->link_m.data, file);

	    move(into, turn, 1);

	    sort(bqtext, node, i, ever, text, size, size);

	    bqfset(bqtext)->link_a.fpnews.link_v.rate = 0;
	    bqfset(bqtext)->link_a.fpnews.node = text;

	    bqfset(bqtext)->link_m.free(bqfset(bqtext)->link_m.data, node);
	}
    } else {
	if (deck < ever + 1) {
	    lest(bqtext, node, i, ever, sans(text, size), file, size);
	} else {
	    void **iran;

	    roll(bqtext, file, ever, i, sans(text, size), node);

	    iran = MD(text, size);

	    memmove(iran, iran + 1, mind * sizeof(void *));

	    mind--;

	    *(integral_q *) text = mind;

	    skip(text, size);

	    move(text, sans(text, size), mind);

	    bqfset(bqtext)->link_m.free(bqfset(bqtext)->link_m.data, node);
	}
    }

    return status;
}


static int
push(void *bqfset, void *node, unsigned size)
{
    int status;
    integral_q news;
    unsigned fold;

    news = *(integral_q *) node;
    bq_q_bitcount(fold, news ^ 1);

    status = norm(bqfset, fold);
    if (status) {
    } else {
	quit(bqfset, node, news, size);

	bqfset(bqfset)->link_m.free(bqfset(bqfset)->link_m.data, node);
    }

    return status;
}


static int
quit(void *bqfset, void *node, integral_q news, unsigned size)
{
    int (*move)(void *, void *, unsigned);
    unsigned i = 3;
    void *data;

    data = bqfset(bqfset)->link_a.fpnews.node;

    move = bqfset(bqfset)->link_f.move;

    if (1) {
	if (news & Q(1) << 1) {
	    skip(data, size);
	    skip(node, size);
	    move(data, node, 1);
	    skip(node, size);
	} else {
	    dana(node, size);
	}
    }
    do {
	skip(data, size);
	if (news & Q(1) << i) {
	    move(data, node, 1);
	    skip(data, size);
	    skip(node, size);
	    move(data, node, 1);
	    skip(node, size);
	} else {
	    move(data, node, 1);
	    dana(node, size);
	}

	i += 2;
    } while (i < integral_q_bits);

    return 0;
}


static int
reel(void *bqfset, void *club, unsigned pick, unsigned *i, void *head,
     void *file)
{
    int (*move)(void *, void *, unsigned);
    integral_q news, seek;
    unsigned j, late, size;
    void *fast, *side;

    size = bqfset(bqfset)->link_a.fpnews.link_v.size;

    move = bqfset(bqfset)->link_f.move;

    /*
     * stuffing pick + 1:
     */
    late = pick;
    l2_q_jls(j, ~*(integral_q *) club, late);
    j ^= 63;
    late = j;

    news = *(integral_q *) club;

    fast = rule(club, size, j);
    side = fast;
    seek = news >> j;
    if (true(j & 1)) {
	if (false(news & Q(1) << j)) {
	} else {
	    skip(side, size);
	}

	j++;
	seek >>= 1;
    }
    for (; seek; seek >>= 2) {
	if (seek & Q(2)) {
	    move(fast, side, 1);
	    skip(fast, size);
	    skip(side, size);
	    move(fast, side, 1);
	    skip(fast, size);
	    skip(side, size);
	} else {
	    move(fast, side, 1);
	    skip(fast, size);
	    dana(side, size);
	}
    }

    move(fast, head, 1);

    *(integral_q *) club = news | (~Q(0) << late);

    if (~*i) {
	sand(bqfset, file, *i, pick, club, integral_q_bits, size, size);
    } else {
	last(bqfset, fast, file, pick, NULL, 0, i, size, size);
    }

    return 0;
}


static int
roll(void *bqfset, void *tier, unsigned pick, unsigned i, void *past,
     void *file)
{
    int (*move)(void *, void *, unsigned);
    integral_q news, seek;
    unsigned j, late, size;
    void *fast, *side;

    size = bqfset(bqfset)->link_a.fpnews.link_v.size;

    move = bqfset(bqfset)->link_f.move;

    /*
     * stuffing pick + 1:
     */
    late = pick + 1;
    l2_q_jfs(j, ~(*(integral_q *) tier & ~1), late);
    late = j;

    news = *(integral_q *) tier;

    fast = rule(tier, size, j + 1);
    side = fast;
    seek = (news & ~Q(3)) << (integral_q_last - j);
    for (; seek; seek <<= 2) {
	if (seek & Q(1) << integral_q_last) {
	    beta(fast, size);
	    beta(side, size);
	    move(fast, side, 2);
	} else {
	    posh(fast, size);
	    beta(side, size);
	    move(fast, side, 1);
	}
    }
    {
	if (news & 2) {
	    posh(fast, size);
	    posh(side, size);
	    move(fast, side, 1);
	} else {
	}
    }

    posh(fast, size);
    move(fast, past, 1);

    *(integral_q *) tier = news | ((Q(2) << late) - 1);

    sort(bqfset, file, i, pick, tier, size, size);

    return 0;
}


static int
rift(void *bqfset, void *data, unsigned size, void **plus)
{
    int status;

    do {
	unsigned copy;

	copy = *(integral_q *) data;

	status = bqfset(bqfset)->link_m.link
	    (bqfset(bqfset)->link_m.data, plus, integral_q_bits * size);
	if (status) {
	    status = LINK_ERROR;
	} else {
	    MD(data, size)[copy] = *plus;
	}
    } while (0);

    return status;
}


static int
sand(void *bqfset, void *club, unsigned i, unsigned fold, void *data,
     unsigned back, unsigned just, unsigned size)
{
    int flip, (*move)(void *, void *, unsigned);
    integral_q news;
    void *node;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) club;

    flip = i & 1;

    i >>= 1;

    node = rule(club, size, i + 1);

    data = rule(data, size, back);

    if (flip) {
	beta(node, size);
	if (news & Q(1) << i) {
	    posh(data, just);
	    move(data, node, 1);
	    fold--;
	} else {
	}
    } else {
	i += 2;
    }

    fold -= news >> 1 & 1;

    while (fold) {
	i -= 2;
	if (news & Q(1) << i) {
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	    fold -= 2;
	} else {
	    posh(data, just);
	    beta(node, size);
	    move(data, node, 1);
	    fold--;
	}
    }
    if (news & 2) {
	if (2) {
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	} else {
	}
    }

    return 0;
}


static int
sort(void *bqfset, void *tier, unsigned i, unsigned fold, void *data,
     unsigned just, unsigned size)
{
    int flip, (*move)(void *, void *, unsigned);
    integral_q news;
    void *node;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) tier;

    flip = i & 1;

    i >>= 1;

    node = rule(tier, size, i - 1);

    if (flip) {
	if (news & Q(1) << i) {
	    skip(node, size);
	    skip(data, just);
	    move(data, node, 1);
	    skip(node, size);
	    fold--;
	} else {
	    dana(node, size);
	}
    } else {
	i -= 2;
    }

    while (fold) {
	i += 2;

	if (news & Q(1) << i) {
	    skip(data, just);
	    move(data, node, 1);
	    skip(node, size);
	    skip(data, just);
	    move(data, node, 1);
	    skip(node, size);
	    fold -= 2;
	} else {
	    skip(data, just);
	    move(data, node, 1);
	    dana(node, size);
	    fold--;
	}
    }

    return 0;
}


static int
tank(void *bqfset, void *club, unsigned i, void *data,
     unsigned just, unsigned size)
{
    int flip, (*move)(void *, void *, unsigned);
    integral_q news;
    void *node;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) club;

    flip = i & 1;

    i >>= 1;

    node = rule(club, size, i + 1);

    data = rule(data, size, integral_q_bits);

    if (flip) {
	beta(node, size);
	if (news & Q(1) << i) {
	    posh(data, just);
	    move(data, node, 1);
	} else {
	}
    } else {
	i += 2;
    }

    while (data != node) {
	i -= 2;
	if (news & Q(1) << i) {
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	} else {
	    posh(data, just);
	    beta(node, size);
	    move(data, node, 1);
	}
    }

    *(integral_q *) club = news & ~Q(0) >> (integral_q_bits - i) | Lx55555555;

    return 0;
}


static int
toll(void *bqfset, void *tier, unsigned i, void *data, unsigned size)
{
    int flip, (*move)(void *, void *, unsigned);
    integral_q news;
    void *node;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) tier;

    flip = i & 1;

    i >>= 1;

    node = rule(tier, size, i - 1);

    if (flip) {
	if (news & Q(1) << i) {
	    skip(node, size);
	    dana(data, size);
	    move(data, node, 1);
	    skip(node, size);
	} else {
	    dana(node, size);
	}
    } else {
	i -= 2;
    }

    dana(data, size);

    while (node != data) {
	i += 2;

	if (news & Q(1) << i) {
	    move(data, node, 1);
	    skip(node, size);
	    dana(data, size);
	    move(data, node, 1);
	    skip(node, size);
	    dana(data, size);
	} else {
	    move(data, node, 1);
	    dana(node, size);
	    dana(data, size);
	}
    }

    *(integral_q *) tier = Lx55555555 | news & ~Q(1) << i;

    return 0;
}


int
_libx1f4l2_gris_bqfset(void *bqdata, void *node, unsigned pass, void *aime,
		       int clear, int class, unsigned copy, int catch,
		       void *turn, void *bqtext)
{
    int (*move)(void *, void *, unsigned), status = 0;
    unsigned size;

    move = bqfset(bqdata)->link_f.move;

    size = bqfset(bqdata)->link_a.fpnews.link_v.size;

    /*
     * node count in -data- may be missing one, while count in -text- is 0.
     * the upper key on the last node has been matched.
     */

    do {
	integral_q news, miss;
	unsigned mind;
	void *data, *text;

	news = *(integral_q *) node;

	mind = *(integral_q *) bqfset(bqtext)->link_a.fpnews.node;

	data = bqfset(bqdata)->link_a.fpnews.node;

	text = bqfset(bqtext)->link_a.fpnews.node;

	miss = news & ~Q(0) << copy << catch;
	if (miss) {
	    news ^= miss | Q(1);
	    if (news) {
		unsigned deck, ever;

		bq_q_bitcount(deck, news);
		bq_q_bitcount(ever, miss);
		if (pass) {
		    if (mind) {
			unsigned i;

			if (deck < integral_q_line) {
			    status = near
				(bqdata, data, news, pass, mind, turn, node,
				 &i, deck);
			    if (status) {
				break;
			    }
			} else {
			    void *side;

			    status = lift(bqtext, text, size, &side);
			    if (status) {
				break;
			    } else {
				land(bqtext, rule(side, size, integral_q_last),
				     node, ever, 0, &i, size, size);

				tank(bqdata, node, i, node, size << 1, size);

				node = side;

				*(integral_q *) node = ~Q(0);

				i = ever & 1;
				i = (integral_q_bits + 1 - ever << 1) - i;
			    }
			}
			if (ever < integral_q_line) {
			    status = over
				(bqtext, text, news, pass, mind, turn, node,
				 i, ever);
			    if (status) {
				break;
			    }
			} else {
			    toll(bqtext, node, i, node, size);
			}
		    } else {
			unsigned i;

			status = norm(bqtext, ever);
			if (status) {
			    break;
			} else {
			    text = bqfset(bqtext)->link_a.fpnews.node;
			    land(bqtext, rule(text, size, ever), node, ever,
				 deck, &i, size, size);
			}
			if (deck < integral_q_line) {
			    status = near
				(bqdata, data, news, pass, mind, turn,
				 node, &i, deck);
			    if (status) {
				break;
			    }
			}
		    }
		} else {
		    status = norm(bqdata, deck);
		    if (status) {
			break;
		    } else {
			unsigned i;

			last(bqdata, bqfset(bqdata)->link_a.fpnews.node, node,
			     deck, NULL, ever, &i, size, size);
			if (ever < integral_q_line) {
			    status = over
				(bqtext, text, news, pass, mind, turn, node, i,
				 ever);
			    if (status) {
				break;
			    }
			}
		    }
		}
	    } else {
		if (pass) {
		    void *trip;

		    if (*(integral_q *) bqfset(bqtext)->link_a.fpnews.node) {
		    } else {
			status = push(bqtext, node, size);
			if (status) {
			    break;
			}
		    }

		    if (pass == 1) {
			node = MD(data, size)[0];

			move(turn, sans(data, size), 1);

			status = rush(bqdata, node, size);
			if (status) {
			    break;
			} else {
			}
		    } else {
			*(integral_q *) data = pass - 1;

			move(turn, rule(data, size, pass), 1);
		    }

		    status = _libx1f4l2_post_bqfset(bqdata, turn, &trip);
		    if (status) {
		    } else {
			move(trip, turn, 1);
		    }
		}
	    }
	} else {
	    void *trip;

	    /*
	     * if -mind- then either or both of -pass- and -class- are zero.
	     *
	     */

	    if (pass) {
		turn = rule(data, size, pass + 1);
	    } else {
		if (mind) {
		    move(turn, sans(text, size), 1);
		} else {
		    move(turn, sans(data, size), 1);
		}
	    }

	    if (mind < 2) {
		if (mind) {
		    status = rush(bqtext, MD(text, size)[1], size);
		    if (status) {
			break;
		    }
		} else {
		    if (class) {
			status = push
			    (bqtext, MD(data, size)[pass + class], size);
			if (status) {
			    break;
			} else {
			    *((integral_q *) data) = pass;
			}
		    } else {
			break;
		    }
		}
	    } else {
		unsigned wrap;
		void **iran, *tune;

		tune = sans(text, size);

		wrap = mind;

		iran = MD(text, size);
		memmove(iran, iran + 1, wrap * sizeof(void *));

		wrap--;

		move(tune, sans(tune, size), wrap);

		*((integral_q *) text) = wrap;
	    }

	    if (pass) {
	    } else {
		status = rush(bqdata, node, size);
		if (status) {
		    break;
		}
	    }

	    status = _libx1f4l2_post_bqfset
		(class & clear ? bqdata : bqtext, turn, &trip);
	    if (status) {
	    } else {
		move(trip, turn, 1);
	    }
	}
    } while (0);

    return status;
}


int
_libx1f4l2_mood_bqfset(void *bqfset, void *node, unsigned size)
{
    int status;

    status = push(bqfset, node, size);
    if (status) {
    } else {
	bqfset(bqfset)->link_a.fpnews.link_v.rate = 0;
    }

    return status;
}


int
_libx1f4l2_noir_bqfset(void *bqdata, void *lock, void *node, void *aime,
		       unsigned copy, int catch, void *turn, void *bqtext)
{
    int (*move)(void *, void *, unsigned), status = 0;
    unsigned size;

    move = bqfset(bqdata)->link_f.move;

    size = bqfset(bqdata)->link_a.fpnews.link_v.size;

    do {
	integral_q news, miss;

	news = *(integral_q *) node;

	miss = news & ~Q(0) << copy << catch;
	if (miss) {
	    news ^= miss | Q(1);
	    if (news) {
		unsigned deck, ever;

		bq_q_bitcount(deck, news);
		bq_q_bitcount(ever, miss);
		if (1) {
		    if (0) {
		    } else {
			unsigned i;
			void *text;

			status = norm(bqtext, ever);
			if (status) {
			    break;
			} else {
			    text = bqfset(bqtext)->link_a.fpnews.node;
			    land(bqtext, rule(text, size, ever), node, ever,
				 deck, &i, size, size);
			}
			if (deck < integral_q_line) {
			    unsigned jack, left;
			    void *file;

			    miss = *(integral_q *) lock;

			    jack = miss >> integral_q_last;
			    left = integral_q_bits - 3 + jack;

			    file = ZD(lock, size)[left];

			    news = *(integral_q *) file;

			    bq_q_bitcount(ever, ~news);

			    if (0) {
			    } else {
				left++;
				if (ever < deck + 1) {
				    lend(bqdata, node, &i, deck, NULL,
					 rule(lock, size, left), file, size);
				} else {
				    reel(bqdata, file, deck, &i,
					 rule(lock, size, left), node);

				    if (jack) {
					void **iran;

					iran = ZD(lock, size)
					    + integral_q_last;
					*iran = *(iran - 1);
					*(integral_q *) lock =
					    miss ^ Q(1) << integral_q_last;
				    } else {
					_libx1f4l2_tank_bqfset
					    (bqdata, lock,
					     (integral_q_bits - 3) << 1,
					     size << 1, size);
				    }

				    bqfset(bqdata)->link_m.free
					(bqfset(bqdata)->link_m.data, node);
				}
			    }
			}
		    }
		}
	    } else {
		if (1) {
		    int jack;
		    void *trip;

		    status = push(bqtext, node, size);
		    if (status) {
			break;
		    }

		    miss = *(integral_q *) lock;

		    jack = miss >> integral_q_last;

		    move(turn, rule(lock, size, integral_q_bits - 2 + jack),
			 1);

		    if (jack) {
			void **iran;

			iran = ZD(lock, size) + integral_q_last;
			*iran = *(iran - 1);
			*(integral_q *) lock = miss ^ Q(1) << integral_q_last;
		    } else {
			_libx1f4l2_tank_bqfset
			    (bqdata, lock, (integral_q_bits - 3) << 1,
			     size << 1, size);
		    }

		    status = _libx1f4l2_post_bqfset(bqdata, turn, &trip);
		    if (status) {
		    } else {
			move(trip, turn, 1);
		    }
		}
	    }
	}
    } while (0);

    return status;
}
