/*
 * bqfv.8.c
 * Copyright (C) 2009-2012, 2018, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <bcount.h>
#include <bqfx-defs.h>
#include <bqfx-inter.h>
#include <bqfx-lines.h>
#include <bqfx-names.h>
#include <bqfx-types.h>
#include <reverse.h>
#include <xfs.h>
#include <xls.h>

#define __HALF_TRANSFER__		1

#define __FINE_EXCESS__			1
#define __FINE_SECOND__			2
#define __FINE_SELECT__			3
#define __FINE_TARGET__			4

#define __FINE_METHOD__			__FINE_EXCESS__

#define ____FINE_METHOD__		__FINE_METHOD__

#define __UNROLL_TRANSFER__		0

#define FINE_ERROR			(LAST_ERROR + 8)

#define false(e)			0

#define beta(___s, ___n) \
    (___s) = (char *) (___s) - ((___n) << 1)

#define cell(___s, ___n, ___t) \
    ((char *) (___s) - (___n) * (___t))

#define dana(___s, ___n) \
    (___s) = (char *) (___s) + ((___n) << 1)

#define life(___s, ___n) \
    ((char *) (___s) - (___n))

#define lola(___s, ___n) \
    ((char *) (___s) - ((___n) << 1))

#define posh(___s, ___n) \
    (___s) = (char *) (___s) - (___n)

#define road(___s, ___n) \
    ((char *) (___s) + ((___n) << 1))

#define rule(___s, ___n, ___t) \
    ((char *) (___s) + (___n) * (___t))

#define sans(___s, ___n) \
    ((char *) (___s) + (___n))

#define skip(___s, ___n) \
    (___s) = (char *) (___s) + (___n)

#define trap(___n, ___s)		((___n) + ((___s) << 2))

#define true(e)				1

#undef Q
#define Q(e)				((integral_q) (e))

#define bqfset(bqfset)			((struct bqfset_type *) (bqfset))

static int clip(void *, void *, unsigned);
static int fine(void *, unsigned, void *);
static int lose(void *, unsigned, void *, unsigned, void *, unsigned);
static int miss(void *, void *, unsigned, unsigned);
static int shut(void *, unsigned, unsigned);

static int
clip(void *bqfset, void *node, unsigned size)
{
    int status;
    unsigned mind;

    mind = *(integral_q *) node;
    if (mind) {
	mind--;
	if (mind) {
	    void *call;

	    call = (char *) node + size;

	    status = bqfset(bqfset)->link_f.move
		(call, (char *) call + size, mind);
	    if (status) {
		status = MOVE_ERROR;
	    } else {
		*((integral_q *) node) = mind;
	    }
	} else {
	    *((integral_q *) node) = 0;
	    if (1) {
		status = 0;
	    }
	}
    } else {
	status = DECK_ERROR;
    }

    return status;
}


static int
fine(void *bqfset, unsigned size, void *node)
{
    int status;

    do {
	integral_q news;

	news = *(integral_q *) node;

	if (news & Q(2)) {
	    status = 0;

	    *(integral_q *) node = news & ~Q(2);

	    if (1) {
		break;
	    }
	} else {
	    if (news & Q(8)) {
		void *fast;

		status = 0;

		*(integral_q *) node = news & ~Q(8);

		fast = rule(node, size, 2);

		bqfset(bqfset)->link_f.move(fast, (char *) fast + size, 1);

		if (1) {
		    break;
		}
	    }
	}
	{
	    integral_q slip;

	    slip = news & ~Lx55555555;
	    if (slip) {
		int (*move)(void *, void *, unsigned);
		void *fast;

		move = bqfset(bqfset)->link_f.move;

		status = 0;

		fast = rule(node, size, 2);

		if (0) {
		} else {
		    if (0) {
		    } else {
#if ____FINE_METHOD__ == __FINE_SELECT__
#elif __FINE_METHOD__ == __FINE_EXCESS__
			integral_q next;
			unsigned copy = 2;

			slip >>= copy + 1;
			slip &= (slip - 1) << 1;
			if (0) {
			} else {
			    next = slip >> 4;
			    size <<= 1;
			    for (; next; next >>= 4) {
				void *side;

				side = (char *) fast + size;
				move(fast, side, 1);
				fast = (char *) side + size;
				move(side, fast, 1);

				copy += 4;
			    }
			    if (slip & Lx44444444) {
				void *side;

				side = (char *) fast + size;
				move(fast, side, 1);
				fast = side;

				copy += 2;
			    }
			    size >>= 1;
			    {
				move(fast, (char *) fast + size, 1);
			    }

			    *(integral_q *) node = news ^ (Q(1) << (copy + 1));
			}
#elif __FINE_METHOD__ == __FINE_SECOND__
#elif __FINE_METHOD__ == __FINE_TARGET__
#endif				/* ____FINE_METHOD__ == __FINE_SELECT__ */
		    }
		}
	    } else {
		*(integral_q *) node = news & ~Q(4);
		if (1) {
		    status = FINE_ERROR + 2;
		}
	    }
	}
    } while (0);

    return status;
}


static int
lose(void *bqfset, unsigned size, void *high, unsigned pass, void *turn,
     unsigned rate)
{
    integral_q pipe;
    unsigned call;
    void *file, *node;
    int status;

    node = ZD(high, size)[pass];

    pipe = *(integral_q *) node;

    while (1) {
	call = ~pipe >> 1 & 1;

	file = ZD(node, size)[call];

	if (rate) {
	    integral_q lane;

	    lane = *(integral_q *) file;

	    if (lane & ~Lx55555555) {
		high = node;

		node = file;

		pipe = lane;

		rate = rate - 1;
	    } else {
		void *past, *tier;

		if (call) {
		    if (1) {
			past = HD(node, size, 2);
			if (pipe & Q(8)) {
			    tier = ZD(node, size)[2];
			} else {
			    tier = ZD(node, size)[3];
			}
		    } else {
		    }
		} else {
		    past = HD(node, size, 1);
		    tier = ZD(node, size)[1];
		}

		status = _libx1f4l2_five_bqfset
		    (bqfset, pipe, &node, call, &file, tier, &past, turn);
		if (status == FREE_ERROR) {
		    break;
		} else {
		}

		high = node;

		node = file;

		pipe = *(integral_q *) file;

		turn = past;

		rate = rate - 1;
	    }
	} else {
	    status = fine(bqfset, size, file);
	    if (status) {
		status = _libx1f4l2_neon_bqfset(bqfset, node, pipe);
	    }

	    break;
	}
    }

    return status;
}


static int
miss(void *bqfset, void *node, unsigned size, unsigned rate)
{
    int status;

    if (1) {
	unsigned mind;
	void *past;

	mind = *(integral_q *) node;

	if (0) {
	} else {
	    past = rule(node, size, 1);
	}

	do {
	    void *file;

	    file = MD(node, size)[0];

	    if (*(integral_q *) file & ~Lx55555555) {
	    } else {
		status = _libx1f4l2_dawn_bqfset
		    (bqfset, mind, node, file, &past, MD(node, size)[1], rate);
		if (status) {
		    if (status < LAST_ERROR) {
			break;
		    } else {
			if (status ^ (LAST_ERROR + integral_q_bits)) {
			    status = shut(bqfset, size, rate);

			    break;
			} else {
			    mind--;

			    file = MD(node, size)[0];
			}
		    }
		}
	    }

	    status = lose
		(bqfset, size, node, trap(0, size), past, rate - 1);
	} while (0);
    }

    return status;
}


static int
shut(void *bqfset, unsigned size, unsigned rate)
{
    int status;
    void *node;

    node = bqfset(bqfset)->link_a.fpnews.node;

    if (rate ^ 1) {
	status = miss(bqfset, node, size, rate - 1);
    } else {
	void *file;

	file = MD(node, size)[0];

	status = fine(bqfset, size, file);
	if (status) {
	    status = _libx1f4l2_grip_bqfset(bqfset, node);
	}
    }

    return status;
}


int
x1f4_mind_bqfset(void *bqfset)
{
    int status;
    unsigned rate, size;

    size = bqfset(bqfset)->link_a.fpnews.link_v.size;

    rate = bqfset(bqfset)->link_a.fpnews.link_v.rate;

    if (rate) {
	status = shut(bqfset, size, rate);
    } else {
	status = clip(bqfset, bqfset(bqfset)->link_a.fpnews.node, size);
    }

    return status;
}
