/*
 * bqfv.d.c
 * Copyright (C) 2009-2011, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <bqfx-bits.h>
#include <bqfx-defs.h>
#include <bqfx-names.h>
#include <bqfx-types.h>

#define bqfset(bqfset)			((struct bqfset_type *) (bqfset))

static int gawk(void *, unsigned, unsigned, unsigned *);
static int slip(void *, unsigned, unsigned, unsigned *);

static int
gawk(void *node, unsigned rate, unsigned size, unsigned *take)
{
    integral_q news;
    unsigned note = half_integral_q_bits;
    void **text;

    news = *(integral_q *) node;

    text = ZD(node, size);

    rate--;

    if (rate) {
	for (; note; note--, news >>= 2) {
	    if (news & 2) {
		gawk(*text, rate, size, take);
	    }

	    text++;

	    gawk(*text++, rate, size, take);
	}
    } else {
	unsigned fold, very = 0;

	for (; note; note--, news >>= 2) {
	    if (news & 2) {
		bq_q_bitcount(fold, *(integral_q *) *text);
		very += fold;
	    }

	    text++;

	    bq_q_bitcount(fold, *(integral_q *) *text++);
	    very += fold;
	}

	*take += very;
    }

    return 0;
}


static int
slip(void *node, unsigned rate, unsigned size, unsigned *take)
{
    unsigned note;
    void **text;

    note = *(integral_q *) node + 1;

    text = MD(node, size);

    rate--;

    if (rate) {
	for (; note; note--) {
	    gawk(*text++, rate, size, take);
	}
    } else {
	unsigned fold, very = 0;

	for (; note; note--) {
	    bq_q_bitcount(fold, *(integral_q *) *text++);
	    very += fold;
	}

	*take += very;
    }

    return 0;
}


int
x1f4_size_bqfset(void *bqfset, unsigned *size)
{
    unsigned rate;
    void *node;

    rate = bqfset(bqfset)->link_a.fpnews.link_v.rate;

    node = bqfset(bqfset)->link_a.fpnews.node;

    if (rate) {
	*size = ~0;
	slip(node, rate, bqfset(bqfset)->link_a.fpnews.link_v.size, size);
    } else {
	*size = *(integral_q *) node;
    }

    return 0;
}
