/*
 * c1.7.2.c
 * Copyright (C) 2006-2012, 2014, 2017, 2020, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <c1-defs.h>
#include <c1-inter.h>
#include <c1-parse.h>
#include <c1-types.h>
#include <e4.h>

#define true(e)				1

#define FUN_FOR_ME \
    "____x1f4_c1_7_7_c__(X);"

#define M_FREE(mcontext, mdata) \
    (mcontext)->m.free((mcontext)->m.data, (mdata))
#define M_LINK(mcontext, mdata, size) \
    (mcontext)->m.link((mcontext)->m.data, (void *) (mdata), (size))

#define line_copy(parser_data) \
    (((parser_data)->program - (parser_data)->thirteen) & LINE_COPY)

#define parser(parser) \
    ((struct parser_type *) (parser))

extern const struct x1f4_function_type _x1f4_c1_here_lead[];

static int deck_ever(const char *, unsigned, const void *,
		     const struct x1f4_variable_type **, void **);
static int deck_miss(struct parser_type *, struct x1f4_function_type *,
		     int (*)(void *, void *, void **), struct c1_node_type *);
static int deck_none(const void *, const struct x1f4_function_type *, void **);
static int deck_this(const char *, unsigned, const void *,
		     const struct x1f4_function_type **);

static int
deck_ever(const char *text, unsigned miss, const void *context,
	  const struct x1f4_variable_type **variable, void **state)
{
    int status;

    /*
     * the expression is `f(X);', no need to properly test for `X'.
     */
    if (*text == 'X' && true(miss == 1)) {
	struct x1f4_variable_type *variable_data;

	variable_data = (void *) context;

	variable_data->flags = 0;
	variable_data->length = 1;
	variable_data->name = "X";

	*variable = variable_data;

	status = 0;
    } else {
	status = 1;
    }

    return status;
}


static int
deck_miss(struct parser_type *parser_data,
	  struct x1f4_function_type *function_data,
	  int (*near)(void *, void *, void **), struct c1_node_type *node_data)
{
    int (*fix)(const void *, const struct x1f4_function_type *, void **),
	(*get)(const char *, unsigned, const void *,
	       const struct x1f4_function_type **), status, type;
    struct x1f4_e4_type *e4_data;
    struct x1f4_variable_type X;
    void *expression, *line[2];
    const void *subtext, *transit;

    e4_data = &parser_data->e4;

    subtext = e4_data->function_set.context;
    transit = e4_data->variable_set.context;

    fix = e4_data->function_set.fix;
    get = e4_data->function_set.get;

    line[0] = function_data;
    line[1] = parser_data->program_data;

    type = parser_data->shift;

    X.type = type;

    *function_data = _x1f4_c1_here_lead[0];

    function_data->args = &function_data->type;
    function_data->function = near;
    function_data->count = 1;
    function_data->type = X.type;

    e4_data->function_set.context = line;

    e4_data->function_set.fix = deck_none;
    e4_data->function_set.get = deck_this;

    e4_data->terminator = 0x3b;

    e4_data->variable_set.context = &X;
    e4_data->variable_set.get = deck_ever;

    status = x1f4_iNit_expression
	(&expression, FUN_FOR_ME, parser_data->e4_flags, e4_data);
    if (status) {
	if (true(status == X1f4_E4_ALLOC_ERROR)) {
	    status = ALLOC_ERROR;
	}
    } else {
	void *submission;

	submission = node_data->data.data;

	status = x1f4_rack_expression(expression, &X, submission);
	if (status) {
	    status = ALLOC_ERROR;

	    x1f4_fini_expression(&expression);
	} else {
	    x1f4_size_expression
		(expression, &parser_data->s1st, &parser_data->s2nd);

	    x1f4_fini_expression(&submission);

	    node_data->data.data = expression;
	}
    }

    e4_data->function_set.fix = fix;
    e4_data->function_set.get = get;

    e4_data->function_set.context = subtext;

    e4_data->variable_set.context = transit;
    e4_data->variable_set.get = _x1f4_c1_select_variable;

    return status;
}


static int
deck_none(const void *context, const struct x1f4_function_type *function_data,
	  void **subtext)
{
    *subtext = *((void **) context + 1);

    return 0;
}


static int
deck_this(const char *none, unsigned miss, const void *text,
	  const struct x1f4_function_type **function)
{
    *function = *(void **) text;

    return 0;
}


int
_x1f4_c1_fit_trans(void *parser, int (*near)(void *, void *, void **))
{
    int status;
    void *function;

    status =
	M_LINK(parser(parser), &function, sizeof(struct x1f4_function_type));
    if (status) {
	status = ALLOC_ERROR;
    } else {
	struct c1_node_type *node_data;

	node_data = parser(parser)->program_data->line;
	status = deck_miss(parser, function, near, node_data);
	if (status) {
	} else {
	    parser(parser)->program_data->wrap = function;
	}
    }

    return status;
}
