/*
 * e4-e.b.c
 * Copyright (C) 2006-2011, 2014, 2017, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e4-defs.h>
#include <e4-types.h>

static int ever_hack(void *, struct e4_hack_type *);
static int ever_last(void *, struct e4_hack_type *, unsigned);

static int
ever_hack(void *expression, struct e4_hack_type *hack_data)
{
    int delete = 0;
    struct e4_atom_type *atom_data;
    unsigned i;

    atom_data = hack_data->atom_data;

    i = hack_data->count;
    for (; i; i--) {
	int type;

	type = atom_data->type;
	if (type == LAST) {
	    const struct e4_last_type *last_data;
	    struct e4_hack_type *hack_data;

	    hack_data = atom_data->data.last.base;
	    last_data = hack_base(hack_data)->last_data;
	    if (last_data->bits & (LEAD_CALL | NODE_LINK | WALK_LINK)) {
		delete = 1;
		if (1) {
		    break;
		}
	    } else {
		delete = ever_last
		    (expression, hack_data, last_data->count);
		if (delete) {
		    break;
		}
	    }
	} else {
	    if (type == THIS) {
		delete = ever_hack(expression, atom_data->data.hack.data);
		if (delete) {
		    break;
		}
	    } else {
		if (type == LOCK) {
		    const struct e4_ever_type *ever_data;
		    struct e4_fine_type *fine_data;
		    struct e4_hack_type *hack_data;

		    hack_data = atom_data->data.last.base;
		    fine_data = hack_fine(hack_data);
		    ever_data = fine_data->ever_data;
		    if (ever_data->bits & FLAT_LINK) {
			const struct e4_last_type *last_data;

			last_data = hack_base(hack_data)->last_data;
			if (last_data->bits
			    & (LEAD_CALL | NODE_LINK | WALK_LINK)) {
			    delete = 1;
			    if (1) {
				break;
			    }
			} else {
			    delete = ever_last
				(expression, hack_data, last_data->count);
			    if (delete) {
				break;
			    }
			}
		    } else {
			delete = 1;
			if (1) {
			    break;
			}
		    }
		} else {
		    if (type == PICK) {
			delete = 1;
			if (1) {
			    break;
			}
		    }
		}
	    }
	}

	atom_data++;
    }

    return delete;
}


static int
ever_last(void *expression, struct e4_hack_type *hack_data, unsigned count)
{
    int delete;

    if (count) {
	do {
	    delete = ever_hack(expression, hack_data);
	    if (delete) {
		break;
	    }

	    count--;

	    hack_data++;
	} while (count);
    } else {
	delete = 0;
    }

    return delete;
}


int
x1f4_seer_expression(struct e4_expression_type *expression_data)
{
    return ever_hack(expression_data, &expression_data->hack);
}
