/*
 * e4.7.2.c
 * Copyright (C) 2006-2014, 2017-2019, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <e4-config.h>

#include <stddef.h>
#include <string.h>

#include <e4-defs.h>
#include <e4-inter.h>
#include <e4-parse.h>
#include <e4-types.h>

#define true(e)				1

#define M_FREE(mcontext, mdata) \
    (mcontext)->m.free((mcontext)->m.data, (mdata))
#define M_LINK(mcontext, mdata, size) \
    (mcontext)->m.link((mcontext)->m.data, (void *) (mdata), (size))
#define M_MODE(mcontext, mdata, size) \
    (mcontext)->m.mode((mcontext)->m.data, (void *) (mdata), (size))

#define case_odb2(lock_data, lock_type, call, atom_data, parser_data) \
    ((lock_data)->odbx_data == _x1f4_e4_pick				      \
     ? 0								      \
     : lead_odb2((lock_data), (lock_type), (call), (parser_data)))

typedef struct half_type {
    int *none;
} half_type;

typedef struct lock_type {
    int type;
    struct e4_atom_type *atom_data;
    const struct e4_odbx_type *odbx_data;
    struct lock_type *lock_data;
} lock_type;

extern const struct e4_odbx_type _x1f4_e4_pick[];

static int case_atom(struct e4_atom_type *, int *, struct parser_type *);
static int lead_odb2(struct lock_type *, int *, int *, struct parser_type *);
static int mind_hack(struct e4_hack_type *, struct half_type *,
		     struct parser_type *);
static int near_ever(struct e4_atom_type *, struct half_type *,
		     struct parser_type *);
static int near_hack(struct e4_hack_type *, struct half_type *,
		     struct parser_type *);
static int near_last(struct e4_atom_type *, struct half_type *,
		     struct parser_type *);
static int near_none(struct e4_atom_type *, struct half_type *,
		     struct parser_type *);
static int near_this(struct e4_atom_type *, struct half_type *,
		     struct parser_type *);
static int push_lock(struct lock_type **, struct e4_atom_type *, int,
		     const struct e4_odbx_type *, struct parser_type *);
static int type_hack(struct e4_hack_type *, struct half_type *,
		     struct parser_type *);

static int (*const near_atom[])(struct e4_atom_type *, struct half_type *,
				struct parser_type *) = {
/* *INDENT-OFF* */
    near_none,
    near_none,
    near_none,
    near_none,
    near_ever,
    near_last,
    near_last,
    near_last,
    near_this
/* *INDENT-ON* */
};

static int
case_atom(struct e4_atom_type *atom_data, int *atom_type,
	  struct parser_type *parser_data)
{
    int status = 0;
    const struct e4_odbx_type *const *list;

    list = parser_data->context_data->odb1_data;
    if (!list) {
	status = _x1f4_e4_080704_error_g(parser_data);
    } else {
	int type;
	struct e4_odb1_type *odb1_data;

	type = *atom_type;

	odb1_data = atom_data->odb1;
	while (odb1_data) {
	    const char *e;
	    struct text_type *text_data;
	    unsigned length;

	    text_data = (struct text_type *) (odb1_data + 1);

	    e = text_data->data;
	    length = text_data->size;
	    while (length) {
		const struct e4_odbx_type *const *odbx, *odbx_data,
		    *odbx_text = NULL;
		unsigned toxic = 0;

		odbx = list;

		odbx_data = *odbx;
		while (odbx_data) {
		    unsigned shift;

		    shift = odbx_data->size;
		    if (toxic < shift) {
			if (shift < length + 1) {
			    if (*odbx_data->args == type) {
				const char *name;

				name = odbx_data->name;
				if (memcmp
				    ((void *) (e - shift), name, shift)) {
				} else {
				    odbx_text = odbx_data;
				    toxic = shift;
				}
			    }
			}
		    }

		    odbx++;
		    odbx_data = *odbx;
		}
		if (!odbx_text) {
		    status = _x1f4_e4_080704_error_i(parser_data, e, length);
		    break;
		} else {
		    e -= toxic;
		    length -= toxic;

		    if (0) {
		    } else {
			type = odbx_text->type;
		    }
		}
	    }
	    if (status) {
		break;
	    }

	    odb1_data = odb1_data->odb1_data;
	}

	if (status) {
	} else {
	    *atom_type = type;
	}
    }

    return status;
}


static int
lead_odb2(struct lock_type *lock_data, int *lock_type, int *call,
	  struct parser_type *parser_data)
{
    const int *args;
    int major, minor, status = 0;
    const struct e4_odbx_type *odbx_data;

    major = lock_data->type;
    minor = *lock_type;

    odbx_data = lock_data->odbx_data;
    args = odbx_data->args;
    if (major != args[0] || minor != args[1]) {
	status = _x1f4_e4_type_risk
	    (&odbx_data, lock_type, major, minor, odbx_data, parser_data);
    } else {
	*lock_type = odbx_data->type;
    }

    return status;
}


static int
mind_hack(struct e4_hack_type *hack_data, struct half_type *half_data,
	  struct parser_type *parser_data)
{
    int none, status;
    struct e4_atom_type *atom_data;
    struct half_type half;

    half.none = &none;

    atom_data = hack_data->atom_data;
    if (1) {
	none = atom_data->type;

	status = near_atom[none](atom_data, &half, parser_data);
	if (status) {
	} else {
	    if (atom_data->odb1) {
		status = case_atom(atom_data, &none, parser_data);
	    }
	}
    }

    if (status) {
    } else {
	*half_data->none = none;
    }

    return status;
}


static int
near_ever(struct e4_atom_type *atom_data, struct half_type *half_data,
	  struct parser_type *parser_data)
{
    *half_data->none = atom_data->data.ever.ever_data->type;

    return 0;
}


static int
near_hack(struct e4_hack_type *hack_data, struct half_type *half_data,
	  struct parser_type *parser_data)
{
    int call = 0, hack_type = 0, none, status = 0;
    struct e4_atom_type *atom_data;
    struct half_type half;
    struct lock_type *lock_data = NULL;
    unsigned count, i;

    count = hack_data->count;

    half.none = &none;

    atom_data = hack_data->atom_data;
    for (i = count; i; i--) {
	const struct e4_odbx_type *odbx_data;

	none = atom_data->type;

	status = near_atom[none](atom_data, &half, parser_data);
	if (status) {
	    break;
	}

	odbx_data = atom_data->odb2._1st;

	if (atom_data->odb1) {
	    status = case_atom(atom_data, &none, parser_data);
	    if (status) {
		break;
	    }
	}

	if (i == 1) {
	    do {
		struct lock_type *free_lock;

		status = case_odb2
		    (lock_data, &none, &call, atom_data, parser_data);
		if (status) {
		    break;
		}

		free_lock = lock_data;
		lock_data = lock_data->lock_data;
		M_FREE(parser_data, free_lock);
	    } while (lock_data);
	    if (status) {
		break;
	    }

	    hack_type = none;
	} else {
	    if (!lock_data) {
		status = push_lock
		    (&lock_data, atom_data, none, odbx_data, parser_data);
		if (status) {
		    atom_data++;
		    break;
		}
	    } else {
		int keep, pass, sink;
		const struct e4_odbx_type *odbx_text;

		odbx_text = lock_data->odbx_data;
		pass = odbx_data->pass;
		sink = odbx_text->pass;

		if (pass < sink) {
		    keep = 1;
		} else {
		    if (pass == sink
			&& (odbx_data->bits & odbx_text->bits & BACK_LINK)) {
			keep = 1;
		    } else {
			keep = 0;

			if (odbx_text->bits & LEAD_TRIP) {
			    if (true((char *) odbx_data
				     - (char *) odbx_text
				     == (offsetof(trip_type, rail)
					 - offsetof(trip_type, lead)))) {
				keep = -1;
			    } else {
				keep = 1;
			    }
			}
		    }
		}

		if (keep == 1) {
		    status = push_lock
			(&lock_data, atom_data, none, odbx_data, parser_data);
		    if (status) {
			atom_data++;
			break;
		    }
		} else {
		    struct lock_type *free_lock = NULL;

		    do {
			status = case_odb2
			    (lock_data, &none, &call, atom_data, parser_data);
			if (status) {
			    break;
			}

			if (free_lock) {
			    M_FREE(parser_data, free_lock);
			}
			free_lock = lock_data;
			lock_data = lock_data->lock_data;

			if (lock_data) {
			} else {
			    break;
			}

			odbx_text = lock_data->odbx_data;

			sink = odbx_text->pass;

			if (pass < sink) {
			    break;
			}

			if (pass == sink
			    && (odbx_data->bits & odbx_text->bits
				& BACK_LINK)) {
			    break;
			}

			if (keep == -1) {
			    status = _x1f4_e4_170621_error_2
				(parser_data, lock_data->odbx_data,
				 odbx_text);

			    break;
			}

			if (odbx_text->bits & LEAD_TRIP) {
			    if (true((char *) odbx_data
				     - (char *) odbx_text
				     == (offsetof(trip_type, rail)
					 - offsetof(trip_type, lead)))) {
				keep = -1;
			    } else {
				break;
			    }
			}
		    } while (1);
		    if (status) {
			atom_data++;

			if (free_lock) {
			    M_FREE(parser_data, free_lock);
			}

			break;
		    }

		    free_lock->lock_data = lock_data;
		    lock_data = free_lock;

		    lock_data->atom_data = NULL;
		    lock_data->odbx_data = odbx_data;
		    lock_data->type = none;
		}
	    }
	}

	atom_data++;
    }

    while (lock_data) {
	struct lock_type *free_lock;

	free_lock = lock_data;
	lock_data = lock_data->lock_data;
	M_FREE(parser_data, free_lock);
    }

    if (status) {
    } else {
	*half_data->none = hack_type;
    }

    return status;
}


static int
near_last(struct e4_atom_type *atom_data, struct half_type *half_data,
	  struct parser_type *parser_data)
{
    int status = 0;

    do {
	struct e4_hack_type *hack_data;
	struct e4_last_type *last_data;

	hack_data = atom_data->data.last.base;
	last_data = (void *) hack_base(hack_data)->last_data;
	if (last_data->bits & NEON_LINE) {
	    C_MODE *neon;
	    int *type;

	    type = half_data->none;

	    neon = (void *) (last_data + 1);

	    atom_data = hack_data->atom_data + hack_data->count - 1;
	    hack_data = atom_data->data.last.base;
	    *type = hack_base(hack_data)->last_data->args[*neon];

	    if (*type == BASE_LOCK) {
		*type = CASE;
	    }

	    if (*type == SIDE) {
		status = type_hack(hack_data + *neon, half_data, parser_data);
		if (status) {
		    break;
		}
	    }

	    /*
	     * it seems the type had been (properly) set already and this one
	     * is not required (or maybe the case to prove otherwise had not
	     * been found):
	     */
	    last_data->type = *type;
	} else {
	    *half_data->none = last_data->type;
	}
    } while (0);

    return status;
}


static int
near_none(struct e4_atom_type *atom_data, struct half_type *half_data,
	  struct parser_type *parser_data)
{
    return 0;
}


static int
near_this(struct e4_atom_type *atom_data, struct half_type *half_data,
	  struct parser_type *parser_data)
{
    return type_hack(atom_data->data.hack.data, half_data, parser_data);
}


static int
push_lock(struct lock_type **lock, struct e4_atom_type *atom_data, int none,
	  const struct e4_odbx_type *odbx_data,
	  struct parser_type *parser_data)
{
    int status;
    struct lock_type *lock_data;

    status = M_LINK(parser_data, &lock_data, sizeof(struct lock_type));
    if (status) {
	status = ALLOC_ERROR;
    } else {
	lock_data->lock_data = *lock;
	*lock = lock_data;

	lock_data->atom_data = atom_data;
	lock_data->odbx_data = odbx_data;
	lock_data->type = none;
    }

    return status;
}


static int
type_hack(struct e4_hack_type *hack_data, struct half_type *half_data,
	  struct parser_type *parser_data)
{
    int status;

    if (hack_data->count == 1) {
	status = mind_hack(hack_data, half_data, parser_data);
    } else {
	status = _x1f4_e4_trip_hack(hack_data, parser_data);
	if (status) {
	} else {
	    if (hack_data->count == 1) {
		status = mind_hack(hack_data, half_data, parser_data);
	    } else {
		status = near_hack(hack_data, half_data, parser_data);
	    }
	}
    }

    return status;
}


int
_x1f4_e4_type_hack(struct e4_hack_type *hack_data, int *type, void *parser)
{
    struct half_type half;

    half.none = type;

    return type_hack(hack_data, &half, parser);
}
