/*
 * lxcall-e.j.c
 * Copyright (C) 2011-2014, 2016, 2017, 2019, 2020, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * recognition of X1f4_E4_SLIP in function_data (actually called function)
 * args vector had been introduced for the benefit of -apply-.  The latter is
 * no longer needing the feature.
 */

#include <string.h>

#include <copy.h>
#include <e4.h>
#include <fd.h>
#include <it.h>
#include <lxcall-defs.h>
#include <lxcall-inter.h>
#include <lxcall-types.h>
#include <lxcast.h>
#include <track.h>

static int rule_line(struct center_type *, const struct x1f4_function_type *,
		     unsigned,
		     unsigned, const struct x1f4_function_type *, unsigned,
		     unsigned, unsigned, unsigned, unsigned,
		     void **, void *);

static int
rule_line(struct center_type *center_data,
	  const struct x1f4_function_type *function_text,
	  unsigned o, unsigned post,
	  const struct x1f4_function_type *function_data, unsigned offset,
	  unsigned h, unsigned flat, unsigned mind, unsigned bits,
	  void **track, void *line)
{
    const int *data;
    int delete, status = 0;
    const int *text;
    struct fxdata_type *missed_data;
    struct x1f4_dxcast_type *dxcast_slip;

    data = function_data->args + offset;
    text = function_text->args;

    missed_data = line;

    dxcast_slip = (void *) (missed_data + flat);

    h += offset;

    delete = o;
    for (; delete; delete--, track++) {
	int name, type;

	name = *data;
	if (name == X1f4_E4_SLIP) {
	    goto skipped;
	}

	type = *text;
	if (type == X1f4_E4_SLIP) {
	    type = name;
	}
	if (type == X1f4_E4_CALL) {
	    if (name == X1f4_E4_CASE) {
		type = X1f4_E4_CASE;
	    }
	}

	if (name ^ type) {
	    status = _libx1f4i0_lxcall_rule_fare
		(center_data, name, type, track, &missed_data, &dxcast_slip);
	    if (status) {
		break;
	    }
	} else {
	    if (bits & POST_COPY) {
		if (type == X1f4_E4_CASE) {
		    *(void **) missed_data = dxcast_slip;
		    *dxcast_slip++ =
			*(struct x1f4_dxcast_type *) *(void **) *track;
		} else {
		    copy_miss(missed_data, name, *track);
		}
		*track = missed_data;
		missed_data++;
	    } else {
		if (post) {
		    if (mind) {
			unsigned pass;

			pass = text[o];
			if ((data[h] ^ pass) & X1f4_E4_POST_XSET) {
			    if (pass & X1f4_E4_POST_XSET) {
			    } else {
				copy_miss(missed_data, name, *track);
				*track = missed_data;
				missed_data++;
			    }
			}
		    }
		} else {
		    if (mind) {
			if (data[h] & X1f4_E4_POST_XSET) {
			    copy_miss(missed_data, name, *track);
			    *track = missed_data;
			    missed_data++;
			}
		    }
		}
	    }
	}

    skipped:
	data++;
	text++;
    }
    if (delete) {
    } else {
	if (bits & POST_COPY) {
	    delete = h - o - offset;
	    for (; delete; delete--, track++) {
		int name;

		name = *data++;
		if (name == X1f4_E4_CASE) {
		    *(void **) missed_data = dxcast_slip;
		    *dxcast_slip++ =
			*(struct x1f4_dxcast_type *) *(void **) *track;
		} else {
		    copy_miss(missed_data, name, *track);
		}
		*track = missed_data;
		missed_data++;
	    }
	} else {
	    if (!mind || function_text->flags & X1f4_E4_LINK_PASS) {
	    } else {
		delete = h - o - offset;
		for (; delete; delete--, track++) {
		    if (data[h] & X1f4_E4_POST_XSET) {
			copy_miss(missed_data, *data, *track);
			*track = missed_data;
			missed_data++;
		    }

		    data++;
		}
	    }
	}
    }

    return status;
}


/*
 * callers may expect only data and link are set in frame_data
 */
int
_libx1f4i0_lxcall_even_list(const struct x1f4_function_type *function_text,
			    const struct x1f4_function_type *function_data,
			    unsigned offset, void ***input, unsigned bits,
			    struct x1f4_frame_type *frame_data,
			    struct x1f4_iturn_type *iturn_data,
			    struct x1f4_composer_type *composer_data,
			    struct x1f4_track_type *track_data,
			    const struct x1f4_eelookup_type *eelookup_data)
{
    struct center_type center;

    center.composer_data = composer_data;
    center.eelookup_data = eelookup_data;
    center.function_text = function_text;
    center.iturn_data = iturn_data;
    center.track_data = track_data;
    center.input = input;

    return _libx1f4i0_lxcall_ever_list
	(function_text, function_data, offset, input, bits, frame_data,
	 &center);
}


/*
 * callers may expect only data and link are set in frame_data
 */
int
_libx1f4i0_lxcall_ever_list(const struct x1f4_function_type *function_text,
			    const struct x1f4_function_type *function_data,
			    unsigned offset, void ***input, unsigned bits,
			    struct x1f4_frame_type *frame_data,
			    struct center_type *center_data)
{
    int delete, status = 0;
    unsigned h, o;

    h = function_data->count;
    o = function_text->count;

    delete = h - o - offset;
    if (delete && !(0 < delete && function_text->flags & X1f4_E4_SIDE_LIST)) {
	status = _libx1f4i0_lxcall_stat_list
	    (center_data->track_data, h - offset, o, function_text);
    } else {
	unsigned flat = 0, mind, post, x = 0;
	const int *data, *text;

	data = function_data->args + offset;
	text = function_text->args;

	mind = function_data->flags & X1f4_E4_POST_TYPE;
	post = function_text->flags & X1f4_E4_POST_TYPE;

	delete = o;
	for (; delete; delete--) {
	    int name, type;

	    name = *data;
	    if (name == X1f4_E4_SLIP) {
		goto skipped;
	    }

	    type = *text;
	    status = _libx1f4i0_lxcall_even_fare
		(center_data, bits, h, o, delete,
		 data, name, mind, text, type, post, 1, &flat, &x);
	    if (status) {
		break;
	    }

	skipped:
	    data++;
	    text++;
	}
	if (delete) {
	} else {
	    if (bits & POST_COPY) {
		flat += h - o - offset;
		delete = h - o - offset;
		for (; delete; delete--, data++) {
		    if (*data == X1f4_E4_CASE) {
			x++;
		    }
		}
	    } else {
		/*
		 * why is this done?  it seems copying these arguments is not
		 * required.
		 */
		if (!mind || function_text->flags & X1f4_E4_LINK_PASS) {
		} else {
		    delete = h - o - offset;
		    for (; delete; delete--) {
			if (data[h] & X1f4_E4_POST_XSET) {
			    flat++;
			}

			data++;
		    }
		}
	    }

	    if (flat | ((bits & (POST_DATA | POST_WAYS)) == POST_WAYS)) {
		unsigned data_size;
		void *line;

		h -= offset;

		if (bits & POST_DATA) {
		    data_size = 0;
		} else {
		    data_size = sizeof(void *) * (1 + h);
		}

		status = frame_data->link
		    (frame_data->data, &line,
		     data_size + flat * sizeof(struct fxdata_type)
		     + x * sizeof(struct x1f4_dxcast_type));
		if (status) {
		    status =
			_libx1f4i0_lxcall_stat_link(center_data->track_data);
		} else {
		    void **track;

		    if (data_size) {
			track = line;
			track++;

			memcpy(track, *input, sizeof(void *) * h);

			*input = track;

			line = track + h;
		    } else {
			track = *input;
		    }

		    status = rule_line
			(center_data, function_text, o, post, function_data,
			 offset, h, flat, mind, bits, track, line);
		}
	    }
	}
    }

    return status;
}
