/*
 * lxcall-e.v.c
 * Copyright (C) 2011-2014, 2016, 2017, 2020, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <e4.h>
#include <fd.h>
#include <it.h>
#include <lxcall-defs.h>
#include <lxcall-inter.h>
#include <lxcall-types.h>
#include <lxcast.h>
#include <track.h>

#define true(e)				1

#define SCALAR_BITS \
    ((1 << X1f4_E4_BILL) | (1 << X1f4_E4_MODE) | (1 << X1f4_E4_REAL))

static int even_grid(struct center_type *, int, const int *, int, unsigned *);

static int
even_grid(struct center_type *center_data, int name, const int *text, int type,
	  unsigned *a)
{
    int status = 0;

    do {
	if (type == X1f4_E4_SLIP) {
	    type = name;
	}

	if (name ^ type) {
	    do {
		if ((type | name) & ~15) {
		} else {
		    if ((1 << type | 1 << name) & ~SCALAR_BITS) {
		    } else {
			type = name;
			break;
		    }
		}

		if (0) {
		} else {
		    if (type == X1f4_E4_CASE) {
			type = name;
			break;
		    }
		}

		if (center_data->iturn_data->link) {
		    const struct x1f4_operator_type *operator_data;

		    if (center_data->iturn_data->link
			(center_data->iturn_data->text, type, name,
			 &operator_data)) {
		    } else {
			type = name;
			break;
		    }
		}

		if (center_data->composer_data->link) {
		    const struct x1f4_linetext_type *linetext_data;

		    if (center_data->composer_data->link
			(center_data->composer_data->text, type, name,
			 &linetext_data)) {
		    } else {
			type = name;
			break;
		    }
		}

		status = _libx1f4i0_lxcall_stat_lock
		    (center_data->track_data, center_data->eelookup_data, name,
		     type, 0 ? -1 : text - center_data->function_text->args,
		     center_data->function_text);
	    } while (0);
	    if (name ^ type) {
		if (1) {
		    break;
		}
	    }

	    (*a)++;
	}

	/*
	 * reference class need not be tested.  it has been tested enough for
	 * the first item.
	 */
    } while (0);

    return status;
}


int
_libx1f4i0_lxcall_even_tile(struct center_type *center_data,
			    int name, int lock,
			    struct fxdata_type *fxdata_data,
			    struct x1f4_dxcast_type *dxcast_data)
{
    int status = 0, type;
    const struct x1f4_function_type *function_text;

    function_text = center_data->function_text;

    if (lock < function_text->count) {
	const int *text;
	unsigned flat = 0;

	text = function_text->args;

	text += lock;

	type = *text;

	status = even_grid(center_data, name, text, type, &flat);

	if (flat && true(!status)) {
	    if (type == X1f4_E4_SLIP) {
		type = name;
	    }

	    if (name ^ type) {
		void **track;

		track = *center_data->input;
		track += lock;

		status = _libx1f4i0_lxcall_rule_fare
		    (center_data, name, type, track, &fxdata_data,
		     &dxcast_data);
	    } else {
	    }
	}
    }

    return status;
}
