/*
 * lxcall.h
 * Copyright (C) 2011-2020, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxcall_H__
#define __LIBx1f4_lxcall_H__

#include <dt.h>
#include <e4.h>
#include <fd.h>
#include <frame.h>
#include <it.h>
#include <lxcast.h>
#include <nt.h>
#include <track.h>
#include <trans.h>

/*
 * _long/post_call flag definitions
 */
#define X1f4_LXCALL_POST_ARGS		(1L << 0)
#define X1f4_LXCALL_POST_COPY		(1L << 1)
#define X1f4_LXCALL_POST_DATA		(1L << 2)
#define X1f4_LXCALL_POST_HOLD		(1L << 3)
#define X1f4_LXCALL_POST_LINE		(1L << 4)
#define X1f4_LXCALL_POST_PLOT		(1L << 5)
#define X1f4_LXCALL_POST_WAYS		(1L << 6)

#define _libx1f4i0_lxcall_suit_type	x1f4_suit_prefix

#define u_case_args_____0 \
    const void *, struct x1f4_linetext_type *, void **
#define u_case_args_____1 \
    const void *screen, struct x1f4_linetext_type *linetext_data, void **args
#define u_case_args_____2 \
    screen, linetext_data, args

#define v_case_args_____0 \
    const void *, const struct x1f4_nodetype_type *,			      \
    struct x1f4_linetext_type *, void **
#define v_case_args_____1 \
    const void *screen, const struct x1f4_nodetype_type *nodetype_data,	      \
    struct x1f4_linetext_type *linetext_data, void **args
#define v_case_args_____2 \
    screen, nodetype_data, linetext_data, args

typedef struct center_type {
    struct x1f4_composer_type *composer_data;
    struct x1f4_dispatch_type *dispatch_data;
    const struct x1f4_eelookup_type *eelookup_data;
    const struct x1f4_function_type *function_text;
    struct x1f4_iturn_type *iturn_data;
    struct x1f4_track_type *track_data;
    void ***input, *output;
} center_type;

typedef struct covert_type {
    int (*lead)(struct center_type *, struct covert_type *, void *, int);
    struct x1f4_frame_type *frame_data;
    struct x1f4_scope_type *scope_data;
    struct x1f4_trans_type *trans_data;
    void *swap, *text;
} covert_type;

typedef struct line_type {
    struct x1f4_function_type function;
    void (*line)(u_case_args_____0);
} line_type;

typedef struct orient_type {
    struct x1f4_function_type function;
    void (*line)(v_case_args_____0);
} orient_type;

typedef struct qssc_type {
    struct x1f4_variable_type variable;
    const void *data;
} qssc_type;

typedef struct ticket_type {
    struct x1f4_2ndscope_type tidespan;
    struct x1f4_operator_type operator;
} ticket_type;

struct lxlock_type;

extern int _libx1f4i0_lxcall_draw_node
    (struct x1f4_frame_type *, struct x1f4_track_type *,
     struct x1f4_nodetype_type *, int, const void *, void *);
extern int _libx1f4i0_lxcall_even_list
    (const struct x1f4_function_type *, const struct x1f4_function_type *,
     unsigned, void ***, unsigned, struct x1f4_frame_type *,
     struct x1f4_iturn_type *, struct x1f4_composer_type *,
     struct x1f4_track_type *, const struct x1f4_eelookup_type *);
extern int _libx1f4i0_lxcall_hold_call
    (void **, void **, unsigned, int *, const int *, unsigned, int, int);
extern int _libx1f4i0_lxcall_iran_call
    (struct x1f4_linetext_type *, const struct x1f4_function_type *,
     struct center_type *, struct covert_type *, unsigned, int *,
     struct fxdata_type *, struct x1f4_dxcast_type *);
extern int _libx1f4i0_lxcall_jack_call
    (struct x1f4_linetext_type *, int, struct center_type *,
     struct covert_type *, int, struct fxdata_type *,
     struct x1f4_dxcast_type *);
extern int _libx1f4i0_lxcall_just_call
    (struct x1f4_linetext_type *, int, unsigned, struct center_type *,
     struct covert_type *, int, const int *,
     struct fxdata_type *, struct x1f4_dxcast_type *);
extern int _libx1f4i0_lxcall_keep_call
    (void **, void **, unsigned, int *, const int *, unsigned, int, int, int);
extern int _libx1f4i0_lxcall_land_node
    (struct x1f4_frame_type *, struct x1f4_track_type *,
     struct x1f4_nodetype_type *, void *, void *);
extern int _libx1f4i0_lxcall_land_slip(int, int);
extern int _libx1f4i0_lxcall_land_text
    (struct x1f4_trans_type *, struct x1f4_track_type *, const char *,
     unsigned, void *);
extern int _libx1f4i0_lxcall_line_node
    (struct x1f4_frame_type *, struct x1f4_track_type *,
     struct x1f4_datatype_type *, void *);
extern int _libx1f4i0_lxcall_line_side
    (void *, void *, const struct x1f4_operator_type *, const char *const *,
     unsigned);
extern int _libx1f4i0_lxcall_line_text
    (struct x1f4_frame_type *, struct x1f4_trans_type *,
     struct x1f4_track_type *, void *);
extern int _libx1f4i0_lxcall_line_type
    (const char **, int, const struct x1f4_eelookup_type *,
     struct trans_type *);
extern int _libx1f4i0_lxcall_line_vice
    (void *, void *, void *, const struct x1f4_operator_type *,
     const char *const *, unsigned);
extern int _libx1f4i0_lxcall_lock_call
    (const struct x1f4_function_type *, unsigned, unsigned,
     struct x1f4_trans_type *, struct x1f4_track_type *,
     struct x1f4_linetext_type **);
extern int _libx1f4i0_lxcall_long_call
    (struct x1f4_linetext_type *, const struct x1f4_function_type *,
     void *, void **, unsigned, struct center_type *, struct covert_type *);
extern int _libx1f4i0_lxcall_mint_call
    (struct covert_type *, struct x1f4_track_type *);
extern int _libx1f4i0_lxcall_miss_line
    (void *, unsigned, void *, struct qssc_type *, struct x1f4_linetext_type *,
     unsigned);
extern int _libx1f4i0_lxcall_near_call(struct covert_type *);
extern int _libx1f4i0_lxcall_pass_node
    (struct x1f4_nodetype_type *, void *, void *);
extern int _libx1f4i0_lxcall_pick_call
    (void **, const struct x1f4_function_type *, unsigned, unsigned,
     struct x1f4_frame_type *, struct x1f4_track_type *,
     struct x1f4_linetext_type **);
extern int _libx1f4i0_lxcall_pick_deck
    (struct x1f4_frame_type *, struct x1f4_track_type *,
     int (*) (void *, void **), int (*) (void *, void **), void *, void *,
     void *);
extern int _libx1f4i0_lxcall_pick_text(struct lxlock_type *, void *);
extern int _libx1f4i0_lxcall_post_call
    (struct x1f4_linetext_type *, const struct x1f4_function_type *,
     void *, unsigned, void **, unsigned, struct x1f4_frame_type *,
     struct x1f4_iturn_type *, struct x1f4_composer_type *,
     struct x1f4_track_type *, const struct x1f4_eelookup_type *);
extern int _libx1f4i0_lxcall_post_text(struct lxlock_type *, void *, void *);
extern int _libx1f4i0_lxcall_push_line
    (void *, unsigned, const void *, struct x1f4_linetext_type **,
     const struct x1f4_nodetype_type *, void **, void **, unsigned,
     const struct orient_type *);
extern int _libx1f4i0_lxcall_rule_call
    (struct x1f4_linetext_type *, const struct x1f4_function_type *,
     void *, struct center_type *, struct covert_type *, unsigned, int *,
     struct fxdata_type *, struct x1f4_dxcast_type *);
extern int _libx1f4i0_lxcall_rule_list
    (const struct x1f4_function_type *, const struct x1f4_function_type *,
     unsigned, unsigned, void ***, struct x1f4_trans_type *,
     struct x1f4_iturn_type *, struct x1f4_composer_type *,
     struct x1f4_track_type *, const struct x1f4_eelookup_type *);
extern int _libx1f4i0_lxcall_rule_node
    (struct x1f4_frame_type *, struct x1f4_track_type *,
     struct x1f4_nodetype_type *, void *, void *);
extern int _libx1f4i0_lxcall_sand_node
    (struct x1f4_frame_type *, struct x1f4_track_type *,
     struct x1f4_nodetype_type *, void *);
extern int _libx1f4i0_lxcall_side_line
    (void *, unsigned, const void *, struct x1f4_linetext_type **, void **,
     unsigned, const struct line_type *);
extern int _libx1f4i0_lxcall_side_node
    (struct x1f4_nodetype_type *, void *, void *);
extern int _libx1f4i0_lxcall_slip_call
    (void **, const struct x1f4_function_type *, unsigned, unsigned,
     struct x1f4_linetext_type **, struct x1f4_linetext_type *, void **);
extern int _libx1f4i0_lxcall_spin_call
    (struct x1f4_linetext_type *, int, void *, struct center_type *,
     struct covert_type *, int);
extern int _libx1f4i0_lxcall_stat_fink
    (struct x1f4_track_type *, int, const struct x1f4_function_type *);
extern int _libx1f4i0_lxcall_stat_free(struct x1f4_track_type *, int);
extern int _libx1f4i0_lxcall_stat_link(struct x1f4_track_type *);
extern int _libx1f4i0_lxcall_stat_lock
    (struct x1f4_track_type *, const struct x1f4_eelookup_type *, int, int,
     int, const struct x1f4_function_type *);
extern int _libx1f4i0_lxcall_such_call
    (struct x1f4_linetext_type *, int, unsigned, void *, struct center_type *,
     struct covert_type *, int, const int *, struct fxdata_type *,
     struct x1f4_dxcast_type *);
extern int _libx1f4i0_lxcall_suit_line
    (struct center_type *, struct covert_type *, const int *, unsigned,
     void **);
extern int _libx1f4i0_lxcall_swap_text
    (struct lxlock_type *, void *, void *, unsigned);
extern int _libx1f4i0_lxcall_zero_line(struct x1f4_linetext_type *);

extern void _libx1f4i0_lxcall_line_0mix
    (v_case_args_____0, int, int, int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_emix
    (v_case_args_____0, int, int, int, struct x1f4_ftsl_type **,
     int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_esix
    (v_case_args_____0, int, void *, int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_fmix
    (v_case_args_____0, int, int, int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_gmix
    (v_case_args_____0, int, int, struct x1f4_ftsl_type **,
     int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_gsix
    (v_case_args_____0, int, void *, int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_imix
    (v_case_args_____0, int, int, struct x1f4_ftaa_type **,
     int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_jmix
    (v_case_args_____0, int, int, int,
     int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_jsix
    (v_case_args_____0, int, int, void *,
     int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_omix
    (v_case_args_____0, int, int, int,
     int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_osix
    (v_case_args_____0, int, int, void *,
     int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_pmix
    (v_case_args_____0, int, int, int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_qmix
    (v_case_args_____0, int, int, struct x1f4_ftca_type **,
     int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_qsix
    (v_case_args_____0, int, void *, int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_wmix
    (v_case_args_____0, int, int, int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_xmix
    (v_case_args_____0, int, int, int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_xsix
    (v_case_args_____0, int, void *, int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_line_ymix
    (v_case_args_____0, int, int, struct x1f4_ftca_type **,
     int (*const *)(void *, void *, void **));
extern void _libx1f4i0_lxcall_lock_wmix
    (v_case_args_____0, int, int, struct x1f4_ftsl_type **, void *,
     int (*const *)(void *, void *, void **));

extern int x1f4_auto_node
    (struct x1f4_frame_type *, struct x1f4_track_type *,
     struct x1f4_nodetype_type *, const void *, void *);
extern int x1f4_auto_text
    (struct x1f4_frame_type *, struct x1f4_track_type *, const char *,
     unsigned, void *);
extern int x1f4_cast_node
    (struct x1f4_frame_type *, struct x1f4_track_type *,
     struct x1f4_nodetype_type *, int, const void *, void *);
extern int x1f4_cast_rate
    (struct x1f4_frame_type *, struct x1f4_track_type *, int, const void *,
     void *);
extern int x1f4_cast_text
    (struct x1f4_frame_type *, struct x1f4_track_type *, const void *, void *);
extern int x1f4_hang_node
    (struct x1f4_frame_type *, struct x1f4_track_type *,
     struct x1f4_datatype_type *, int, void *, void *);
extern int x1f4_raft_node
    (struct x1f4_frame_type *, struct x1f4_track_type *,
     struct x1f4_datatype_type *, void *);
extern int x1f4_suit_prefix(void *, int, const char *, unsigned, void *);

#endif				/* __LIBx1f4_lxcall_H__ */
