/*
 * lxcast-types.h
 * Copyright (C) 2008-2015, 2017, 2021, 2022, 2024, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxcast_types_H__
#define __LIBx1f4_lxcast_types_H__

#include <lxcast-config.h>

#include <dt.h>
#include <frame.h>
#include <e4.h>
#include <fd.h>
#include <it.h>
#include <ln.h>
#include <lxlong.h>
#include <nt.h>
#include <track.h>
#include <trans.h>

typedef struct cxline_type {
    int code;
    const struct x1f4_linetext_type *linetext_data;
} cxline_type;

typedef struct cxnear_type {
    struct {
	int args[3];
	struct x1f4_2ndscope_type scope2nd;
	struct x1f4_linetext_type linetext;
	struct x1f4_operator_type operator;
    } copy;
    struct cxline_type *pick, *rail;
    void *lxcast;
} cxnear_type;

typedef struct dxcast_type {
    int call, type;
    struct fxdata_type data;
} dxcast_type;

typedef struct fringe_type {
    struct fringe_type *fringe_data;
    struct x1f4_linetext_type linetext;
} fringe_type;

typedef struct lxcast_type {
    struct lxlong_type ever_d;
    struct {
	void *lock, *near;
    } line_f;
    struct {
	void *driver_fringe, *object_fringe;
    } link_k;
    struct x1f4_iturn_type link_p;
    struct x1f4_composer_type link_q;
    struct {
	void *fine;
    } link_o;
    struct {
	struct x1f4_datatype_type datatype[2];
    } link_t;
} lxcast_type;

typedef struct lxmiss_type {
    union {
	unsigned miss;
	void *data;
    } data;
} lxmiss_type;

typedef struct lxtext_type {
    struct x1f4_nodetype_type nodetype;
    void *text;
} lxtext_type;

typedef struct screen_type {
    struct x1f4_track_type lock_e;
    struct {
	const struct x1f4_eelookup_type *eelookup_data;
    } lock_g;
    struct x1f4_trans_type lock_m;
    struct {
	void *fine;
    } link_o;
    struct x1f4_iturn_type lock_p;
    struct x1f4_composer_type lock_q;
    struct {
	unsigned miss;
	const void *data;
    } link_t;
    struct x1f4_frame_type lock_v;
    struct x1f4_trans_type lock_w;
} screen_type;

#endif				/* __LIBx1f4_lxcast_types_H__ */
