/*
 * lxdash-a.b.c
 * Copyright (C) 2008-2016, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <d_fset.h>
#include <exerrors.h>
#include <lxfset.h>
#include <lxlead.h>
#include <lxdash-inter.h>
#include <lxdash-types.h>
#include <tcvset.h>

#define dxdash(line) \
    ((struct dxdash_type *) (line))

#define tccase(vset) \
    ((struct x1f4_tccase_type *) (vset))

int
x1f4_copy_lxdash(void *subtext, void *bppp, void **copy,
		 const void *line, const struct excase_type *excase_data)
{
    int status;
    struct lxdash_type *lxdash_data;
    unsigned call;
    void *dxdash;

    lxdash_data = subtext;

    _libx1f4i0_call_lxfset(&call);

    status = lxdash_data->link_m.link
	(lxdash_data->link_m.data, &dxdash,
	 __struct_dxdash_type_alloc_size__ + call);
    if (status) {
	status = _libx1f4i0_lxdash_stat_link(lxdash_data);
    } else {
	void *bqpp;

	do {
	    void *record;

	    if (bppp) {
		bqpp = bppp;
	    } else {
		status = x1f4_init_lxfset(&bqpp);
		if (status) {
		    status = _libx1f4i0_lxdash_stat_link(lxdash_data);
		    if (1) {
			break;
		    }
		}
	    }

	    status = x1f4_post_lxfset(bqpp, (void *) &line, &record);
	    if (status) {
		status = _libx1f4i0_lxdash_stat_link(lxdash_data);
		if (1) {
		    break;
		}
	    } else {
		void *pset;
		const void *text;

		dxdash = pick_dxdash(dxdash);

		{
		    {
			void **node;

			node = record;
			node[0] = (void *) line;
			node[1] = dxdash;
		    }
		}

		lock_excase(dxdash)->nodetype_data =
		    lxdash_data->link_s.nodetype;

		lock_excase(dxdash)->slip = NULL;

		lock_exline(dxdash)->call = 1;

		_libx1f4i0_init_tcvset(&pset, &text);

		status = tccase(text)->push
		    (&pset, &text, excase_data, (void *) &lxdash_data->link_m);
		if (status) {
		    status = X1f4_EX_CRITICAL;

		    _libx1f4i0_fini_tcvset
			(&pset, &text, (void *) &lxdash_data->link_m);
		} else {
		    void *becca[6];

		    lock_exline(dxdash)->pset = pset;
		    lock_exline(dxdash)->text = text;

		    becca[0] = subtext;
		    becca[1] = NULL;
		    becca[2] = dxdash;
		    becca[3] = bqpp;
		    becca[4] = &lxdash_data->link_m;
		    becca[5] = NULL;

		    status = _libx1f4i0_lxdash_fast_line(subtext, dxdash);
		    if (status) {
		    } else {
			status = _libx1f4i0_line_lxfset
			    (dxdash(dxdash)->dash, dxdash(line)->dash,
			     _libx1f4i0_lxdash_copy_link,
			     _libx1f4i0_lxdash_free_link, becca);
			if (status) {
			    status = _libx1f4i0_lxdash_stat_line
				(lxdash_data, 0, status);

			    _libx1f4i0_flat_lxfset
				(((struct dxdash_type *) dxdash)->dash);
			}
		    }
		    if (status) {
			x1f4_lose_lxlead(becca[5], &lxdash_data->link_m);

			_libx1f4i0_fini_tcvset
			    (&pset, &text, (void *) &lxdash_data->link_m);
		    } else {
			dxdash(dxdash)->size = dxdash(line)->size;

			status = x1f4_beta_lxlead
			    (bqpp, becca[5], &lxdash_data->link_m);
			if (status) {
			    _libx1f4i0_lxdash_slip_line(lxdash_data, &dxdash);
			} else {
			    *copy = dxdash;
			}

			dxdash = NULL;
		    }
		}
	    }
	} while (0);

	if (bqpp) {
	    if (bppp) {
	    } else {
		int excess;

		excess = x1f4_fini_lxfset(&bqpp);
		if (excess) {
		    status = _libx1f4i0_lxdash_stat_free(lxdash_data, status);
		}
	    }
	}

	if (dxdash) {
	    lxdash_data->link_m.free
		(lxdash_data->link_m.data, lock_cxline(dxdash));
	}
    }

    return status;
}
