/*
 * lxdash-c.f.c
 * Copyright (C) 2013-2019, 2021, 2022, 2024, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <copy.h>
#include <e4.h>
#include <exerrors.h>
#include <lxcast.h>
#include <lxdash-drive.h>
#include <lxdash-inter.h>

static int line_trap(void *, void *, void **);

static int
line_trap(void *context, void *output, void **input)
{
    int status;
    struct lxdash_type *lxdash_data;
    struct x1f4_function_type *function_data;
    unsigned count;

    lxdash_data = context;

    function_data = ((void **) input[-1])[X1f4_E4_RENDITION];

    status = 0;

    count = function_data->count - 1;
    if (count) {
	const int *args;
	int line[3];
	struct x1f4_function_type function;
	void *slip[4], **strip;

	strip = input[-1];

	slip[0] = strip;
	slip[1] = *input;

	/*
	 * pretty error reporting:
	 */
	function.length = 5;
	function.name = "i_fit";

	function.args = line;

	args = function_data->args + 1;

	strip[X1f4_E4_RENDITION] = &function;

	do {
	    int type;

	    type = *args++;

	    line[2] = *args++;

	    if (type == X1f4_E4_CASE) {
		slip[2] = &I_CASE(*++input)->data;
	    } else {
		slip[2] = *++input;
	    }

	    slip[3] = *++input;
	    status = _libx1f4i0_lxdash_p_forward(lxdash_data, NULL, slip + 1);
	    if (status) {
		break;
	    }
	} while (count -= 2);
    }

    return status;
}


int
_libx1f4i0_lxdash_f_centric(void *context, void *output, void **input)
{
    int status;

    do {
	const int *args;
	struct lxdash_type *lxdash_data;
	struct x1f4_function_type *function_data;
	unsigned class, count;

	lxdash_data = context;

	function_data = ((void **) input[-1])[X1f4_E4_RENDITION];

	class = count = function_data->count;

	if (count & 1) {
	} else {
	    status = _libx1f4i0_lxdash_stat_even(lxdash_data);
	    if (1) {
		break;
	    }
	}

	args = function_data->args + 1;
	for (--count; count; count -= 2) {
	    int type;

	    type = *args;
	    if (type == X1f4_E4_CASE) {
		type = I_CASE(input[class - count])->type;
	    }
	    if (type != X1f4_E4_MODE) {
		break;
	    } else {
		args += 2;
	    }
	}
	if (count) {
	    status = _libx1f4i0_lxdash_stat_gear
		(lxdash_data, "i_fit", class - count);
	    if (1) {
		break;
	    }
	}

	status = line_trap(context, output, input);
	if (status) {
	} else {
	    l_LINE(I_LINE(*input), output);
	}
    } while (0);

    return status;
}
