/*
 * lxdash-d.y.c
 * Copyright (C) 2008-2016, 2018, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxdash-config.h>

#include <copy.h>
#include <d_fset.h>
#include <e4.h>
#include <fd.h>
#include <lxdash-drive.h>
#include <lxdash-inter.h>
#include <lxdash-types.h>

extern const struct x1f4_bqnorm_type _libx1f4i0_lxdash_u_norm,
    _libx1f4i0_lxdash_w_norm;

int
_libx1f4i0_lxdash_y_derived(void *context, void *output, void **input)
{
    int call, status;
    struct dxdash_type *dxdash_data;
    struct effect_type effect;
    struct lxdash_type *lxdash_data;
    void *dash;
#if __CLIP_EXTRACT__
#else
    void *data;
#endif				/* __CLIP_EXTRACT__ */

    lxdash_data = context;

    dxdash_data = I_LINE(input[1]);

    dash = dxdash_data->dash;

    call = *((struct x1f4_function_type *)
	((void **) input[-1])[X1f4_E4_RENDITION])->args;

    effect.mode = I_MODE(input[2]);
    effect.type = call;

#if __CLIP_EXTRACT__
    _libx1f4i0_norm_lxfset(dash, &_libx1f4i0_lxdash_w_norm);
    status = _libx1f4i0_cast_lxfset(dash, &effect);
    _libx1f4i0_norm_lxfset(dash, &_libx1f4i0_lxdash_u_norm);
    if (status) {
	if (status == X1f4_BQFSET_MISS_CLASS) {
	    if (effect.type ^ call) {
		status = _libx1f4i0_lxdash_stat_type
		    (lxdash_data, effect.mode, effect.type, call);
	    } else {
		status = _libx1f4i0_lxdash_stat_miss(lxdash_data, effect.mode);
	    }
	} else {
	    status = _libx1f4i0_lxdash_stat_line(lxdash_data, 0, status);
	}
    } else {
	unsigned link;

	dxdash_data->size--;

	link = effect.link;

	if (~link) {
	    struct lxtext_type *lxtext_data;

	    lxtext_data = lxdash_data->link_f.text;
	    lxtext_data += link;
	    if (lxtext_data->lxtype.push) {
		status = lxtext_data->lxtype.push
		    (lxtext_data->lxtype.context, *input,
		     I_USER(&effect.fxdata), lock_excase(dxdash_data));
	    } else {
		copy_miss(*input, effect.type, &effect.fxdata);
	    }
	} else {
	    copy_miss(*input, effect.type, &effect.fxdata);
	}

	status = _libx1f4i0_lxrail_cast_data
	    (lxdash_data, output, effect.type, *input);
    }
#else
    status = x1f4_find_lxfset(dash, &effect, &data);
    if (status) {
	struct effect_type *effect_data;
	unsigned call, type;

	effect_data = data;

	type = effect_data->type;
	call = *((struct x1f4_function_type *)
		 ((void **) input[-1])[X1f4_E4_RENDITION])->args;
	if (type != call) {
	    status = _libx1f4i0_lxdash_stat_type
		(lxdash_data, effect.mode, type, call);
	} else {
	    unsigned link;

	    link = effect_data->link;

	    do {
		if (~link) {
		    struct lxtext_type *lxtext_data;

		    lxtext_data = lxdash_data->link_f.text;
		    lxtext_data += link;
		    if (lxtext_data->lxtype.push) {
			status = lxtext_data->lxtype.push
			    (lxtext_data->lxtype.context, *input,
			     I_USER(&effect_data->fxdata),
			     lock_excase(dxdash_data));
			if (status) {
			    break;
			}
		    } else {
			copy_miss(*input, type, &effect_data->fxdata);
		    }

		    effect_data->link = ~0;
		} else {
		    copy_miss(*input, type, &effect_data->fxdata);
		}

		status = _libx1f4i0_cast_lxfset(dash, &effect);
		if (status) {
		    status =
			_libx1f4i0_lxdash_stat_line(lxdash_data, 0, status);
		} else {
		    dxdash_data->size--;

		    status = _libx1f4i0_lxrail_cast_data
			(lxdash_data, output, type, *input);
		}
	    } while (0);
	}
    } else {
	status = _libx1f4i0_lxdash_stat_miss(lxdash_data, effect.mode);
    }
#endif				/* __CLIP_EXTRACT__ */

    return status;
}
