/*
 * lxdash-i.w.c
 * Copyright (C) 2008-2016, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxdash-config.h>

#include <config.h>

#include <string.h>

#include <d_fset.h>
#include <lxdash-inter.h>
#include <lxdash-types.h>

#if SIZEOF_VOID_P == 8
# define HALF				040
#else
# define HALF				020
#endif				/* SIZEOF_VOID_P == 8 */

#define SECOND(___v)			(effect(___v)->mode)

#define effect(___v)			((struct effect_type *) (___v))

#define rule_type(text, effect_data, miss, plus) \
    {									      \
	int type;							      \
									      \
	type = effect_data->type;					      \
	if (effect(text)->type ^ type) {				      \
	    status = miss;						      \
	    effect(text)->type = type;					      \
	} else {							      \
	    status = plus;						      \
	    *effect(text) = *effect_data;				      \
	}								      \
    }

static int fine_fare(void *, void *);
static int fine_look(void *, void *, unsigned *);
static int fine_move(void *, void *, unsigned);
static int fine_pick(void *, void *, unsigned *);

const struct x1f4_bqnorm_type _libx1f4i0_lxdash_w_norm = {
/* *INDENT-OFF* */
    fine_fare,
    fine_look,
    fine_move,
    fine_pick
/* *INDENT-ON* */
};

static int
fine_fare(void *text, void *node)
{
    int status;
    X1f4_E4_C_MODE u, v;

    u = SECOND(text);
    v = SECOND(node);

    if (u ^ v) {
	status = u < v ? -1 : 1;
    } else {
	rule_type(text, effect(node), 1, 0);
    }

    return status;
}


static int
fine_look(void *text, void *node, unsigned *pick)
{
    int status;
    unsigned size;

    size = *(integral_q *) node;
    if (size) {
	X1f4_E4_C_MODE u, v;
	struct effect_type *effect_call;

	u = SECOND(text);

	effect_call = node;
	effect_call++;

#if __LINEAR_LOOK__
	for (--size; size; size--) {
	    if (u <= SECOND(effect_call)) {
		break;
	    } else {
		effect_call++;
	    }
	}
#else
	while (size ^ 1) {
	    struct effect_type *effect_slip;
	    unsigned half;

	    half = size >> 1;
	    effect_slip = effect_call + half;

	    v = SECOND(effect_slip);
	    if (u < v) {
		size = half;
	    } else {
		effect_call = effect_slip;
		size -= half;
	    }
	}
#endif				/* __LINEAR_LOOK__ */

	v = SECOND(effect_call);
	if (v ^ u) {
	    status = 0;

	    if (u < v) {
		effect_call--;
	    } else {
	    }
	} else {
	    rule_type(text, effect_call, 0, 1);
	}

	*pick = effect_call - (struct effect_type *) node;
    } else {
	*pick = 0;

	status = 0;
    }

    return status;
}


static int
fine_move(void *target, void *source, unsigned count)
{
    memmove(target, source, count * sizeof(struct effect_type));

    return 0;
}


static int
fine_pick(void *text, void *node, unsigned *pick)
{
    X1f4_E4_C_MODE u;
    int status;
    struct effect_type *effect_call;
#if __BINARY_PICK__
    unsigned half = HALF;
#endif				/* __BINARY_PICK__ */
    unsigned slip;

    u = SECOND(text);

    effect_call = node;

#if __LINEAR_PICK__
    effect_call += 2;
    slip = (SIZEOF_VOID_P << 2) - 1;
    for (; slip; slip--) {
	if (u < SECOND(effect_call)) {
	    break;
	} else {
	    effect_call += 2;
	}
    }

    effect_call = node;

    slip = ((SIZEOF_VOID_P << 2) - 1 - slip) << 1;
#else
    slip = 0;

    do {
	if (u < SECOND(effect_call + slip + half)) {
	} else {
	    slip += half;
	}

	half >>= 1;
    } while (half ^ 1);
#endif				/* __LINEAR_PICK__ */

    effect_call += slip;

    if (*(integral_q *) node & (integral_q) 2 << slip) {
	if (u < SECOND(effect_call + 1)) {
	    if (slip) {
		if (u ^ SECOND(effect_call)) {
		    status = 0;
		} else {
		    rule_type(text, effect_call, 0, 1);
		}
	    } else {
		status = 0;
	    }
	} else {
	    slip++;
	    effect_call++;
	    if (u ^ SECOND(effect_call)) {
		status = 0;
	    } else {
		rule_type(text, effect_call, 0, 1);
	    }
	}

	*pick = slip;
    } else {
	*pick = slip + 1;
	if (slip) {
	    if (u ^ SECOND(effect_call)) {
		status = 0;
	    } else {
		rule_type(text, effect_call, 0, 1);
	    }
	} else {
	    if (1) {
		status = 0;
	    }
	}
    }

    return status;
}
