/*
 * lxdash-p.e.c
 * Copyright (C) 2008-2018, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e4.h>
#include <d_fset.h>
#include <lxdash-drive.h>
#include <lxdash-inter.h>
#include <lxdash-parts.h>
#include <lxdash-types.h>

extern const struct x1f4_bqnorm_type _libx1f4i0_lxdash_u_norm;

int
_libx1f4i0_lxdash_e_pending(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE head, tail;
    int status;
    struct dxdash_type *dxdash_data;
    struct lxdash_type *lxdash_data;
    struct x1f4_bqfset_type bqfset;
    void *dash, *part;

    dxdash_data = I_LINE(input[0]);

    lxdash_data = context;

    bqfset.fare = _libx1f4i0_lxdash_u_norm.fare;
    bqfset.look = _libx1f4i0_lxdash_u_norm.look;
    bqfset.move = _libx1f4i0_lxdash_u_norm.move;
    bqfset.pick = _libx1f4i0_lxdash_u_norm.pick;

    bqfset.trans = &lxdash_data->link_m;

    bqfset.size = sizeof(struct effect_type);

    dash = dxdash_data->dash;

    head = I_MODE(input[1]);
    tail = I_MODE(input[2]);

    status = _libx1f4i0_init_lxfset
	(&part, X1f4_BQFSET_SIZE_FRAME | X1f4_BQFSET_TRANS_MASK, &bqfset);
    if (status) {
    } else {
	int excess;
	struct effect_type effect;
	struct pxedit_type pxedit;

	pxedit.excase_data = lock_excase(dxdash_data);
	pxedit.lxtext_data = lxdash_data->link_f.text;
	pxedit.miss = 0;
	pxedit.success = 0;

	if (tail < head) {
	    effect.mode = head;
	    status = _libx1f4i0_tear_lxfset(dash, &effect, 0, part);
	    if (status) {
	    } else {
		_libx1f4i0_lime_lxfset
		    (part, &pxedit, _libx1f4i0_lxdash_miss_link);

		_libx1f4i0_high_lxfset(part);

		dxdash_data->size -= pxedit.miss;

		pxedit.miss = 0;
		pxedit.success = 0;

		effect.mode = tail;
		status = _libx1f4i0_tear_lxfset(dash, &effect, 1, part);
		if (status) {
		} else {
		    _libx1f4i0_swap_lxfset(part, dash);

		    _libx1f4i0_lime_lxfset
			(part, &pxedit, _libx1f4i0_lxdash_miss_link);

		    dxdash_data->size -= pxedit.miss;
		}
	    }
	} else {
	    void *half;

	    status = _libx1f4i0_init_lxfset
		(&half, X1f4_BQFSET_SIZE_FRAME | X1f4_BQFSET_TRANS_MASK,
		 &bqfset);
	    if (status) {
	    } else {
		effect.mode = tail;
		status = _libx1f4i0_tear_lxfset(dash, &effect, 1, part);
		if (status) {
		} else {
		    effect.mode = head;
		    status = _libx1f4i0_tear_lxfset(dash, &effect, 0, half);
		    if (status) {
		    } else {
			status = _libx1f4i0_join_lxfset(dash, part);
			if (status) {
			} else {
			    _libx1f4i0_lime_lxfset
				(half, &pxedit, _libx1f4i0_lxdash_miss_link);

			    dxdash_data->size -= pxedit.miss;
			}
		    }
		}

		excess = x1f4_fini_lxfset(&half);
		if (excess) {
		    if (status) {
		    } else {
			status = excess;
		    }
		}
	    }
	}

	excess = x1f4_fini_lxfset(&part);
	if (excess) {
	    if (status) {
	    } else {
		status = excess;
	    }
	}
    }

    if (status) {
	status = _libx1f4i0_lxdash_stat_line(lxdash_data, 0, status);
    } else {
	l_LINE(dxdash_data, output);
    }

    return status;
}
