/*
 * lxdata-p.t.c
 * Copyright (C) 2013-2015, 2017, 2021, 2023, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <stdint.h>

#include <e4.h>
#include <integral_q.h>
#include <lxdata-drive.h>
#include <lxdata-types.h>

#if SIZEOF_UNSIGNED_LONG == 4
# define HEAD_ZERO			UINT32_C(0x80808080)
# define MISS_ZERO			UINT32_C(0x01010101)
#else
# define HEAD_ZERO			UINT64_C(0x8080808080808080)
# define MISS_ZERO			UINT64_C(0x0101010101010101)
#endif				/* SIZEOF_UNSIGNED_LONG == 4 */

static void fail_data(unsigned char *, unsigned, unsigned, unsigned);

static void
fail_data(unsigned char *data, unsigned size, unsigned turn, unsigned side)
{
    unsigned slip;

    slip =
	(SIZEOF_UNSIGNED_LONG
	 - ((integral_q) data & (SIZEOF_UNSIGNED_LONG - 1)))
	& (SIZEOF_UNSIGNED_LONG - 1);
    if (slip) {
	if (size < slip) {
	    slip = size;
	}

	size -= slip;

	for (; slip; slip--, data++) {
	    if (*data == turn) {
		*data = side;
	    }
	}
    }

    if (size) {
	slip = size / SIZEOF_UNSIGNED_LONG;
	if (slip) {
	    unsigned long *deck, hole, rule;

            hole = turn;

	    hole |= hole << 010;
	    hole |= hole << 020;
# if SIZEOF_UNSIGNED_LONG == 8
	    hole |= hole << 040;
#endif				/* SIZEOF_UNSIGNED_LONG == 8 */

	    rule = side;

	    rule |= rule << 010;
	    rule |= rule << 020;
# if SIZEOF_UNSIGNED_LONG == 8
	    rule |= rule << 040;
#endif				/* SIZEOF_UNSIGNED_LONG == 8 */

	    deck = (unsigned long *) data;

	    do {
		unsigned long miss, this, trap;

		miss = *deck;
		this = miss ^ hole;
		trap = ((this - ((this ^ MISS_ZERO) & MISS_ZERO)) & ~this)
		    & HEAD_ZERO;
		if (trap) {
		    trap |= trap - (trap >> 7);
		    *deck = (miss & ~trap) | (trap & rule);
		}

		deck++;

		slip--;
	    } while (slip);

	    data = (unsigned char *) deck;

	    size &= SIZEOF_UNSIGNED_LONG - 1;
	}

	for (; size; size--, data++) {
	    if (*data == turn) {
		*data = side;
	    }
	}
    }
}


int
_libx1f4i0_lxdata_t_pending(void *context, void *output, void **input)
{
    struct dxdata_type *dxdata_data;
    unsigned size;

    dxdata_data = I_DATA(input[0]);

    size = dxdata_data->length;
    if (size) {
	unsigned side, turn;

	turn = I_MODE(input[1]);
	side = I_MODE(input[2]);
	if ((side | turn) < 256) {
	    fail_data((void *) dxdata_data->string, size, turn, side);
	}
    }

    l_DATA(dxdata_data, output);

    return 0;
}
