/*
 * lxline-a.0.c
 * Copyright (C) 2008-2019, 2022, 2024, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * TODO
 *
 * merge copy_line into fast_line
 */

#include <lxline-config.h>

#include <stddef.h>
#include <string.h>

#include <af.h>
#include <e4-m.0.h>
#include <e4.h>
#include <e4fine.h>
#include <l_note.h>
#include <lxcall.h>
#include <lxline-defs.h>
#include <lxline-inter.h>
#include <lxline-types.h>
#include <lxnext.h>
#include <mxdeck.h>

#define MAKE_DOUBLE(a, b)		a b
#define MAKE_SINGLE(a, b)		a

#define l_e_type_post			3

#define l_ec_set_post			3

#define l_g_type_post			3

#define l_gc_set_post			3

#define l_j_type_post			3

#define l_o_type_post			3

#define lbe_type_post			2

#define lbg_type_post			2

#define lfe_type_post			2

#define lfg_type_post			2

#define true(e)				1

#define part(screen) \
    ((struct depart_type *) (screen))
#define screen(screen) \
    ((struct screen_type *) ((struct depart_type *) (screen))->screen_data)

#define fb_size() \
    (sizeof(side_o) / sizeof(struct x1f4_operator_type))
#define fp_size() \
    (sizeof(side_u) / sizeof(struct x1f4_operator_type))

#define fi_size(___p)			((___p) + 7)

#define fm_size(none) \
    sizeof(struct lxmiss_type)

extern const struct x1f4_eelookup_type _x1f4_type_lookup;

static int init_line(struct lxline_type *, unsigned,
		     const struct screen_type *);
static int line_data(struct lxline_type *, unsigned,
		     const struct depart_type *, void *, void *, void **,
		     void **);
static int line_line(struct lxline_type *, unsigned,
		     const struct screen_type *);
static int line_plus(struct lxline_type *);
static int line_rail(void *, unsigned, struct x1f4_operator_type *,
		     struct x1f4_2ndscope_type *, struct x1f4_linetext_type *);
static int line_sail(struct lxline_type *, const struct screen_type *,
		     struct lxtext_type *, const struct lxtype_type *);
static int line_text(struct lxline_type *, unsigned,
		     const struct depart_type *, void *, void **);
static int link_line(struct lxline_type *, unsigned,
		     const struct screen_type *);
static int part_text(struct lxline_type *, unsigned,
		     const struct depart_type *, struct x1f4_linetext_type **,
		     void **, unsigned, const struct line_type *);
static int pick_miss(unsigned *, unsigned *, unsigned *, unsigned *, unsigned,
		     const struct screen_type *);

static void _slttx00x(u_case_args_____0);
static void bo_ltt00x(u_case_args_____0);
static void bo_slfx00(u_case_args_____0);
static void bo_sltx00(u_case_args_____0);
static void bo_sltx0x(u_case_args_____0);
static void bo_slty00(u_case_args_____0);
static void born_lt0x(u_case_args_____0);
static void born_n_lt(v_case_args_____0);
static void born_sly0(u_case_args_____0);
static void borne_lcs(u_case_args_____0);
static void borne_ltc(u_case_args_____0);
static void borne_lts(u_case_args_____0);
static void borne_ltt(u_case_args_____0);
static void borneo__l(u_case_args_____0);
static void borneo_l_(u_case_args_____0);
static void borneo_lc(u_case_args_____0);
static void borneo_lf(u_case_args_____0);
static void borneo_lt(u_case_args_____0);
static void last_amix(v_case_args_____0);
static void last_smix(v_case_args_____0);
static void line_amix(v_case_args_____0);
static void line_bfar(v_case_args_____0);
static void line_cfar(v_case_args_____0);
static void line_dfar(v_case_args_____0);
static void line_emix(v_case_args_____0);
static void line_epix(v_case_args_____0);
static void line_esix(v_case_args_____0);
static void line_evix(v_case_args_____0);
static void line_fmix(v_case_args_____0);
static void line_gmix(v_case_args_____0);
static void line_gpix(v_case_args_____0);
static void line_gsix(v_case_args_____0);
static void line_gvix(v_case_args_____0);
static void line_jmix(v_case_args_____0);
static void line_kway(v_case_args_____0);
static void line_lfar(v_case_args_____0);
static void line_nset(u_case_args_____0);
static void line_omix(v_case_args_____0);
static void line_pmix(v_case_args_____0);
static void line_qmix(v_case_args_____0);
static void line_qpix(v_case_args_____0);
static void line_qsix(v_case_args_____0);
static void line_qvix(v_case_args_____0);
static void line_rmix(v_case_args_____0);
static void line_smap(v_case_args_____0);
static void line_sset(u_case_args_____0);
static void line_vmap(u_case_args_____0);
static void line_wmap(u_case_args_____0);
static void line_xmix(v_case_args_____0);
static void line_xpix(v_case_args_____0);
static void line_xsix(v_case_args_____0);
static void line_xvix(v_case_args_____0);
static void lock_gmix(v_case_args_____0);
static void lock_lmix(v_case_args_____0);
static void lock_omix(v_case_args_____0);
static void long_rmix(v_case_args_____0);
static void long_smix(v_case_args_____0);
static void lock_umix(v_case_args_____0);
static void rail_plus(struct x1f4_linetext_type *, struct lxline_type *,
		      const struct x1f4_operator_type *);
static void themed__l(u_case_args_____0);
static void themed_ll(u_case_args_____0);
static void themed_lt(u_case_args_____0);
static void theme_ltt(u_case_args_____0);

static const char fr_addf[] = "r_add", fr_bindf[] = "r_bind", fr_casef[] =
    "r_case", fr_dockf[] = "r_dock", fr_fixf[] = "r_fix", fr_getf[] = "r_get",
    fr_linkf[] = "r_link", fr_pickf[] = "r_pick", fr_putf[] = "r_put",
    fr_queryf[] = "r_query", fr_replacef[] = "r_replace", fr_setf[] = "r_set",
    fr_sizef[] = "r_size", fr_yankf[] = "r_yank", frc_queryf[] = "rc_query",
    fro_casef[] = "ro_case", fro_queryf[] = "ro_query", frs_greaterf[] =
    "rs_greater", frs_lessf[] =
    "rs_less", frs_lowerf[] = "rs_lower", frs_upperf[] = "rs_upper",
    *const side_f[] = {
/* *INDENT-OFF* */
    fr_setf,
    fr_queryf,
    frc_queryf,
    fro_queryf,
    fr_queryf
/* *INDENT-ON* */
}, *const side_l[] = {
/* *INDENT-OFF* */
    fr_sizef
/* *INDENT-ON* */
};
static int (*const rate_0vix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_a_break_b,
    _libx1f4i0_lxline_a_break_m,
    _libx1f4i0_lxline_a_break_r
/* *INDENT-ON* */
}, (*const rate_amix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_a_class_b,
    _libx1f4i0_lxline_a_class_m,
    _libx1f4i0_lxline_a_class_r,
    _libx1f4i0_lxline_a_class_t,
    _libx1f4i0_lxline_a_class_u,
    _libx1f4i0_lxline_a_class_v
/* *INDENT-ON* */
}, (*const rate_emix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_e_class_b,
    _libx1f4i0_lxline_e_class_m,
    _libx1f4i0_lxline_e_class_r,
    _libx1f4i0_lxline_e_class_t,
    _libx1f4i0_lxline_e_class_u,
    _libx1f4i0_lxline_e_class_v
/* *INDENT-ON* */
}, (*const rate_epix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_e0class_b,
    _libx1f4i0_lxline_e0class_m,
    _libx1f4i0_lxline_e0class_r,
    _libx1f4i0_lxline_e0class_t,
    _libx1f4i0_lxline_ebclass_u,
    _libx1f4i0_lxline_ebclass_v
/* *INDENT-ON* */
}, (*const rate_esix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_e0class_b,
    _libx1f4i0_lxline_e0class_m,
    _libx1f4i0_lxline_e0class_r,
    _libx1f4i0_lxline_e0class_t,
    _libx1f4i0_lxline_efclass_u,
    _libx1f4i0_lxline_efclass_v
/* *INDENT-ON* */
}, (*const rate_evix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_e_press_b,
    _libx1f4i0_lxline_e_press_m,
    _libx1f4i0_lxline_e_press_r,
    _libx1f4i0_lxline_e_press_t,
    _libx1f4i0_lxline_e_class_u,
    _libx1f4i0_lxline_e_press_v
/* *INDENT-ON* */
}, (*const rate_fmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_f_class_b,
    _libx1f4i0_lxline_f_class_m,
    _libx1f4i0_lxline_f_class_r,
    _libx1f4i0_lxline_f_class_t,
    _libx1f4i0_lxline_f_class_u,
    _libx1f4i0_lxline_f_class_v
/* *INDENT-ON* */
}, (*const rate_gmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_g_class_b,
    _libx1f4i0_lxline_g_class_m,
    _libx1f4i0_lxline_g_class_r,
    _libx1f4i0_lxline_g_class_t,
    _libx1f4i0_lxline_g_class_u,
    _libx1f4i0_lxline_g_class_v
/* *INDENT-ON* */
}, (*const rate_gpix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_g0class_b,
    _libx1f4i0_lxline_g0class_m,
    _libx1f4i0_lxline_g0class_r,
    _libx1f4i0_lxline_g0class_t,
    _libx1f4i0_lxline_gbclass_u,
    _libx1f4i0_lxline_gbclass_v
/* *INDENT-ON* */
}, (*const rate_gsix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_g0class_b,
    _libx1f4i0_lxline_g0class_m,
    _libx1f4i0_lxline_g0class_r,
    _libx1f4i0_lxline_g0class_t,
    _libx1f4i0_lxline_gfclass_u,
    _libx1f4i0_lxline_gfclass_v
/* *INDENT-ON* */
}, (*const rate_gvix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_i_class_b,
    _libx1f4i0_lxline_i_class_m,
    _libx1f4i0_lxline_i_class_r,
    _libx1f4i0_lxline_i_class_t,
    _libx1f4i0_lxline_g_class_u,
    _libx1f4i0_lxline_i_class_v
/* *INDENT-ON* */
}, (*const rate_jmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_j_class_b,
    _libx1f4i0_lxline_j_class_m,
    _libx1f4i0_lxline_j_class_r,
    _libx1f4i0_lxline_j_class_t,
    _libx1f4i0_lxline_j_class_u,
    _libx1f4i0_lxline_j_class_v
/* *INDENT-ON* */
}, (*const rate_omix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_o_class_b,
    _libx1f4i0_lxline_o_class_m,
    _libx1f4i0_lxline_o_class_r,
    _libx1f4i0_lxline_o_class_t,
    _libx1f4i0_lxline_o_class_u,
    _libx1f4i0_lxline_o_class_v
/* *INDENT-ON* */
}, (*const rate_pmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_p_class_b,
    _libx1f4i0_lxline_p_class_m,
    _libx1f4i0_lxline_p_class_r,
    _libx1f4i0_lxline_p_class_t,
    _libx1f4i0_lxline_p_class_u,
    _libx1f4i0_lxline_p_class_v
/* *INDENT-ON* */
}, (*const rate_qmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_q_class_b,
    _libx1f4i0_lxline_q_class_m,
    _libx1f4i0_lxline_q_class_r,
    _libx1f4i0_lxline_q_class_t,
    _libx1f4i0_lxline_q_class_u,
    _libx1f4i0_lxline_q_class_v
/* *INDENT-ON* */
}, (*const rate_qpix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_q0class_b,
    _libx1f4i0_lxline_q0class_m,
    _libx1f4i0_lxline_q0class_r,
    _libx1f4i0_lxline_q0class_t,
    _libx1f4i0_lxline_qbclass_u,
    _libx1f4i0_lxline_qbclass_v
/* *INDENT-ON* */
}, (*const rate_qsix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_q0class_b,
    _libx1f4i0_lxline_q0class_m,
    _libx1f4i0_lxline_q0class_r,
    _libx1f4i0_lxline_q0class_t,
    _libx1f4i0_lxline_qfclass_u,
    _libx1f4i0_lxline_qfclass_v
/* *INDENT-ON* */
}, (*const rate_qvix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_y_class_b,
    _libx1f4i0_lxline_y_class_m,
    _libx1f4i0_lxline_y_class_r,
    _libx1f4i0_lxline_y_class_t,
    _libx1f4i0_lxline_q_class_u,
    _libx1f4i0_lxline_y_class_v
/* *INDENT-ON* */
}, (*const rate_rmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_r_class_b,
    _libx1f4i0_lxline_r_class_m,
    _libx1f4i0_lxline_r_class_r,
    _libx1f4i0_lxline_r_class_t,
    _libx1f4i0_lxline_r_class_u,
    _libx1f4i0_lxline_r_class_v
/* *INDENT-ON* */
}, (*const rate_xmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_x_class_b,
    _libx1f4i0_lxline_x_class_m,
    _libx1f4i0_lxline_x_class_r,
    _libx1f4i0_lxline_x_class_t,
    _libx1f4i0_lxline_x_class_u,
    _libx1f4i0_lxline_x_class_v
/* *INDENT-ON* */
}, (*const rate_xpix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_x0class_b,
    _libx1f4i0_lxline_x0class_m,
    _libx1f4i0_lxline_x0class_r,
    _libx1f4i0_lxline_x0class_t,
    _libx1f4i0_lxline_xbclass_u,
    _libx1f4i0_lxline_xbclass_v
/* *INDENT-ON* */
}, (*const rate_xsix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_x0class_b,
    _libx1f4i0_lxline_x0class_m,
    _libx1f4i0_lxline_x0class_r,
    _libx1f4i0_lxline_x0class_t,
    _libx1f4i0_lxline_xfclass_u,
    _libx1f4i0_lxline_xfclass_v
/* *INDENT-ON* */
}, (*const rate_xvix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_x_press_b,
    _libx1f4i0_lxline_x_press_m,
    _libx1f4i0_lxline_x_press_r,
    _libx1f4i0_lxline_x_press_t,
    _libx1f4i0_lxline_x_class_u,
    _libx1f4i0_lxline_x_press_v
/* *INDENT-ON* */
}, (*const turn_gmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_0_point_b,
    _libx1f4i0_lxline_0_point_m,
    _libx1f4i0_lxline_0_point_r,
    _libx1f4i0_lxline_0_point_t,
    _libx1f4i0_lxline_g_point_u,
    _libx1f4i0_lxline_g_point_v
/* *INDENT-ON* */
}, (*const turn_lmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_0_point_b,
    _libx1f4i0_lxline_0_point_m,
    _libx1f4i0_lxline_0_point_r,
    _libx1f4i0_lxline_0_point_t,
    _libx1f4i0_lxline_l_point_u,
    _libx1f4i0_lxline_l_point_v
/* *INDENT-ON* */
}, (*const turn_omix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_0_point_b,
    _libx1f4i0_lxline_0_point_m,
    _libx1f4i0_lxline_0_point_r,
    _libx1f4i0_lxline_0_point_t,
    _libx1f4i0_lxline_o_point_u,
    _libx1f4i0_lxline_o_point_v
/* *INDENT-ON* */
}, (*const turn_umix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxline_0_point_b,
    _libx1f4i0_lxline_0_point_m,
    _libx1f4i0_lxline_0_point_r,
    _libx1f4i0_lxline_0_point_t,
    _libx1f4i0_lxline_u_point_u,
    _libx1f4i0_lxline_u_point_v
/* *INDENT-ON* */
};
static const struct line_type auto_line[] = {
/* *INDENT-OFF* */
#define r_express_count			0
    {	{	"r_express",
		_libx1f4i0_lxline_e_centric,	0,
		NULL,				r_express_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,
						9		},
	borneo_l_,							},
#define r_express_reach			(0 + r_express_count)
#define r_fit_count			1
    {	{	"r_fit",
		_libx1f4i0_lxline_f_centric,	0,
		NULL,				r_fit_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,		5		},
	themed__l,							},
#define r_fit_reach			(r_express_reach + r_fit_count)
#define r_get_count			3
    {	{	fr_getf,
		_libx1f4i0_lxline_g_derived,	X1f4_E4_CASE,
		NULL,				r_get_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						5		},
	bo_slty00,							},
#define r_get_reach			(r_fit_reach + (r_get_count << 1))
#define r_jack_count			3
    {	{	"r_jack",
		_libx1f4i0_lxline_j_derived,	X1f4_E4_MODE,
		NULL,				r_jack_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						6		},
	bo_sltx00,							},
#define r_jack_reach			(r_get_reach + (r_jack_count << 1))
#define r_new_count			1
    {	{	"r_new",
		_libx1f4i0_lxline_n_forward,	X1f4_E4_VOID,
		NULL,				r_new_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		5		},
	line_nset,							},
#define r_new_reach			(r_jack_reach + (r_new_count << 1))
#define r_over_count			3
    {	{	"r_over",
		_libx1f4i0_lxline_o_derived,	X1f4_E4_MODE,
		NULL,				r_over_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						6		},
	bo_sltx00,							},
#define r_over_reach			(r_new_reach + (r_over_count << 1))
#define r_render_count			0
    {	{	"r_render",
		_libx1f4i0_lxline_r_centric,	0,
		NULL,				r_render_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,
						8		},
	borneo_l_,							},
#define r_render_reach			(r_over_reach + r_render_count)
#define r_set_count			2
    {	{	fr_setf,
		_libx1f4i0_lxline_s_forward,	0,
		NULL,				r_set_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		5		},
	line_sset,							},
#define r_set_reach			(r_render_reach + (r_set_count << 1))
#define r_tie_count			1
    {	{	"r_tie",
		_libx1f4i0_lxline_t_centric,	0,
		NULL,				r_tie_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,
						5		},
	themed__l,							},
#define r_tie_reach			(r_set_reach + r_tie_count)
#define r_ucall_count			3
    {	{	"r_ucall",
		_libx1f4i0_lxline_u_pending,	X1f4_E4_VOID,
		NULL,				r_ucall_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_LINK_PASS
		| X1f4_E4_SIDE_LIST | X1f4_E4_TEXT_LINK
		| X1f4_E4_WALK_LINK,
						7		},
	line_vmap,							},
#define r_ucall_reach			(r_tie_reach + r_ucall_count)
#define r_vcall_count			3
    {	{	"r_vcall",
		_libx1f4i0_lxline_v_pending,	X1f4_E4_VOID,
		NULL,				r_vcall_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_LINK_PASS
		| X1f4_E4_SIDE_LIST | X1f4_E4_TEXT_LINK
		| X1f4_E4_WALK_LINK,
						7		},
	line_vmap,							},
#define r_vcall_reach			(r_ucall_reach + r_vcall_count)
#define r_wcall_count			4
    {	{	"r_wcall",
		_libx1f4i0_lxline_w_pending,	X1f4_E4_VOID,
		NULL,				r_wcall_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_LINK_PASS
		| X1f4_E4_SIDE_LIST | X1f4_E4_TEXT_LINK
		| X1f4_E4_WALK_LINK,
						7		},
	line_wmap,							},
#define r_wcall_reach			(r_vcall_reach + r_wcall_count)
#define r_yank_count			3
    {	{	fr_yankf,
		_libx1f4i0_lxline_y_derived,	X1f4_E4_CASE,
		NULL,				r_yank_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						6		},
	bo_slty00,							},
#define r_yank_reach			(r_wcall_reach + (r_yank_count << 1))
#define record_count			0
    {	{	"record",
		_libx1f4i0_lxline_e_centric,	0,
		NULL,				record_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,
						6		},
	borneo_l_,							}
#define record_reach			(r_yank_reach + record_count)
/* *INDENT-ON* */
#define auto_line_count			record_reach
}, beta_line[] = {
/* *INDENT-OFF* */
#define rc_key_count			2
    {	{	"rc_key",
		_libx1f4i0_lxline_k_forward,	X1f4_E4_MODE,
		NULL,				rc_key_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo_lf,							},
#define rc_key_reach			(0 + rc_key_count)
/* *INDENT-ON* */
#define beta_line_count			rc_key_reach
}, blue_line[] = {
/* *INDENT-OFF* */
#define rs_greater_count		4
    {	{	frs_greaterf,
		_libx1f4i0_lxline_0_centric,	X1f4_E4_MODE,
		NULL,				rs_greater_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						10		},
	_slttx00x,							},
#define rs_greater_reach		(0 + (rs_greater_count << 1))
#define rsd_greater_count		3
    {	{	"rsd_greater",
		_libx1f4i0_lxline_1_centric,	X1f4_E4_MODE,
		NULL,				rsd_greater_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						11		},
	bo_sltx00,							},
#define rsd_greater_reach \
    (rs_greater_reach + (rsd_greater_count << 1))
#define rs_upper_count			4
    {	{	frs_upperf,
		_libx1f4i0_lxline_0_centric,	X1f4_E4_MODE,
		NULL,				rs_upper_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						8		},
	_slttx00x,							},
#define rs_upper_reach \
    (rsd_greater_reach + (rs_upper_count << 1))
#define rsd_upper_count			3
    {	{	"rsd_upper",
		_libx1f4i0_lxline_1_centric,	X1f4_E4_MODE,
		NULL,				rsd_upper_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						9		},
	bo_sltx00,							},
#define rsd_upper_reach \
    (rs_upper_reach + (rsd_upper_count << 1))
#define rs_lower_count			4
    {	{	frs_lowerf,
		_libx1f4i0_lxline_0_centric,	X1f4_E4_MODE,
		NULL,				rs_lower_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						8		},
	_slttx00x,							},
#define rs_lower_reach \
    (rsd_upper_reach + (rs_lower_count << 1))
#define rsd_lower_count			3
    {	{	"rsd_lower",
		_libx1f4i0_lxline_1_centric,	X1f4_E4_MODE,
		NULL,				rsd_lower_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						9		},
	bo_sltx00,							},
#define rsd_lower_reach \
    (rs_lower_reach + (rsd_lower_count << 1))
#define rs_less_count			4
    {	{	frs_lessf,
		_libx1f4i0_lxline_0_centric,	X1f4_E4_MODE,
		NULL,				rs_less_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						7		},
	_slttx00x,							},
#define rs_less_reach \
    (rsd_lower_reach + (rs_less_count << 1))
#define rsd_less_count			3
    {	{	"rsd_less",
		_libx1f4i0_lxline_1_centric,	X1f4_E4_MODE,
		NULL,				rsd_less_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						8		},
	bo_sltx00,							},
#define rsd_less_reach \
    (rs_less_reach + (rsd_less_count << 1))
/* *INDENT-ON* */
#define blue_line_count			rsd_less_reach
}, case_line[] = {
/* *INDENT-OFF* */
#define rcsik_greater_count		2
    {	{	"rcsik_greater",
		_libx1f4i0_lxline_0ik_newer,	X1f4_E4_MODE,
		NULL,				rcsik_greater_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						13		},
	borneo_lf,							},
#define rcsik_greater_reach		(0 + rcsik_greater_count)
#define rcsik_upper_count		2
    {	{	"rcsik_upper",
		_libx1f4i0_lxline_0ik_newer,	X1f4_E4_MODE,
		NULL,				rcsik_upper_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						11		},
	borneo_lf,							},
#define rcsik_upper_reach \
    (rcsik_greater_reach + rcsik_upper_count)
#define rcsik_lower_count		2
    {	{	"rcsik_lower",
		_libx1f4i0_lxline_0ik_newer,	X1f4_E4_MODE,
		NULL,				rcsik_lower_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						11		},
	borneo_lf,							},
#define rcsik_lower_reach \
    (rcsik_upper_reach + rcsik_lower_count)
#define rcsik_less_count		2
    {	{	"rcsik_less",
		_libx1f4i0_lxline_0ik_newer,	X1f4_E4_MODE,
		NULL,				rcsik_less_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						10		},
	borneo_lf,							}
#define rcsik_less_reach \
    (rcsik_lower_reach + rcsik_less_count)
/* *INDENT-ON* */
#define case_line_count			rcsik_less_reach
}, deep_line[] = {
/* *INDENT-OFF* */
#define rc_last_count			2
    {	{	"rc_last",
		_libx1f4i0_lxline_k_keeping,	X1f4_E4_MODE,
		NULL,				rc_last_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo_lf,							}
#define rc_last_reach			(0 + rc_last_count)
/* *INDENT-ON* */
#define deep_line_count			rc_last_reach
}, fade_line[] = {
/* *INDENT-OFF* */
#define rcsd_greater_count			3
    {	{	"rcsd_greater",
		_libx1f4i0_lxline_0d_notion,	X1f4_E4_MODE,
		NULL,				rcsd_greater_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						12		},
	bo_slfx00,							},
#define rcsd_greater_reach		(0 + (rcsd_greater_count << 1))
#define rcsi_greater_count		3
    {	{	"rcsi_greater",
		_libx1f4i0_lxline_0i_notion,	X1f4_E4_MODE,
		NULL,				rcsi_greater_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						12		},
	bo_slfx00,							},
#define rcsi_greater_reach \
    (rcsd_greater_reach + (rcsi_greater_count << 1))
#define rcsd_upper_count			3
    {	{	"rcsd_upper",
		_libx1f4i0_lxline_0d_notion,	X1f4_E4_MODE,
		NULL,				rcsd_upper_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						10		},
	bo_slfx00,							},
#define rcsd_upper_reach \
    (rcsi_greater_reach + (rcsd_upper_count << 1))
#define rcsi_upper_count			3
    {	{	"rcsi_upper",
		_libx1f4i0_lxline_0i_notion,	X1f4_E4_MODE,
		NULL,				rcsi_upper_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						10		},
	bo_slfx00,							},
#define rcsi_upper_reach \
    (rcsd_upper_reach + (rcsi_upper_count << 1))
#define rcsd_lower_count			3
    {	{	"rcsd_lower",
		_libx1f4i0_lxline_0d_notion,	X1f4_E4_MODE,
		NULL,				rcsd_lower_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						10		},
	bo_slfx00,							},
#define rcsd_lower_reach \
    (rcsi_upper_reach + (rcsd_lower_count << 1))
#define rcsi_lower_count			3
    {	{	"rcsi_lower",
		_libx1f4i0_lxline_0i_notion,	X1f4_E4_MODE,
		NULL,				rcsi_lower_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						10		},
	bo_slfx00,							},
#define rcsi_lower_reach \
    (rcsd_lower_reach + (rcsi_lower_count << 1))
#define rcsd_less_count			3
    {	{	"rcsd_less",
		_libx1f4i0_lxline_0d_notion,	X1f4_E4_MODE,
		NULL,				rcsd_less_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						9		},
	bo_slfx00,							},
#define rcsd_less_reach \
    (rcsi_lower_reach + (rcsd_less_count << 1))
#define rcsi_less_count			3
    {	{	"rcsi_less",
		_libx1f4i0_lxline_0i_notion,	X1f4_E4_MODE,
		NULL,				rcsi_less_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						9		},
	bo_slfx00,							},
#define rcsi_less_reach \
    (rcsd_less_reach + (rcsi_less_count << 1))
/* *INDENT-ON* */
#define fade_line_count			rcsi_less_reach
}, fail_line[] = {
/* *INDENT-OFF* */
#define r_add_count			3
    {	{	fr_addf,
		_libx1f4i0_lxline_a_forward,	X1f4_E4_VOID,
		NULL,				r_add_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SLIP_LIST
		| X1f4_E4_TEXT_LINK,		5		},
	borne_lts,							},
#define r_add_reach			(0 + r_add_count)
#define r_bind_count			3
    {	{	fr_bindf,
		_libx1f4i0_lxline_b_derived,	X1f4_E4_VOID,
		NULL,				r_bind_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SLIP_LIST
		| X1f4_E4_TEXT_LINK,		6		},
	borne_lts,							},
#define r_bind_reach			(r_add_reach + r_bind_count)
#define r_case_count			3
    {	{	fr_casef,
		_libx1f4i0_lxline_c_derived,	X1f4_E4_CASE,
		NULL,				r_case_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SLIP_LIST
		| X1f4_E4_TEXT_LINK,		6		},
	borne_lts,							},
#define r_case_reach			(r_bind_reach + r_case_count)
#define r_dock_count			3
    {	{	fr_dockf,
		_libx1f4i0_lxline_d_derived,	X1f4_E4_VOID,
		NULL,				r_dock_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SLIP_LIST
		| X1f4_E4_TEXT_LINK,		6		},
	borne_lts,							},
#define r_dock_reach			(r_case_reach + r_dock_count)
#define r_fix_count			3
    {	{	fr_fixf,
		_libx1f4i0_lxline_f_derived,	X1f4_E4_VOID,
		NULL,				r_fix_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SLIP_LIST
		| X1f4_E4_TEXT_LINK,		5		},
	borne_lts,							},
#define r_fix_reach			(r_dock_reach + r_fix_count)
#define r_gauge_count			3
    {	{	"r_gauge",
		_libx1f4i0_lxline_g_forward,	X1f4_E4_CASE,
		NULL,				r_gauge_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borne_ltc,							},
#define r_gauge_reach			(r_fix_reach + r_gauge_count)
#define r_link_count			3
    {	{	fr_linkf,
		_libx1f4i0_lxline_l_derived,	X1f4_E4_VOID,
		NULL,				r_link_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SLIP_LIST
		| X1f4_E4_TEXT_LINK,		6		},
	borne_lts,							},
#define r_link_reach			(r_gauge_reach + r_link_count)
#define r_net_count			2
    {	{	"r_net",
		_libx1f4i0_lxline_n_pending,	X1f4_E4_TEXT,
		NULL,				r_net_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	borneo_lt,							},
#define r_net_reach			(r_link_reach + r_net_count)
#define r_pick_count			2
    {	{	fr_pickf,
		_libx1f4i0_lxline_p_derived,	X1f4_E4_CASE,
		NULL,				r_pick_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo_lt,							},
#define r_pick_reach			(r_net_reach + r_pick_count)
#define r_put_count			3
    {	{	fr_putf,
		_libx1f4i0_lxline_p_forward,	X1f4_E4_VOID,
		NULL,				r_put_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SLIP_LIST
		| X1f4_E4_TEXT_LINK,		5		},
	borne_lts,							},
#define r_put_reach			(r_pick_reach + r_put_count)
#define r_query_count			2
    {	{	fr_queryf,
		_libx1f4i0_lxline_q_forward,	X1f4_E4_CASE,
		NULL,				r_query_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo_lt,							},
#define r_query_reach			(r_put_reach + r_query_count)
#define r_replace_count			3
    {	{	fr_replacef,
		_libx1f4i0_lxline_r_derived,	X1f4_E4_VOID,
		NULL,				r_replace_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SLIP_LIST
		| X1f4_E4_TEXT_LINK,		9		},
	borne_lts,							},
#define r_replace_reach			(r_query_reach + r_replace_count)
#define ro_case_count			3
    {	{	fro_casef,
		_libx1f4i0_lxline_c_forward,	X1f4_E4_CASE,
		NULL,				ro_case_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SLIP_LIST
		| X1f4_E4_TEXT_LINK,		6		},
	borne_lcs,							},
#define ro_case_reach			(r_replace_reach + ro_case_count)
#define ro_query_count			2
    {	{	fro_queryf,
		_libx1f4i0_lxline_q_pending,	X1f4_E4_CASE,
		NULL,				ro_query_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	borneo_lc,							}
#define ro_query_reach			(ro_case_reach + ro_query_count)
/* *INDENT-ON* */
#define fail_line_count			ro_query_reach
}, fine_line[] = {
/* *INDENT-OFF* */
#define rc_jack_count			3
    {	{	"rc_jack",
		_libx1f4i0_lxline_j_forward,	X1f4_E4_MODE,
		NULL,				rc_jack_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						7		},
	bo_slfx00,							},
#define rc_jack_reach			(0 + (rc_jack_count << 1))
#define rc_query_count			2
    {	{	frc_queryf,
		_libx1f4i0_lxline_q_derived,	X1f4_E4_CASE,
		NULL,				rc_query_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	borneo_lf,							}
#define rc_query_reach			(rc_jack_reach + rc_query_count)
/* *INDENT-ON* */
#define fine_line_count			rc_query_reach
}, fold_line[] = {
/* *INDENT-OFF* */
#define rc_first_count			2
    {	{	"rc_first",
		_libx1f4i0_lxline_k_keeping,	X1f4_E4_MODE,
		NULL,				rc_first_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	borneo_lf,							},
#define rc_first_reach			(0 + rc_first_count)
/* *INDENT-ON* */
#define fold_line_count			rc_first_reach
}, gray_line[] = {
/* *INDENT-OFF* */
#define r_last_count			2
    {	{	"r_last",
		_libx1f4i0_lxline_h_keeping,	X1f4_E4_MODE,
		NULL,				r_last_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		6		},
	born_lt0x,							},
#define r_last_reach			(0 + (r_last_count << 1))
#define r_short_count			1
    {	{	"r_short",
		_libx1f4i0_lxline_a_keeping,	0,
		NULL,				r_short_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	themed__l,							},
#define r_short_reach			(r_last_reach + r_short_count)
#define rb_delete_count			1
    {	{	"rb_delete",
		_libx1f4i0_lxline_a_keeping,	0,
		NULL,				rb_delete_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						9		},
	themed__l,							},
#define rb_delete_reach			(r_short_reach + rb_delete_count)
/* *INDENT-ON* */
#define gray_line_count			rb_delete_reach
}, head_line[] = {
/* *INDENT-OFF* */
#define r_front_count			1
    {	{	"r_front",
		_libx1f4i0_lxline_c_keeping,	X1f4_E4_CASE,
		NULL,				r_front_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo__l,							},
#define r_front_reach			(0 + r_front_count)
#define r_low_count			1
    {	{	"r_low",
		_libx1f4i0_lxline_g_keeping,	X1f4_E4_TEXT,
		NULL,				r_low_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	borneo__l,							},
#define r_low_reach			(r_front_reach + r_low_count)
#define rf_get_count			2
    {	{	"rf_get",
		_libx1f4i0_lxline_j_keeping,	X1f4_E4_CASE,
		NULL,				rf_get_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						6		},
	born_sly0,							},
#define rf_get_reach			(r_low_reach + (rf_get_count << 1))
#define rf_pick_count			1
    {	{	"rf_pick",
		_libx1f4i0_lxline_e_keeping,	X1f4_E4_CASE,
		NULL,				rf_pick_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo__l,							},
#define rf_pick_reach			(rf_get_reach + rf_pick_count)
#define rf_query_count			1
    {	{	"rf_query",
		_libx1f4i0_lxline_c_keeping,	X1f4_E4_CASE,
		NULL,				rf_query_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	borneo__l,							},
#define rf_query_reach			(rf_pick_reach + rf_query_count)
#define rf_yank_count			2
    {	{	"rf_yank",
		_libx1f4i0_lxline_i_keeping,	X1f4_E4_CASE,
		NULL,				rf_yank_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						7		},
	born_sly0,							},
#define rf_yank_reach			(rf_query_reach + (rf_yank_count << 1))
#define rs_first_count			3
    {	{	"rs_first",
		_libx1f4i0_lxline_f_keeping,	X1f4_E4_MODE,
		NULL,				rs_first_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						8		},
	bo_sltx0x,							}
#define rs_first_reach			(rf_yank_reach + (rs_first_count << 1))
/* *INDENT-ON* */
#define head_line_count			rs_first_reach
}, hold_line[] = {
/* *INDENT-OFF* */
#define rcs_first_count			3
    {	{	"rcs_first",
		_libx1f4i0_lxline_l_keeping,	X1f4_E4_MODE,
		NULL,				rcs_first_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						9		},
	bo_slfx00,							}
#define rcs_first_reach			(0 + (rcs_first_count << 1))
/* *INDENT-ON* */
#define hold_line_count			rcs_first_reach
}, keep_line[] = {
/* *INDENT-OFF* */
#define rcs_last_count			3
    {	{	"rcs_last",
		_libx1f4i0_lxline_l_keeping,	X1f4_E4_MODE,
		NULL,				rcs_last_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						8		},
	bo_slfx00,							}
#define rcs_last_reach			(0 + (rcs_last_count << 1))
/* *INDENT-ON* */
#define keep_line_count			rcs_last_reach
}, slip_line[] = {
/* *INDENT-OFF* */
#define r_cast_count			2
    {	{	"r_cast",
		_libx1f4i0_lxline_c_nearest,	0,
		NULL,				r_cast_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed_ll,							},
#define r_cast_reach			(0 + r_cast_count)
#define r_clear_count			1
    {	{	"r_clear",
		_libx1f4i0_lxline_c_pending,	0,
		NULL,				r_clear_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	themed__l,							},
#define r_clear_reach			(r_cast_reach + r_clear_count)
#define r_copy_count			2
    {	{	"r_copy",
		_libx1f4i0_lxline_c_logique,	0,
		NULL,				r_copy_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed_ll,							},
#define r_copy_reach			(r_clear_reach + r_copy_count)
#define r_delete_count			2
    {	{	"r_delete",
		_libx1f4i0_lxline_d_logique,	0,
		NULL,				r_delete_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	themed_lt,							},
#define r_delete_reach			(r_copy_reach + r_delete_count)
#define r_down_count			2
    {	{	"r_down",
		_libx1f4i0_lxline_d_forward,	X1f4_E4_VOID,
		NULL,				r_down_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo_lt,							},
#define r_down_reach			(r_delete_reach + r_down_count)
#define r_drop_count			2
    {	{	"r_drop",
		_libx1f4i0_lxline_d_pending,	0,
		NULL,				r_drop_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed_lt,							},
#define r_drop_reach			(r_down_reach + r_drop_count)
#define r_erase_count			3
    {	{	"r_erase",
		_libx1f4i0_lxline_e_pending,	0,
		NULL,				r_erase_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	theme_ltt,							},
#define r_erase_reach			(r_drop_reach + r_erase_count)
#define r_id_count			2
    {	{	"r_id",
		_libx1f4i0_lxline_i_pending,	X1f4_E4_MODE,
		NULL,				r_id_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	borneo_lt,							},
#define r_id_reach			(r_erase_reach + r_id_count)
#define r_join_count			2
    {	{	"r_join",
		_libx1f4i0_lxline_j_logique,	0,
		NULL,				r_join_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed_ll,							},
#define r_join_reach			(r_id_reach + r_join_count)
#define r_key_count			2
    {	{	"r_key",
		_libx1f4i0_lxline_k_logique,	X1f4_E4_MODE,
		NULL,				r_key_count,
		X1f4_E4_KEEP_CALL,
						5		},
	borneo_lt,							},
#define r_key_reach			(r_join_reach + r_key_count)
#define r_move_count			3
    {	{	"r_move",
		_libx1f4i0_lxline_m_logique,	X1f4_E4_VOID,
		NULL,				r_move_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borne_ltt,							},
#define r_move_reach			(r_key_reach + r_move_count)
#define r_resign_count			2
    {	{	"r_resign",
		_libx1f4i0_lxline_r_logique,	X1f4_E4_VOID,
		NULL,				r_resign_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	themed_lt,							},
#define r_resign_reach			(r_move_reach + r_resign_count)
#define r_size_count			1
    {	{	fr_sizef,
		_libx1f4i0_lxline_r_forward,	X1f4_E4_MODE,
		NULL,				r_size_count,
		0,				6		},
	borneo__l,							},
#define r_size_reach			(r_resign_reach + r_size_count)
#define r_spin_count			3
    {	{	"r_spin",
		_libx1f4i0_lxline_s_logique,	X1f4_E4_VOID,
		NULL,				r_spin_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borne_ltt,							},
#define r_spin_reach			(r_size_reach + r_spin_count)
#define r_swap_count			2
    {	{	"r_swap",
		_libx1f4i0_lxline_s_sequent,	X1f4_E4_VOID,
		NULL,				r_swap_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed_ll,							},
#define r_swap_reach			(r_spin_reach + r_swap_count)
#define r_type_count			2
    {	{	"r_type",
		_libx1f4i0_lxline_t_pending,	X1f4_E4_TEXT,
		NULL,				r_type_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo_lt,							},
#define r_type_reach			(r_swap_reach + r_type_count)
#define r_up_count			2
    {	{	"r_up",
		_libx1f4i0_lxline_u_forward,	X1f4_E4_VOID,
		NULL,				r_up_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	borneo_lt,							}
#define r_up_reach			(r_type_reach + r_up_count)
/* *INDENT-ON* */
#define slip_line_count			r_up_reach
}, tail_line[] = {
/* *INDENT-OFF* */
#define r_back_count			1
    {	{	"r_back",
		_libx1f4i0_lxline_c_keeping,	X1f4_E4_CASE,
		NULL,				r_back_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo__l,							},
#define r_back_reach			(0 + r_back_count)
#define r_high_count			1
    {	{	"r_high",
		_libx1f4i0_lxline_g_keeping,	X1f4_E4_TEXT,
		NULL,				r_high_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo__l,							},
#define r_high_reach			(r_back_reach + r_high_count)
#define rb_get_count			2
    {	{	"rb_get",
		_libx1f4i0_lxline_j_keeping,	X1f4_E4_CASE,
		NULL,				rb_get_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						6		},
	born_sly0,							},
#define rb_get_reach			(r_high_reach + (rb_get_count << 1))
#define rb_pick_count			1
    {	{	"rb_pick",
		_libx1f4i0_lxline_e_keeping,	X1f4_E4_CASE,
		NULL,				rb_pick_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo__l,							},
#define rb_pick_reach			(rb_get_reach + rb_pick_count)
#define rb_query_count			1
    {	{	"rb_query",
		_libx1f4i0_lxline_c_keeping,	X1f4_E4_CASE,
		NULL,				rb_query_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	borneo__l,							},
#define rb_query_reach			(rb_pick_reach + rb_query_count)
#define rb_yank_count			2
    {	{	"rb_yank",
		_libx1f4i0_lxline_i_keeping,	X1f4_E4_CASE,
		NULL,				rb_yank_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						7		},
	born_sly0,							},
#define rb_yank_reach			(rb_query_reach + (rb_yank_count << 1))
#define rs_last_count			3
    {	{	"rs_last",
		_libx1f4i0_lxline_f_keeping,	X1f4_E4_MODE,
		NULL,				rs_last_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						7		},
	bo_sltx0x,							}
#define rs_last_reach			(rb_yank_reach + (rs_last_count << 1))
/* *INDENT-ON* */
#define tail_line_count			rs_last_reach
}, used_line[] = {
/* *INDENT-OFF* */
#define r_class_count			1
    {	{	"r_class",
		_libx1f4i0_lxline_a_keeping,	0,
		NULL,				r_class_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	themed__l,							},
#define r_class_reach			(0 + r_class_count)
#define r_first_count			2
    {	{	"r_first",
		_libx1f4i0_lxline_h_keeping,	X1f4_E4_MODE,
		NULL,				r_first_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		7		},
	born_lt0x,							},
#define r_first_reach			(r_class_reach + (r_first_count << 1))
#define rf_delete_count			1
    {	{	"rf_delete",
		_libx1f4i0_lxline_a_keeping,	0,
		NULL,				rf_delete_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						9		},
	themed__l,							},
#define rf_delete_reach			(r_first_reach + rf_delete_count)
/* *INDENT-ON* */
#define used_line_count			rf_delete_reach
}, wrap_line[] = {
/* *INDENT-OFF* */
#define rsk_greater_count			3
    {	{	"rsk_greater",
		_libx1f4i0_lxline_0k_queued,	X1f4_E4_MODE,
		NULL,				rsk_greater_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		11		},
	bo_ltt00x,							},
#define rsk_greater_reach			(0 + (rsk_greater_count << 1))
#define rsk_upper_count			3
    {	{	"rsk_upper",
		_libx1f4i0_lxline_0k_queued,	X1f4_E4_MODE,
		NULL,				rsk_upper_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		9		},
	bo_ltt00x,							},
#define rsk_upper_reach \
    (rsk_greater_reach + (rsk_upper_count << 1))
#define rsk_lower_count			3
    {	{	"rsk_lower",
		_libx1f4i0_lxline_0k_queued,	X1f4_E4_MODE,
		NULL,				rsk_lower_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		9		},
	bo_ltt00x,							},
#define rsk_lower_reach \
    (rsk_upper_reach + (rsk_lower_count << 1))
#define rsk_less_count			3
    {	{	"rsk_less",
		_libx1f4i0_lxline_0k_queued,	X1f4_E4_MODE,
		NULL,				rsk_less_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		8		},
	bo_ltt00x,							}
#define rsk_less_reach \
    (rsk_lower_reach + (rsk_less_count << 1))
/* *INDENT-ON* */
#define wrap_line_count			rsk_less_reach
};
static const struct orient_type cast_line[] = {
/* *INDENT-OFF* */
#define r_q__class			4
#define r_q__count			2
    {	{	"r_q_",
		NULL,				0,
		NULL,				r_q__count,
		X1f4_E4_TEXT_LINK,		r_q__class	},
	line_qmix,							},
#define r_q__reach			(0 + r_q__count)
#define r_q__trans			(0 + r_q__class)
#define r_qc__class			5
#define r_qc__count			2
    {	{	"r_qc_",
		NULL,				0,
		NULL,				r_qc__count,
		X1f4_E4_TEXT_LINK,		r_qc__class	},
	line_qvix,							},
#define r_qc__reach			(r_q__reach + r_qc__count)
#define r_qc__trans			(r_q__trans + r_qc__class)
#define rb_q__class			5
#define rb_q__count			1
    {	{	"rb_q_",
		NULL,				0,
		NULL,				rb_q__count,
		X1f4_E4_TEXT_LINK,		rb_q__class	},
	line_qpix,							},
#define rb_q__reach			(r_qc__reach + rb_q__count)
#define rb_q__trans			(r_qc__trans + rb_q__class)
#define rf_q__class			5
#define rf_q__count			1
    {	{	"rf_q_",
		NULL,				0,
		NULL,				rf_q__count,
		X1f4_E4_TEXT_LINK,		rf_q__class	},
	line_qsix,							}
#define rf_q__reach			(rb_q__reach + rf_q__count)
#define rf_q__trans			(rb_q__trans + rf_q__class)
/* *INDENT-ON* */
#define cast_line_class			rf_q__trans
#define cast_line_count			rf_q__reach
#define cast_line_shift \
    sizeof(cast_line) / sizeof(struct orient_type)
}, copy_line[] = {
/* *INDENT-OFF* */
#define r_a__class			4
#define r_a__count			3
    {	{	"r_a_",
		NULL,				X1f4_E4_VOID,
		NULL,				r_a__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						r_a__class	},
	line_amix,							},
#define r_a__reach			(0 + r_a__count)
#define r_a__trans			(0 + r_a__class)
#define r_f__class			4
#define r_f__count			3
    {	{	"r_f_",
		NULL,				0,
		NULL,				r_f__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						r_f__class	},
	line_fmix,							},
#define r_f__reach			(r_a__reach + r_f__count)
#define r_f__trans			(r_a__trans + r_f__class)
#define r_p__class			4
#define r_p__count			3
    {	{	"r_p_",
		NULL,				0,
		NULL,				r_p__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						r_p__class	},
	line_pmix,							},
#define r_p__reach			(r_f__reach + r_p__count)
#define r_p__trans			(r_f__trans + r_p__class)
#define r_r__class			4
#define r_r__count			3
    {	{	"r_r_",
		NULL,				0,
		NULL,				r_r__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						r_r__class	},
	line_rmix,							}
#define r_r__reach			(r_p__reach + r_r__count)
#define r_r__trans			(r_p__trans + r_r__class)
/* *INDENT-ON* */
#define copy_line_class			r_r__trans
#define copy_line_count			r_r__reach
#define copy_line_shift \
    sizeof(copy_line) / sizeof(struct orient_type)
}, fast_line[] = {
/* *INDENT-OFF* */
#define r_e__class			4
#define r_e__count			3
    {	{	"r_e_",
		NULL,				X1f4_E4_VOID,
		NULL,				r_e__count,
		0,				r_e__class	},
	line_emix,							},
#define r_e__reach			(0 + r_e__count)
#define r_e__trans			(0 + r_e__class)
#define r_ec__class			5
#define r_ec__count			3
    {	{	"r_ec_",
		NULL,				0,
		NULL,				r_ec__count,
		X1f4_E4_TEXT_LINK,		r_ec__class	},
	line_evix,							},
#define r_ec__reach			(r_e__reach + r_ec__count)
#define r_ec__trans			(r_e__trans + r_ec__class)
#define r_g__class			4
#define r_g__count			3
    {	{	"r_g_",
		NULL,				0,
		NULL,				r_g__count,
		0,				r_g__class	},
	line_gmix,							},
#define r_g__reach			(r_ec__reach + r_g__count)
#define r_g__trans			(r_ec__trans + r_g__class)
#define r_gc__class			5
#define r_gc__count			3
    {	{	"r_gc_",
		NULL,				0,
		NULL,				r_gc__count,
		X1f4_E4_TEXT_LINK,		r_gc__class	},
	line_gvix,							},
#define r_gc__reach			(r_g__reach + r_gc__count)
#define r_gc__trans			(r_g__trans + r_gc__class)
#define r_j__class			4
#define r_j__count			3
    {	{	"r_j_",
		NULL,				X1f4_E4_MODE,
		NULL,				r_j__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		4		},
	line_jmix,							},
#define r_j__reach			(r_gc__reach + r_j__count)
#define r_j__trans			(r_gc__trans + r_j__class)
#define r_o__class			4
#define r_o__count			3
    {	{	"r_o_",
		NULL,				0,
		NULL,				r_o__count,
		0,				r_o__class	},
	line_omix,							},
#define r_o__reach			(r_j__reach + r_o__count)
#define r_o__trans			(r_j__trans + r_o__class)
#define r_s__class			4
#define r_s__count			2
    {	{	"r_s_",
		_libx1f4i0_lxline_s_pending,	X1f4_E4_MODE,
		NULL,				r_s__count,
		X1f4_E4_TEXT_LINK,		r_s__class	},
	line_smap,							},
#define r_s__reach			(r_o__reach + r_s__count)
#define r_s__trans			(r_o__trans + r_s__class)
#define rb_e__class			5
#define rb_e__count			2
    {	{	"rb_e_",
		NULL,				X1f4_E4_VOID,
		NULL,				rb_e__count,
		0,				rb_e__class	},
	line_epix,							},
#define rb_e__reach			(r_s__reach + rb_e__count)
#define rb_e__trans			(r_s__trans + rb_e__class)
#define rb_g__class			5
#define rb_g__count			2
    {	{	"rb_g_",
		NULL,				0,
		NULL,				rb_g__count,
		0,				rb_g__class	},
	line_gpix,							},
#define rb_g__reach			(rb_e__reach + rb_g__count)
#define rb_g__trans			(rb_e__trans + rb_g__class)
#define rf_e__class			5
#define rf_e__count			2
    {	{	"rf_e_",
		NULL,				X1f4_E4_VOID,
		NULL,				rf_e__count,
		0,				rf_e__class	},
	line_esix,							},
#define rf_e__reach			(rb_g__reach + rf_e__count)
#define rf_e__trans			(rb_g__trans + rf_e__class)
#define rf_g__class			5
#define rf_g__count			2
    {	{	"rf_g_",
		NULL,				0,
		NULL,				rf_g__count,
		0,				rf_g__class	},
	line_gsix,							},
#define rf_g__reach			(rf_e__reach + rf_g__count)
#define rf_g__trans			(rf_e__trans + rf_g__class)
#define rs_g__class			5
#define rs_g__count			4
    {	{	"rs_g_",
		NULL,				X1f4_E4_MODE,
		NULL,				rs_g__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		rs_g__class	},
	lock_gmix,							},
#define rs_g__reach			(rf_g__reach + (rs_g__count << 1))
#define rs_g__trans			(rf_g__trans + rs_g__class)
#define rs_l__class			5
#define rs_l__count			4
    {	{	"rs_l_",
		NULL,				X1f4_E4_MODE,
		NULL,				rs_l__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		rs_l__class	},
	lock_lmix,							},
#define rs_l__reach			(rs_g__reach + (rs_l__count << 1))
#define rs_l__trans			(rs_g__trans + rs_l__class)
#define rs_o__class			5
#define rs_o__count			4
    {	{	"rs_o_",
		NULL,				X1f4_E4_MODE,
		NULL,				rs_o__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		rs_o__class	},
	lock_omix,							},
#define rs_o__reach			(rs_l__reach + (rs_o__count << 1))
#define rs_o__trans			(rs_l__trans + rs_o__class)
#define rs_u__class			5
#define rs_u__count			4
    {	{	"rs_u_",
		NULL,				X1f4_E4_MODE,
		NULL,				rs_u__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		rs_u__class	},
	lock_umix,							}
#define rs_u__reach			(rs_o__reach + (rs_u__count << 1))
#define rs_u__trans			(rs_o__trans + rs_u__class)
/* *INDENT-ON* */
#define fast_line_class			rs_u__trans
#define fast_line_count			rs_u__reach
#define fast_line_shift \
    sizeof(fast_line) / sizeof(struct orient_type)
}, long_line[] = {
/* *INDENT-OFF* */
#define r_b__count			3
    {	{	"r_b_",
		_libx1f4i0_lxline_b_sequent,	0,
		NULL,				r_b__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	line_bfar,							},
#define r_b__reach			(0 + r_b__count)
#define r_c__count			3
    {	{	"r_c_",
		_libx1f4i0_lxline_c_sequent,	0,
		NULL,				r_c__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	line_cfar,							},
#define r_c__reach			(r_b__reach + r_c__count)
#define r_d__count			3
    {	{	"r_d_",
		_libx1f4i0_lxline_d_sequent,	X1f4_E4_VOID,
		NULL,				r_d__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	line_dfar,							},
#define r_d__reach			(r_c__reach + r_d__count)
#define r_l__count			3
    {	{	"r_l_",
		_libx1f4i0_lxline_l_sequent,	0,
		NULL,				r_l__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	line_lfar,							}
#define r_l__reach			(r_d__reach + r_l__count)
/* *INDENT-ON* */
#define long_line_class			(long_line_shift * 4)
#define long_line_count			r_l__reach
#define long_line_shift \
    sizeof(long_line) / sizeof(struct orient_type)
}, past_line[] = {
/* *INDENT-OFF* */
#define r_x__class			4
#define r_x__count			2
    {	{	"r_x_",
		NULL,				0,
		NULL,				r_x__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						r_x__class	},
	line_xmix,							},
#define r_x__reach			(0 + r_x__count)
#define r_x__trans			(0 + r_x__class)
#define r_xc__class			5
#define r_xc__count			2
    {	{	"r_xc_",
		NULL,				0,
		NULL,				r_xc__count,
		X1f4_E4_TEXT_LINK,		r_xc__class	},
	line_xvix,							},
#define r_xc__reach			(r_x__reach + r_xc__count)
#define r_xc__trans			(r_x__trans + r_xc__class)
#define rb_x__class			5
#define rb_x__count			1
    {	{	"rb_x_",
		NULL,				X1f4_E4_VOID,
		NULL,				rb_x__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						rb_x__class	},
	line_xpix,							},
#define rb_x__reach			(r_xc__reach + rb_x__count)
#define rb_x__trans			(r_xc__trans + rb_x__class)
#define rf_x__class			5
#define rf_x__count			1
    {	{	"rf_x_",
		NULL,				X1f4_E4_VOID,
		NULL,				rf_x__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						rf_x__class	},
	line_xsix,							}
#define rf_x__reach			(rb_x__reach + rf_x__count)
#define rf_x__trans			(rb_x__trans + rf_x__class)
/* *INDENT-ON* */
#define past_line_class			rf_x__trans
#define past_line_count			rf_x__reach
#define past_line_shift \
    sizeof(past_line) / sizeof(struct orient_type)
}, pipe_line[] = {
/* *INDENT-OFF* */
#define rcsi_g__count			3
    {	{	"rcsi_g_",
		NULL,				X1f4_E4_MODE,
		NULL,				rcsi_g__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		7		},
	long_rmix,							},
#define rcsi_g__reach			(0 + (rcsi_g__count << 1))
#define rcsi_l__count			3
    {	{	"rcsi_l_",
		NULL,				X1f4_E4_MODE,
		NULL,				rcsi_l__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		7		},
	long_smix,							}
#define rcsi_l__reach			(rcsi_g__reach + (rcsi_l__count << 1))
/* *INDENT-ON* */
#define pipe_line_class			(pipe_line_shift * 7)
#define pipe_line_count			rcsi_l__reach
#define pipe_line_shift \
    sizeof(pipe_line) / sizeof(struct orient_type)
}, seek_line[] = {
/* *INDENT-OFF* */
#define r_k__count			3
    {	{	"r_k_",
		_libx1f4i0_lxline_k_derived,	X1f4_E4_MODE,
		NULL,				r_k__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,		4		},
	line_kway,							}
#define r_k__reach			(0 + (r_k__count << 1))
/* *INDENT-ON* */
#define seek_line_class			(seek_line_shift * 4)
#define seek_line_count			r_k__reach
#define seek_line_shift \
    sizeof(seek_line) / sizeof(struct orient_type)
}, skew_line[] = {
/* *INDENT-OFF* */
#define rn_a__count			3
    {	{	"rn_a_",
		NULL,				0,
		NULL,				rn_a__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	last_amix,							}
#define rn_a__reach			(0 + rn_a__count)
/* *INDENT-ON* */
#define skew_line_class			(skew_line_shift * 5)
#define skew_line_count			rn_a__reach
#define skew_line_shift \
    sizeof(skew_line) / sizeof(struct orient_type)
}, spin_line[] = {
/* *INDENT-OFF* */
#define r_n__count			2
    {	{	"r_n_",
		_libx1f4i0_lxline_n_logique,	0,
		NULL,				r_n__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	born_n_lt,							},
#define r_n__reach			(0 + r_n__count)
#define r_v__count			2
    {	{	"r_v_",
		_libx1f4i0_lxline_v_logique,	0,
		NULL,				r_v__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	born_n_lt,							}
#define r_v__reach			(r_n__reach + r_v__count)
/* *INDENT-ON* */
#define spin_line_class			(spin_line_shift * 4)
#define spin_line_count			r_v__reach
#define spin_line_shift \
    sizeof(spin_line) / sizeof(struct orient_type)
}, tune_line[] = {
/* *INDENT-OFF* */
#define rn_s__count			3
    {	{	"rn_s_",
		NULL,				0,
		NULL,				rn_s__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	last_smix,							}
#define rn_s__reach			(0 + rn_s__count)
/* *INDENT-ON* */
#define tune_line_class			(tune_line_shift * 5)
#define tune_line_count			rn_s__reach
#define tune_line_shift \
    sizeof(tune_line) / sizeof(struct orient_type)
};
static const struct patent_type linked[] = {
/* *INDENT-OFF* */
    {	fr_getf,		5,				},
    {	fr_queryf, 		7,				},
    {	fr_pickf,		6,				},
    {	fr_yankf,		6,				}
/* *INDENT-ON* */
}, phased[] = {
/* *INDENT-OFF* */
    {	frs_greaterf,		10,				},
    {	frs_lessf,		7,				},
    {	frs_lowerf,		8,				},
    {	frs_upperf,		8,				}
/* *INDENT-ON* */
}, yanked[] = {
/* *INDENT-OFF* */
    {	fr_addf,		5,				},
    {	fr_bindf,		6,				},
    {	fr_casef,		6,				},
    {	fr_dockf,		6,				},
    {	fr_fixf,		5,				},
    {	fr_linkf,		6,				},
    {	fr_putf,		5,				},
    {	fr_replacef,		9,				}
/* *INDENT-ON* */
};
static const struct x1f4_postmark_type side_k[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("]", " "),	1				}
/* *INDENT-ON* */
};
static const struct x1f4_operator_type rail_f[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("=", " "),  NULL,		0400,
	0,			NULL,
	X1f4_E4_BACK_LINK | X1f4_E4_E2ND_LINK,
				1,
	NULL,			NULL					}
/* *INDENT-ON* */
}, rail_m[] = {
/* *INDENT-OFF* */
    {	MAKE_DOUBLE("-", "="),  NULL,		0400,
	0,			NULL,
	X1f4_E4_BACK_LINK | X1f4_E4_E2ND_LINK,
				2,
	NULL,			NULL					}
/* *INDENT-ON* */
}, rail_p[] = {
/* *INDENT-OFF* */
    {	MAKE_DOUBLE("+", "="),  NULL,		0400,
	0,			NULL,
	X1f4_E4_BACK_LINK | X1f4_E4_E2ND_LINK,
				2,
	NULL,			NULL					}
/* *INDENT-ON* */
}, side_o[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("=", " "),  NULL,		0400,
	0,			NULL,
	X1f4_E4_BACK_LINK | X1f4_E4_E2ND_LINK | X1f4_E4_LEFT_XSET,
				1,
	NULL,			NULL					},
    {	MAKE_SINGLE("[", " "),  NULL,		0200,
	0,			NULL,
	X1f4_E4_E2ND_LINK,	1,
	NULL,			NULL					},
    {	MAKE_SINGLE("[", " "),  NULL,		0200,
	0,			NULL,
	X1f4_E4_E2ND_LINK,	1,
	NULL,			NULL					},
    {	MAKE_SINGLE("[", " "),  NULL,		0200,
	0,			NULL,
	X1f4_E4_E2ND_LINK,	1,
	NULL,			NULL					},
    {	MAKE_SINGLE("`", " "),  NULL,		0200,
	0,			NULL,
	X1f4_E4_E2ND_LINK | X1f4_E4_POST_COPY,
				1,
	NULL,			NULL					}
/* *INDENT-ON* */
}, side_u[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("~", " "),  NULL,		0210,
	0,			NULL,
	X1f4_E4_E2ND_LINK,
				1,
	NULL,			NULL					}
/* *INDENT-ON* */
};

static int
init_line(struct lxline_type *lxline_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    int status;

    if (bits & AUTOLINK_LINK) {
	lxline_data->link_v = screen_data->lock_v;
    } else {
	lxline_data->link_v.data = NULL;
	lxline_data->link_v.free = NULL;
	lxline_data->link_v.link = NULL;
	lxline_data->link_v.mode = NULL;
	lxline_data->link_v.pick = NULL;
	lxline_data->link_v.slip = NULL;
    }

    if (bits & CODELINK_LINK) {
	lxline_data->link_w = screen_data->lock_w;
    } else {
	lxline_data->link_w.data = NULL;
	lxline_data->link_w.free = _x1f4_e4_free_data;
	lxline_data->link_w.link = _x1f4_e4_link_data;
	lxline_data->link_w.mode = _x1f4_e4_mode_data;
    }

    if (bits & EELOOKUP_LINK) {
	lxline_data->trap_e.eelookup_data = screen_data->lock_g.eelookup_data;
    } else {
	lxline_data->trap_e.eelookup_data = &_x1f4_type_lookup;
    }

    if (bits & MISSBAIL_LINK) {
	lxline_data->link_r.call = screen_data->lock_r.call;
	lxline_data->link_r.fine = screen_data->lock_r.fine;
	lxline_data->link_r.miss = screen_data->lock_r.miss;
	lxline_data->link_r.text = screen_data->lock_r.text;
    } else {
	lxline_data->link_r.call = NULL;
	lxline_data->link_r.fine = NULL;
	lxline_data->link_r.miss = NULL;
    }

    if (bits & COMPOSER_LINK) {
	lxline_data->link_q = screen_data->lock_q;
    } else {
	lxline_data->link_q.link = NULL;
    }

    if (bits & IMPLICIT_LINK) {
	lxline_data->link_p.link = screen_data->lock_p.link;
	lxline_data->link_p.text = screen_data->lock_p.text;
    } else {
	lxline_data->link_p.link = NULL;
	lxline_data->link_p.text = NULL;
    }

    if (bits & RESOURCE_LINK) {
	lxline_data->link_m = screen_data->lock_m;
    } else {
	lxline_data->link_m.data = NULL;
	lxline_data->link_m.free = _x1f4_e4_free_data;
	lxline_data->link_m.link = _x1f4_e4_link_data;
	lxline_data->link_m.mode = _x1f4_e4_mode_data;
    }

    if (bits & TIERLINK_LINK) {
	lxline_data->link_x = screen_data->lock_x;
    } else {
	lxline_data->link_x.deck = NULL;
    }

    if (bits & TEXTFLAT_LINK) {
	lxline_data->link_e = screen_data->lock_e;
    } else {
	lxline_data->link_e.data = NULL;
	lxline_data->link_e.line = NULL;
	lxline_data->link_e.post = NULL;
	lxline_data->link_e.push = NULL;
    }

    if (bits & TRANSFER_LINK) {
	lxline_data->link_o.fine = screen_data->link_o.fine;
    } else {
	lxline_data->link_o.fine = NULL;
    }

    status = link_line(lxline_data, bits, screen_data);

    return status;
}


static int
line_data(struct lxline_type *lxline_data, unsigned bits,
	  const struct depart_type *part_data, void *data, void *text,
	  void **args, void **dana)
{
#define cxbend_ever			lxline_data->rule_f.cxbend + 1
#define cxbend_lead			lxline_data->rule_f.cxbend + 0

    struct lxtext_type *lxtext_data;
    const struct lxtype_type *lxtype_data;
    const struct screen_type *screen_data;
    struct x1f4_linetext_type *linetext_data;
    unsigned i;

    screen_data = part_data->screen_data;

    lxtype_data = screen_data->lock_t.data;

    linetext_data = data;
    lxtext_data = text;

    lxtype_data--;

    _x1f4_type_effect(&lxtext_data->lxtype, X1f4_E4_CALL);
    lxtext_data->text = lxline_data;

    i = screen_data->lock_t.miss;
    for (; i; i--) {
	int code;
	unsigned flags;

	lxtext_data++;

	lxtype_data++;

	line_sail(lxline_data, screen_data, lxtext_data, lxtype_data);

	flags = lxtype_data->flags;

	if (flags & X1f4_LX_TYPE_EXCEPT) {
	    continue;
	}

	if (1) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, fast_line_shift, fast_line);
	}
	if (bits & AUTOLINK_LINK) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, cast_line_shift, cast_line);
	}
	if (bits & DATALINK_LINK) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, pipe_line_shift, pipe_line);
	}

	code = lxtype_data->code;

	if (code == X1f4_E4_BILL || code == X1f4_E4_MODE
	    || code == X1f4_E4_REAL) {
	    _libx1f4i0_lxcall_push_line
		(cxbend_lead, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, skew_line_shift, skew_line);
	    _libx1f4i0_lxcall_push_line
		(cxbend_ever, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, tune_line_shift, tune_line);
	}

	if (bits & AUTOLINK_LINK
	    || (!lxtype_data->lead && true(code != X1f4_E4_TEXT))) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, past_line_shift, past_line);
	}

	if (flags & LINK_ACCESS) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, long_line_shift, long_line);
	}
	if (0) {
	} else {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, copy_line_shift, copy_line);
	}
	if (flags & POST_ACCESS && bits & AUTOLINK_LINK) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, spin_line_shift, spin_line);
	}
	if (flags & POST_ACCESS && X1f4_E4_LAST < code && lxtype_data->lead) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, seek_line_shift, seek_line);
	}
    }

    return 0;

#undef cxbend_lead
#undef cxbend_ever
}


static int
line_line(struct lxline_type *lxline_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    lxline_data->link_t.datatype[0].context = lxline_data;

    lxline_data->link_t.datatype[0].flat = _libx1f4i0_lxline_flat_line;
    lxline_data->link_t.datatype[0].lead = _libx1f4i0_lxline_lead_line;
    lxline_data->link_t.datatype[0].line = _libx1f4i0_lxline_line_line;
    lxline_data->link_t.datatype[0].link = _libx1f4i0_lxline_link_line;
    lxline_data->link_t.datatype[0].name = "record";
    lxline_data->link_t.datatype[0].slip = _libx1f4i0_lxline_slip_line;
    lxline_data->link_t.datatype[0].size = 6;
    lxline_data->link_t.datatype[0].type = screen_data->link_l.type;

    lxline_data->link_s.nodetype[0].context = lxline_data;

    _libx1f4i0_lxline_type_line
	(lxline_data->link_s.nodetype + 0, screen_data->link_l.type);

    lxline_data->link_t.datatype[1].context = NULL;

    lxline_data->link_t.datatype[1].flat = NULL;
    lxline_data->link_t.datatype[1].lead = NULL;
    lxline_data->link_t.datatype[1].line = NULL;
    lxline_data->link_t.datatype[1].link = NULL;
    lxline_data->link_t.datatype[1].name = NULL;
    lxline_data->link_t.datatype[1].slip = NULL;
    lxline_data->link_t.datatype[1].size = 0;
    lxline_data->link_t.datatype[1].type = 0;

    return 0;
}


static int
line_plus(struct lxline_type *lxline_data)
{
    int i;
    struct cxbend_type *cxbend_data;
    struct cxseek_type *cxseek_data;

    cxbend_data = lxline_data->rule_f.cxbend;

    cxbend_data[0].bend = 0;
    cxbend_data[0].gate = _libx1f4i0_hike_sfnote;
    cxbend_data[0].line = lxline_data;
    cxbend_data[0].pick = _libx1f4i0_lead_sfnote;
    cxbend_data[0].sign = 1;
    cxbend_data[0].slip = _libx1f4i0_mind_sfnote;

    cxbend_data++;

    cxbend_data[0].bend = 1;
    cxbend_data[0].gate = _libx1f4i0_fold_sfnote;
    cxbend_data[0].line = lxline_data;
    cxbend_data[0].pick = _libx1f4i0_ever_sfnote;
    cxbend_data[0].sign = -1;
    cxbend_data[0].slip = _libx1f4i0_slip_sfnote;

    cxseek_data = lxline_data->rule_f.cxseek;

    cxseek_data[0].seek = _libx1f4i0_fgxq_sfnote;
    cxseek_data[1].seek = _libx1f4i0_fxxq_sfnote;
    cxseek_data[2].seek = _libx1f4i0_lxxq_sfnote;
    cxseek_data[3].seek = _libx1f4i0_llxq_sfnote;

    i = 4;
    for (; i; i--) {
	cxseek_data->line = lxline_data;
	cxseek_data++;
    }

    return 0;
}


static int
line_rail(void *data, unsigned pass,
	  struct x1f4_operator_type *operator_rail,
	  struct x1f4_2ndscope_type *scope2nd_rail,
	  struct x1f4_linetext_type *linetext_data)
{
    struct ticket_type *ticket_data;
    struct x1f4_2ndscope_type *scope2nd_data;
    struct x1f4_operator_type *operator_data; 
    struct x1f4_raillist_type *raillist_data;

    ticket_data = data;
    ticket_data -= fb_size() - 1;
    raillist_data = (void *) scope2nd_rail;
    raillist_data -= 1;
    raillist_data->miss = fi_size(pass);
    raillist_data->operator_data = operator_rail;
    scope2nd_data = &ticket_data->tidespan;
    scope2nd_data->bits |= X1f4_E4_POST_MARK | X1f4_E4_RAIL_LIST;
    scope2nd_data->post = side_k;
    scope2nd_data->rail = raillist_data;

    while (linetext_data->function.name != fr_casef) {
	linetext_data++;
    }

    operator_data = operator_rail + fi_size(pass) - 1;
    scope2nd_data = scope2nd_rail + fi_size(pass) - 1;
    *operator_data = *rail_f;
    operator_data->args = linetext_data->function.args + 1;
    operator_data->type = X1f4_E4_CASE;
    operator_data->extension2 = scope2nd_data;
    scope2nd_data->bits = 0;
    scope2nd_data->line = linetext_data;

    ticket_data++;
    scope2nd_data = &ticket_data->tidespan;
    scope2nd_data->bits |= X1f4_E4_POST_MARK;
    scope2nd_data->post = side_k;

    while (linetext_data->function.name != fro_casef) {
	linetext_data++;
    }

    operator_data++;

    ticket_data++;
    scope2nd_data = &ticket_data->tidespan;
    scope2nd_data->bits |= X1f4_E4_POST_MARK | X1f4_E4_RAIL_MARK;
    scope2nd_data->post = side_k;
    scope2nd_data->rail = operator_data;

    scope2nd_data = scope2nd_rail + fi_size(pass) - 0;
    *operator_data = *(operator_data - 1);
    operator_data->extension2 = scope2nd_data;
    *scope2nd_data = *(scope2nd_data - 1);
    scope2nd_data->line = linetext_data;

    ticket_data++;
    scope2nd_data = &ticket_data->tidespan;
    scope2nd_data->bits |= X1f4_E4_RAIL_LIST;
    scope2nd_data->rail = raillist_data;

    return 0;
}


static int
line_sail(struct lxline_type *lxline_data,
	  const struct screen_type *screen_data,
	  struct lxtext_type *lxtext_data,
	  const struct lxtype_type *lxtype_data)
{
    int code;

    code = lxtype_data->code;

    lxtext_data->lxtype = *lxtype_data;
    lxtext_data->text = lxline_data;

    if (0) {
    } else {
	if (code == X1f4_E4_TEXT) {
	    lxtext_data->lxtype.context = &lxline_data->ever_a;

	    lxtext_data->lxtype.copy = _libx1f4i0_lxnext_copy_text;
	    lxtext_data->lxtype.free = _libx1f4i0_lxnext_free_text;
	    lxtext_data->lxtype.lead = _libx1f4i0_lxnext_lead_text;
	    lxtext_data->lxtype.move = NULL;
	    lxtext_data->lxtype.push = _libx1f4i0_lxnext_push_text;
	    lxtext_data->lxtype.slip = _libx1f4i0_lxnext_slip_text;
	}
    }

    return 0;
}


static int
line_text(struct lxline_type *lxline_data, unsigned bits,
	  const struct depart_type *part_data, void *data, void **args)
{
#define cxbend_ever			lxline_data->rule_f.cxbend + 1
#define cxbend_lead			lxline_data->rule_f.cxbend + 0

    struct x1f4_linetext_type *linetext_data;

    linetext_data = data;

    if (1) {
	_libx1f4i0_lxcall_side_line
	    (cxbend_ever, bits, part_data, &linetext_data, args,
	     sizeof(gray_line) / sizeof(struct line_type), gray_line);
	_libx1f4i0_lxcall_side_line
	    (lxline_data, bits, part_data, &linetext_data, args,
	     sizeof(slip_line) / sizeof(struct line_type), slip_line);
	_libx1f4i0_lxcall_side_line
	    (cxbend_lead, bits, part_data, &linetext_data, args,
	     sizeof(used_line) / sizeof(struct line_type), used_line);
    }
    if (1) {
	part_text
	    (lxline_data, bits, part_data, &linetext_data, args,
	     sizeof(wrap_line) / sizeof(struct line_type), wrap_line);
    }
    if (bits & AUTOLINK_LINK) {
	_libx1f4i0_lxcall_side_line
	    (lxline_data, bits, part_data, &linetext_data, args,
	     sizeof(auto_line) / sizeof(struct line_type), auto_line);
	part_text
	    (lxline_data, bits, part_data, &linetext_data, args,
	     sizeof(blue_line) / sizeof(struct line_type), blue_line);
    }
    if ((bits & (AUTOLINK_LINK | DATALINK_LINK))
	== (AUTOLINK_LINK | DATALINK_LINK)) {
	_libx1f4i0_lxcall_side_line
	    (lxline_data, bits, part_data, &linetext_data, args,
	     sizeof(fine_line) / sizeof(struct line_type), fine_line);
	part_text
	    (lxline_data, bits, part_data, &linetext_data, args,
	     sizeof(fade_line) / sizeof(struct line_type), fade_line);
	_libx1f4i0_lxcall_side_line
	    (cxbend_lead, bits, part_data, &linetext_data, args,
	     sizeof(hold_line) / sizeof(struct line_type), hold_line);
	_libx1f4i0_lxcall_side_line
	    (cxbend_ever, bits, part_data, &linetext_data, args,
	     sizeof(keep_line) / sizeof(struct line_type), keep_line);
    }
    if (bits & AUTOLINK_LINK) {
	_libx1f4i0_lxcall_side_line
	    (lxline_data, bits, part_data, &linetext_data, args,
	     sizeof(fail_line) / sizeof(struct line_type), fail_line);
	_libx1f4i0_lxcall_side_line
	    (cxbend_lead, bits, part_data, &linetext_data, args,
	     sizeof(head_line) / sizeof(struct line_type), head_line);
	_libx1f4i0_lxcall_side_line
	    (cxbend_ever, bits, part_data, &linetext_data, args,
	     sizeof(tail_line) / sizeof(struct line_type), tail_line);
    }
    if (bits & DATALINK_LINK) {
	_libx1f4i0_lxcall_side_line
	    (lxline_data, bits, part_data, &linetext_data, args,
	     sizeof(beta_line) / sizeof(struct line_type), beta_line);
	_libx1f4i0_lxcall_side_line
	    (cxbend_lead, bits, part_data, &linetext_data, args,
	     sizeof(fold_line) / sizeof(struct line_type), fold_line);
	_libx1f4i0_lxcall_side_line
	    (cxbend_ever, bits, part_data, &linetext_data, args,
	     sizeof(deep_line) / sizeof(struct line_type), deep_line);
	part_text
	    (lxline_data, bits, part_data, &linetext_data, args,
	     sizeof(case_line) / sizeof(struct line_type), case_line);
    }
    if (1) {
	_libx1f4i0_lxcall_zero_line(linetext_data);
    }

    return 0;

#undef cxbend_lead
#undef cxbend_ever
}


static int
link_line(struct lxline_type *lxline_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    int status;
    unsigned args_class, args_count, line_class, line_count, main_class,
	main_count, miss, pass, trap, trip;
    void *data;

    args_count = 0;
    line_count = 1;
    main_count = 0;

    miss = screen_data->lock_t.miss;

    pick_miss(&args_class, &line_class, &main_class, &pass, bits, screen_data);

    trap = bits & RELATION_LINK ? 2 : 0;
    trip = bits & RELATION_LINK ? 1 : 0;

    {
	args_count += gray_line_count;
	line_count += sizeof(gray_line) / sizeof(struct line_type);
	args_count += slip_line_count;
	line_count += sizeof(slip_line) / sizeof(struct line_type);
	args_count += used_line_count;
	line_count += sizeof(used_line) / sizeof(struct line_type);
    }
    if (1) {
	args_count += wrap_line_count;
	line_count += sizeof(wrap_line) / sizeof(struct line_type);
    }
    if (bits & AUTOLINK_LINK) {
	args_count += auto_line_count;
	line_count += sizeof(auto_line) / sizeof(struct line_type);
	args_count += blue_line_count;
	line_count += sizeof(blue_line) / sizeof(struct line_type);
    }
    if ((bits & (AUTOLINK_LINK | DATALINK_LINK))
	== (AUTOLINK_LINK | DATALINK_LINK)) {
	args_count += fade_line_count;
	line_count += sizeof(fade_line) / sizeof(struct line_type);
	args_count += fine_line_count;
	line_count += sizeof(fine_line) / sizeof(struct line_type);
	args_count += hold_line_count;
	line_count += sizeof(hold_line) / sizeof(struct line_type);
	args_count += keep_line_count;
	line_count += sizeof(keep_line) / sizeof(struct line_type);
    }
    if (bits & AUTOLINK_LINK) {
	args_count += fail_line_count;
	line_count += sizeof(fail_line) / sizeof(struct line_type);
	args_count += head_line_count;
	line_count += sizeof(head_line) / sizeof(struct line_type);
	args_count += tail_line_count;
	line_count += sizeof(tail_line) / sizeof(struct line_type);
    }
    if (bits & DATALINK_LINK) {
	args_count += beta_line_count;
	line_count += sizeof(beta_line) / sizeof(struct line_type);
	args_count += case_line_count;
	line_count += sizeof(case_line) / sizeof(struct line_type);
	args_count += deep_line_count;
	line_count += sizeof(deep_line) / sizeof(struct line_type);
	args_count += fold_line_count;
	line_count += sizeof(fold_line) / sizeof(struct line_type);
    }

    status = lxline_data->link_w.link
	(lxline_data->link_w.data, &data,
	 sizeof(struct lxtext_type) * (1 + miss) + fm_size(away)
	 + (args_class + args_count) * sizeof(int)
	 + (line_class + line_count) * sizeof(struct x1f4_linetext_type)
	 + (sizeof(struct ticket_type) + sizeof(struct x1f4_operator_type *))
	 * (fb_size() + fp_size()) + sizeof(struct x1f4_operator_type *) * 2
	 + ((sizeof(struct x1f4_ftaa_type) + sizeof(struct x1f4_ftca_type))
	    * (pass + 1) + sizeof(struct x1f4_fpdetail_type)) * trap
	 + ((sizeof(struct x1f4_ftsl_type)) * (pass + 1)
	    + sizeof(struct x1f4_fpdetail_type)) * trap
	 + ((sizeof(struct x1f4_ftsl_type)) * (pass + 1)
	    + sizeof(struct x1f4_fpdetail_type)) * trip * (8 + 4)
	 + (sizeof(struct x1f4_operator_type)
	    + sizeof(struct x1f4_2ndscope_type)) * (fi_size(pass) + 1)
	 + sizeof(struct x1f4_raillist_type) + main_class + main_count);
    if (status) {
	status = LINK_ERROR;
    } else {
	struct depart_type part;
	void *args, *dana, *port, *rail, *rate, *side, *tile, *text;

	tile = (struct x1f4_linetext_type *) data + line_class;
	dana = (struct x1f4_linetext_type *) tile + line_count;

	if (trap) {
	    _libx1f4i0_lxrail_line_past(&part, pass, &dana);
	    _libx1f4i0_lxrail_line_fast(&part, pass, &dana);
	    _libx1f4i0_lxrail_line_cast(&part, pass, &dana);
	} else {
	    memset(&part, 0, sizeof(struct depart_type));
	}

	/*
	 * lxmiss and lxtext records required together
	 */
	text = (struct lxmiss_type *) dana + 1;
	side = (struct lxtext_type *) text + 1 + miss;
	side = (struct ticket_type *) side + fb_size();
	rate = (struct x1f4_operator_type **) side + fb_size() + 1;
	rate = (struct ticket_type *) rate + fp_size();
	port = (struct x1f4_operator_type **) rate + fp_size() + 1;
	port = (struct x1f4_raillist_type *) port + 1;
	rail = (struct x1f4_2ndscope_type *) port + fi_size(pass) + 1;
	args = (struct x1f4_operator_type *) rail + fi_size(pass) + 1;
	dana = (int *) args + args_class + args_count;

	((struct lxmiss_type *) text - 1)->data.miss = miss;

	part.screen_data = screen_data;

	line_plus(lxline_data);

	lxline_data->link_f.data = port;
	lxline_data->link_f.side = rail;

	line_data(lxline_data, bits, &part, data, text, &args, &dana);
	line_text(lxline_data, bits, &part, tile, &args);
	_libx1f4i0_lxcall_line_side(side, data, side_o, side_f, fb_size());
	_libx1f4i0_lxcall_line_side(rate, data, side_u, side_l, fp_size());

	line_rail(side, pass, rail, port, data);

	/*
	 * link_f abused for line_data, etc, not to be set until after.
	 */

	lxline_data->link_f.data = data;
	lxline_data->link_f.rate = rate;
	lxline_data->link_f.side = side;
	lxline_data->link_f.text = text;

	line_line(lxline_data, bits, screen_data);

	if (trap) {
	    _libx1f4i0_lxrail_link_past
		(screen_data->link_y.linetrap_data, &part, linked);
	    _libx1f4i0_lxrail_link_fast
		(screen_data->link_y.linetrap_data, &part, phased);
	    _libx1f4i0_lxrail_link_cast
		(screen_data->link_y.linetrap_data, &part, yanked);
	}

	if (status) {
	    lxline_data->link_w.free(lxline_data->link_w.data, data);
	} else {
	    if (bits & DATALINK_LINK) {
		lxline_data->link_i.data = screen_data->lock_f.type;
	    } else {
		lxline_data->link_i.data = -1;
	    }
	}
    }

    return status;
}


static int
part_text(struct lxline_type *lxline_data, unsigned bits,
	  const struct depart_type *part_data,
	  struct x1f4_linetext_type **linetext, void **args, unsigned count,
	  const struct line_type *line_data)
{
    struct cxseek_type *cxseek_data;
    struct x1f4_linetext_type *linetext_data;
    unsigned i = 4;

    cxseek_data = lxline_data->rule_f.cxseek;

    linetext_data = *linetext;

    count >>= 2;

    for (; i; i--) {
	_libx1f4i0_lxcall_side_line
	    (cxseek_data, bits, part_data, &linetext_data, args, count,
	     line_data);
	cxseek_data++;
	line_data += count;
    }

    *linetext = linetext_data;

    return 0;
}


static int
pick_miss(unsigned *args, unsigned *line, unsigned *pick, unsigned *pass,
	  unsigned bits, const struct screen_type *screen_data)
{
    const struct lxtype_type *lxtype_data;
    unsigned args_count = 0, line_count = 0, i, main_count = 0, miss;

    lxtype_data = screen_data->lock_t.data;
    lxtype_data--;

    miss = screen_data->lock_t.miss;

    i = miss;
    for (; i; i--) {
	int code;
	unsigned deck, flags;

	lxtype_data++;

	flags = lxtype_data->flags;

	if (flags & X1f4_LX_TYPE_EXCEPT) {
	    miss--;
	    if (1) {
		continue;
	    }
	}

	deck = lxtype_data->size + 1;

	if (1) {
	    args_count += fast_line_count;
	    line_count += fast_line_shift;
	    main_count += deck * fast_line_shift + fast_line_class;
	}
	if (bits & AUTOLINK_LINK) {
	    args_count += cast_line_count;
	    line_count += cast_line_shift;
	    main_count += deck * cast_line_shift + cast_line_class;
	}
	if (bits & DATALINK_LINK) {
	    args_count += pipe_line_count;
	    line_count += pipe_line_shift;
	    main_count += deck * pipe_line_shift + pipe_line_class;
	}

	code = lxtype_data->code;

	if (code == X1f4_E4_BILL || code == X1f4_E4_MODE
	    || code == X1f4_E4_REAL) {
	    args_count += skew_line_count;
	    line_count += skew_line_shift;
	    main_count += deck * skew_line_shift + skew_line_class;
	    args_count += tune_line_count;
	    line_count += tune_line_shift;
	    main_count += deck * tune_line_shift + tune_line_class;
	    if (flags & FLAT_REFLEX) {
	    } else {
	    }
	}

	if (bits & AUTOLINK_LINK
	    || (!lxtype_data->lead && true(code != X1f4_E4_TEXT))) {
	    args_count += past_line_count;
	    line_count += past_line_shift;
	    main_count += deck * past_line_shift + past_line_class;
	}

	if (flags & FLAT_REFLEX) {
	} else {
	    args_count += l_e_type_post;
	    args_count += l_ec_set_post;
	    args_count += l_g_type_post;
	    args_count += l_gc_set_post;
	    args_count += l_j_type_post;
	    args_count += l_o_type_post;
	    args_count += lbe_type_post;
	    args_count += lbg_type_post;
	    args_count += lfe_type_post;
	    args_count += lfg_type_post;
	}
	if (flags & LINK_ACCESS) {
	    args_count += long_line_count;
	    line_count += long_line_shift;
	    main_count += deck * long_line_shift + long_line_class;
	}
	if (0) {
	} else {
	    args_count += copy_line_count;
	    line_count += copy_line_shift;
	    main_count += deck * copy_line_shift + copy_line_class;
	}
	if (flags & POST_ACCESS && bits & AUTOLINK_LINK) {
	    args_count += spin_line_count;
	    line_count += spin_line_shift;
	    main_count += deck * spin_line_shift + spin_line_class;
	}
	if (flags & POST_ACCESS && X1f4_E4_LAST < code && lxtype_data->lead) {
	    args_count += seek_line_count;
	    line_count += seek_line_shift;
	    main_count += deck * seek_line_shift + seek_line_class;
	}
    }

    *pass = miss;

    *args = args_count;
    *line = line_count;
    *pick = main_count;

    return 0;
}


static void
_slttx00x(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = X1f4_E4_SLIP;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = X1f4_E4_TEXT;
    *line++ = X1f4_E4_POST_XSET;
    *line++ = 0;
    *line++ = 0;
    *line++ = X1f4_E4_POST_XSET;

    *args = line;
}


static void
bo_ltt00x(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = X1f4_E4_TEXT;
    *line++ = 0;
    *line++ = 0;
    *line++ = X1f4_E4_POST_XSET;

    *args = line;
}


static void
bo_slfx00(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = X1f4_E4_SLIP;
    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->lock_f.type;
    *line++ = X1f4_E4_POST_XSET;
    *line++ = 0;
    *line++ = 0;

    *args = line;
}


static void
bo_sltx00(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = X1f4_E4_SLIP;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = X1f4_E4_POST_XSET;
    *line++ = 0;
    *line++ = 0;

    *args = line;
}


static void
bo_sltx0x(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = X1f4_E4_SLIP;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = X1f4_E4_POST_XSET;
    *line++ = 0;
    *line++ = X1f4_E4_POST_XSET;

    *args = line;
}


static void
bo_slty00(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = X1f4_E4_SLIP;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = X1f4_E4_POST_FLAT | X1f4_E4_POST_XSET;
    *line++ = 0;
    *line++ = 0;

    *args = line;
}


static void
born_lt0x(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = 0;
    *line++ = X1f4_E4_POST_XSET;

    *args = line;
}


static void
born_n_lt(v_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;

    *args = line;

    linetext_data->function.type = nodetype_data->code;
}


static void
born_sly0(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = X1f4_E4_SLIP;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_POST_FLAT | X1f4_E4_POST_XSET;
    *line++ = 0;

    *args = line;
}


static void
borne_lcs(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_CASE;
    *line++ = X1f4_E4_SLIP;

    *args = line;
}


static void
borne_ltc(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = X1f4_E4_CASE;

    *args = line;
}


static void
borne_lts(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = X1f4_E4_SLIP;

    *args = line;
}


static void
borne_ltt(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = X1f4_E4_TEXT;

    *args = line;
}


static void
borneo__l(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;

    *args = line;
}


static void
borneo_l_(u_case_args_____1)
{
    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
borneo_lc(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_CASE;

    *args = line;
}


static void
borneo_lf(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->lock_f.type;

    *args = line;
}


static void
borneo_lt(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;

    *args = line;
}


static void
last_amix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_0mix
        (v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
         rate_0vix);

    rail_plus
	(linetext_data, ((struct cxbend_type *) linetext_data->context)->line,
	 rail_p);
}


static void
last_smix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_0mix
        (v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
         rate_0vix);

    rail_plus
	(linetext_data, ((struct cxbend_type *) linetext_data->context)->line,
	 rail_m);
}


static void
line_amix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_pmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 rate_amix);

    if (nodetype_data->flags & LINK_ACCESS) {
    } else {
	struct x1f4_ftsl_type *ftsl_data;

	ftsl_data = part(screen)->ftsl_dlaw;
	if (ftsl_data) {
	    ftsl_data->linetext_data = linetext_data;
	    ftsl_data->type = nodetype_data->code;
	    part(screen)->ftsl_dlaw = ftsl_data + 1;
	}
    }

    if (0) {
    } else {
	struct x1f4_ftsl_type *ftsl_data;

	ftsl_data = part(screen)->ftsl_alaw;
	if (ftsl_data) {
	    ftsl_data->linetext_data = linetext_data;
	    ftsl_data->type = nodetype_data->code;
	    part(screen)->ftsl_alaw = ftsl_data + 1;
	}
    }
}


static void
line_bfar(v_case_args_____1)
{
    int *line, type;
    struct x1f4_ftsl_type *ftsl_data;

    line = *args;

    linetext_data->function.args = line;

    type = nodetype_data->code;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = type;

    *args = line;

    linetext_data->function.type = type;

    ftsl_data = part(screen)->ftsl_blaw;
    if (ftsl_data) {
	ftsl_data->linetext_data = linetext_data;
	ftsl_data->type = type;
	part(screen)->ftsl_blaw = ftsl_data + 1;
    }
}


static void
line_cfar(v_case_args_____1)
{
    int *line, type;
    struct x1f4_ftsl_type *ftsl_data;

    line = *args;

    linetext_data->function.args = line;

    type = nodetype_data->code;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = type;

    *args = line;

    linetext_data->function.type = type;

    rail_plus
	(linetext_data, ((struct lxtext_type *) linetext_data->context)->text,
	 rail_f);

    ftsl_data = part(screen)->ftsl_claw;
    if (ftsl_data) {
	ftsl_data->linetext_data = linetext_data;
	ftsl_data->type = type;
	part(screen)->ftsl_claw = ftsl_data + 1;
    }
}


static void
line_dfar(v_case_args_____1)
{
    int *line, type;
    struct x1f4_ftsl_type *ftsl_data;

    line = *args;

    linetext_data->function.args = line;

    type = nodetype_data->code;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = type;

    *args = line;

    ftsl_data = part(screen)->ftsl_dlaw;
    if (ftsl_data) {
	ftsl_data->linetext_data = linetext_data;
	ftsl_data->type = type;
	part(screen)->ftsl_dlaw = ftsl_data + 1;
    }
}


static void
line_emix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_emix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT, ~0,
	 &part(screen)->ftsl_ylaw, rate_emix);
}


static void
line_epix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_esix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_epix);
}


static void
line_esix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_esix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_esix);
}


static void
line_evix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_imix
        (v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
         &part(screen)->ftaa_xlaw, rate_evix);
}


static void
line_fmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_fmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 rate_fmix);

    if (nodetype_data->flags & LINK_ACCESS) {
    } else {
	struct x1f4_ftsl_type *ftsl_data;

	rail_plus
	    (linetext_data,
	     ((struct lxtext_type *) linetext_data->context)->text, rail_f);

	ftsl_data = part(screen)->ftsl_claw;
	if (ftsl_data) {
	    ftsl_data->linetext_data = linetext_data;
	    ftsl_data->type = nodetype_data->code;
	    part(screen)->ftsl_claw = ftsl_data + 1;
	}
    }

    if (0) {
    } else {
	struct x1f4_ftsl_type *ftsl_data;

	ftsl_data = part(screen)->ftsl_flaw;
	if (ftsl_data) {
	    ftsl_data->linetext_data = linetext_data;
	    ftsl_data->type = nodetype_data->code;
	    part(screen)->ftsl_flaw = ftsl_data + 1;
	}
    }
}


static void
line_gmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_gmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 &part(screen)->ftsl_glaw, rate_gmix);
}


static void
line_gpix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_gsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_gpix);
}


static void
line_gsix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_gsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_gsix);
}


static void
line_gvix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_imix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 &part(screen)->ftaa_qlaw, rate_gvix);
}


static void
line_jmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_jmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 X1f4_E4_MODE, rate_jmix);
}


static void
line_kway(v_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = nodetype_data->code;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = X1f4_E4_POST_XSET;
    *line++ = 0;
    *line++ = 0;

    *args = line;
}


static void
line_lfar(v_case_args_____1)
{
    int *line, type;
    struct x1f4_ftsl_type *ftsl_data;

    line = *args;

    linetext_data->function.args = line;

    type = nodetype_data->code;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = type;

    *args = line;

    linetext_data->function.type = type;

    ftsl_data = part(screen)->ftsl_llaw;
    if (ftsl_data) {
	ftsl_data->linetext_data = linetext_data;
	ftsl_data->type = type;
	part(screen)->ftsl_llaw = ftsl_data + 1;
    }
}


static void
line_nset(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_POST_XSET;

    *args = line;
}


static void
line_omix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_omix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 X1f4_E4_MODE, rate_omix);
}


static void
line_pmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_fmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 rate_pmix);

    if (nodetype_data->flags & LINK_ACCESS) {
    } else {
	struct x1f4_ftsl_type *ftsl_data;

	ftsl_data = part(screen)->ftsl_llaw;
	if (ftsl_data) {
	    ftsl_data->linetext_data = linetext_data;
	    ftsl_data->type = nodetype_data->code;
	    part(screen)->ftsl_llaw = ftsl_data + 1;
	}
    }

    if (0) {
    } else {
	struct x1f4_ftsl_type *ftsl_data;

	ftsl_data = part(screen)->ftsl_plaw;
	if (ftsl_data) {
	    ftsl_data->linetext_data = linetext_data;
	    ftsl_data->type = nodetype_data->code;
	    part(screen)->ftsl_plaw = ftsl_data + 1;
	}
    }
}


static void
line_qmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_wmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 rate_qmix);
}


static void
line_qpix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_qsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_qpix);
}


static void
line_qsix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_qsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_qsix);
}


static void
line_qvix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_qmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 &part(screen)->ftca_qlaw, rate_qvix);
}


static void
line_rmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_fmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 rate_rmix);

    if (nodetype_data->flags & LINK_ACCESS) {
    } else {
	struct x1f4_ftsl_type *ftsl_data;

	ftsl_data = part(screen)->ftsl_blaw;
	if (ftsl_data) {
	    ftsl_data->linetext_data = linetext_data;
	    ftsl_data->type = nodetype_data->code;
	    part(screen)->ftsl_blaw = ftsl_data + 1;
	}
    }

    if (0) {
    } else {
	struct x1f4_ftsl_type *ftsl_data;

	ftsl_data = part(screen)->ftsl_rlaw;
	if (ftsl_data) {
	    ftsl_data->linetext_data = linetext_data;
	    ftsl_data->type = nodetype_data->code;
	    part(screen)->ftsl_rlaw = ftsl_data + 1;
	}
    }
}


static void
line_smap(v_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;

    *args = line;
}


static void
line_sset(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_POST_FLAT | X1f4_E4_POST_XSET;
    *line++ = 0;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
line_vmap(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_CALL;
    *line++ = X1f4_E4_MODE;

    *args = line;
}


static void
line_wmap(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_CALL;
    *line++ = X1f4_E4_MODE;
    *line++ = X1f4_E4_MODE;

    *args = line;
}


static void
line_xmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_xmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 rate_xmix);
}


static void
line_xpix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_xsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_xpix);
}


static void
line_xsix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_xsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_xsix);
}


static void
line_xvix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_ymix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 &part(screen)->ftca_xlaw, rate_xvix);
}


static void
lock_gmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_lock_wmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 (void *) &part(screen)->fpdetail_slot[0].ftsl_data,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxseek + 0, turn_gmix);
}


static void
lock_lmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_lock_wmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 (void *) &part(screen)->fpdetail_slot[1].ftsl_data,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxseek + 3, turn_lmix);
}


static void
lock_omix(v_case_args_____1)
{
    _libx1f4i0_lxcall_lock_wmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 (void *) &part(screen)->fpdetail_slot[2].ftsl_data,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxseek + 2, turn_omix);
}


static void
long_rmix(v_case_args_____1)
{
    int *line, type;
    unsigned flags;

    flags = nodetype_data->flags;

    line = *args;

    linetext_data->function.args = line;

    type = nodetype_data->code;

    *line++ = type;
    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->lock_f.type;
    if (flags & FLAT_REFLEX) {
	*line++ = 0;
    } else {
	*line++ = X1f4_E4_POST_XSET;
    }
    {
	*line++ = 0;
	*line++ = 0;
    }

    *args = line;

    if (nodetype_data->lead) {
	linetext_data->function.function = _libx1f4i0_lxline_r_order_v;
    } else {
	if (X1f4_E4_LAST < type) {
	    linetext_data->function.function = _libx1f4i0_lxline_r_order_u;
	} else {
	    linetext_data->context =
		((struct lxline_type *)
		 ((struct lxtext_type *)
		  linetext_data->context)->text)->rule_f.cxseek + 0;

	    if (type == X1f4_E4_TEXT) {
		linetext_data->function.function = _libx1f4i0_lxline_1_order_t;
	    } else {
		if (type == X1f4_E4_REAL) {
		    linetext_data->function.function =
			_libx1f4i0_lxline_1_order_r;
		} else {
		    if (type == X1f4_E4_MODE) {
			linetext_data->function.function =
			    _libx1f4i0_lxline_1_order_m;
		    } else {
			if (type == X1f4_E4_BILL) {
			    linetext_data->function.function =
				_libx1f4i0_lxline_1_order_b;
			} else {
			}
		    }
		}
	    }
	}
    }
}


static void
long_smix(v_case_args_____1)
{
    int *line, type;
    unsigned flags;

    flags = nodetype_data->flags;

    line = *args;

    linetext_data->function.args = line;

    type = nodetype_data->code;

    *line++ = type;
    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->lock_f.type;
    if (flags & FLAT_REFLEX) {
	*line++ = 0;
    } else {
	*line++ = X1f4_E4_POST_XSET;
    }
    {
	*line++ = 0;
	*line++ = 0;
    }

    *args = line;

    if (nodetype_data->lead) {
	linetext_data->function.function = _libx1f4i0_lxline_s_order_v;
    } else {
	if (X1f4_E4_LAST < type) {
	    linetext_data->function.function = _libx1f4i0_lxline_s_order_u;
	} else {
	    linetext_data->context =
		((struct lxline_type *)
		 ((struct lxtext_type *)
		  linetext_data->context)->text)->rule_f.cxseek + 3;

	    if (type == X1f4_E4_TEXT) {
		linetext_data->function.function = _libx1f4i0_lxline_1_order_t;
	    } else {
		if (type == X1f4_E4_REAL) {
		    linetext_data->function.function =
			_libx1f4i0_lxline_1_order_r;
		} else {
		    if (type == X1f4_E4_MODE) {
			linetext_data->function.function =
			    _libx1f4i0_lxline_1_order_m;
		    } else {
			if (type == X1f4_E4_BILL) {
			    linetext_data->function.function =
				_libx1f4i0_lxline_1_order_b;
			} else {
			}
		    }
		}
	    }
	}
    }
}


static void
lock_umix(v_case_args_____1)
{
    _libx1f4i0_lxcall_lock_wmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_TEXT,
	 (void *) &part(screen)->fpdetail_slot[3].ftsl_data,
	 ((struct lxline_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxseek + 1, turn_umix);
}


static void
rail_plus(struct x1f4_linetext_type *linetext_data,
	  struct lxline_type *lxline_data,
	  const struct x1f4_operator_type *rail_p)
{
    struct x1f4_2ndscope_type *scope2nd_data;
    struct x1f4_operator_type *operator_data;

    operator_data = lxline_data->link_f.side;
    scope2nd_data = lxline_data->link_f.data;

    lxline_data->link_f.data = scope2nd_data + 1;
    lxline_data->link_f.side = operator_data + 1;
    *operator_data = *rail_p;
    operator_data->args = linetext_data->function.args + 1;
    operator_data->type = linetext_data->function.type;
    operator_data->extension2 = scope2nd_data;
    scope2nd_data->bits = 0;
    scope2nd_data->line = linetext_data;
}


static void
themed__l(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
themed_ll(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->link_l.type;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
themed_lt(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
theme_ltt(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_TEXT;
    *line++ = X1f4_E4_TEXT;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


int
x1f4_init_lxline(void **lxline, unsigned bits,
		 const struct screen_type *screen_data)
{
    int (*link)(void *, void **, unsigned), status;
    void *line, *text;

    if (bits & CODELINK_LINK) {
	link = screen_data->lock_w.link;
	text = screen_data->lock_w.data;
    } else {
	link = _x1f4_e4_link_data;
	text = (void *) 0;
    }

    status = link(text, &line, sizeof(struct lxline_type));
    if (status) {
	status = LINK_ERROR;
    } else {
	status = init_line(line, bits, screen_data);
	if (status) {
	    if (bits & CODELINK_LINK) {
		screen_data->lock_w.free(text, line);
	    } else {
		_x1f4_e4_free_data(text, line);
	    }
	} else {
	    *lxline = line;
	}
    }

    return status;
}
