/*
 * lxline-a.j.c
 * Copyright (C) 2017-2019, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e4.h>
#include <lxline-types.h>
#include <lxrail.h>
#include <progress.h>

static const struct x1f4_position_type position_0x[] = {
/* *INDENT-OFF* */
    {
    {	"r_key(;*, ;0)",	1,	0,	0,	1,	13	},
	X1f4_PROGRESS_EQCLAUSE
    },
    {
    {	"rsk_upper(;*, ;0, ;0)",
				2,	0,	0,	1,	21	},
	X1f4_PROGRESS_GECLAUSE
    },
    {
    {	"rsk_greater(;*, ;0, ;0)",
				2,	0,	0,	1,	23	},
	X1f4_PROGRESS_GTCLAUSE
    },
    {
    {	"rsk_lower(;*, ;0, ;0)",
				2,	0,	0,	1,	21	},
	X1f4_PROGRESS_LECLAUSE
    },
    {
    {	"rsk_less(;*, ;0, ;0)",	2,	0,	0,	1,	20	},
	X1f4_PROGRESS_LTCLAUSE
    },
/* *INDENT-ON* */
}, position_1x[] = {
/* *INDENT-OFF* */
    {
    {	"r_jack(;1, ;*, ;0)",	1,	1,	0,	1,	18	},
	X1f4_PROGRESS_EQCLAUSE
    },
    {
    {	"rs_upper(;1, ;*, ;0, ;0)",
				2,	1,	0,	1,	24	},
	X1f4_PROGRESS_GECLAUSE
    },
    {
    {	"rs_greater(;1, ;*, ;0, ;0)",
				2,	1,	0,	1,	26	},
	X1f4_PROGRESS_GTCLAUSE
    },
    {
    {	"rs_lower(;1, ;*, ;0, ;0)",
				2,	1,	0,	1,	24	},
	X1f4_PROGRESS_LECLAUSE
    },
    {
    {	"rs_less(;1, ;*, ;0, ;0)",
				2,	1,	0,	1,	23	},
	X1f4_PROGRESS_LTCLAUSE
    },
/* *INDENT-ON* */
}, position_2x[] = {
/* *INDENT-OFF* */
    {
    {	"rc_key(;*, ;0)",	1,	0,	0,	1,	14	},
	X1f4_PROGRESS_EQCLAUSE
    },
    {
    {	"rcsik_upper(;*, ;0)",	1,	0,	0,	1,	19	},
	X1f4_PROGRESS_GECLAUSE
    },
    {
    {	"rcsik_greater(;*, ;0)",
				1,	0,	0,	1,	21	},
	X1f4_PROGRESS_GTCLAUSE
    },
    {
    {	"rcsik_lower(;*, ;0)",	1,	0,	0,	1,	19	},
	X1f4_PROGRESS_LECLAUSE
    },
    {
    {	"rcsik_less(;*, ;0)",	1,	0,	0,	1,	18	},
	X1f4_PROGRESS_LTCLAUSE
    },
/* *INDENT-ON* */
}, position_3x[] = {
/* *INDENT-OFF* */
    {
    {	"rc_jack(;1, ;*, ;0)",	1,	1,	0,	1,	19	},
	X1f4_PROGRESS_EQCLAUSE
    },
    {
    {	"rcsi_upper(;1, ;*, ;0)",
				1,	1,	0,	1,	22	},
	X1f4_PROGRESS_GECLAUSE
    },
    {
    {	"rcsi_greater(;1, ;*, ;0)",
				1,	1,	0,	1,	24	},
	X1f4_PROGRESS_GTCLAUSE
    },
    {
    {	"rcsi_lower(;1, ;*, ;0)",
				1,	1,	0,	1,	22	},
	X1f4_PROGRESS_LECLAUSE
    },
    {
    {	"rcsi_less(;1, ;*, ;0)",
				1,	1,	0,	1,	21	},
	X1f4_PROGRESS_LTCLAUSE
    },
/* *INDENT-ON* */
};
static const struct x1f4_progress_type progress_0b = {
/* *INDENT-OFF* */
    {	"rsk_less(;*, ;0, ;0)",	2,	0,	0,	1,	20	},
    {	"r_last(;*, ;0)",	1,	0,	0,	1,	14	},
	X1f4_E4_TEXT,	0,		0,
	X1f4_PROGRESS_BACKWARD | X1f4_PROGRESS_POSITION,	1,
	position_0x
/* *INDENT-ON* */
}, progress_0f = {
/* *INDENT-OFF* */
    {	"rsk_greater(;*, ;0, ;0)",
				2,	0,	0,	1,	23	},
    {	"r_first(;*, ;0)",	1,	0,	0,	1,	15	},
	X1f4_E4_TEXT,	0,		0,
	X1f4_PROGRESS_POSITION,		1,
	position_0x
/* *INDENT-ON* */
}, progress_1b = {
/* *INDENT-OFF* */
    {	"rs_less(;1, ;*, ;0, ;0)",
				2,	1,	0,	1,	23	},
    {	"rs_last(;1, ;*, ;0)",	1,	1,	0,	1,	19	},
	X1f4_E4_TEXT,	X1f4_E4_SLIP,	0,
	X1f4_PROGRESS_BACKWARD | X1f4_PROGRESS_POSITION,	2,
	position_1x
/* *INDENT-ON* */
}, progress_1f = {
/* *INDENT-OFF* */
    {	"rs_greater(;1, ;*, ;0, ;0)",
				2,	1,	0,	1,	26	},
    {	"rs_first(;1, ;*, ;0)",	1,	1,	0,	1,	20	},
	X1f4_E4_TEXT,	X1f4_E4_SLIP,	0,
	X1f4_PROGRESS_POSITION,		2,
	position_1x
/* *INDENT-ON* */
}, progress_2b = {
/* *INDENT-OFF* */
    {	"rcsik_less(;*, ;0)",	1,	0,	0,	1,	18	},
    {	"rc_last(;*, ;0)",	1,	0,	0,	1,	15	},
	0,		0,		0,
	X1f4_PROGRESS_BACKWARD | X1f4_PROGRESS_POSITION,	1,
	position_2x
/* *INDENT-ON* */
}, progress_2f = {
/* *INDENT-OFF* */
    {	"rcsik_greater(;*, ;0)",
				1,	0,	0,	1,	21	},
    {	"rc_first(;*, ;0)",	1,	0,	0,	1,	16	},
	0,		0,		0,
	X1f4_PROGRESS_POSITION,		1,
	position_2x
/* *INDENT-ON* */
}, progress_3b = {
/* *INDENT-OFF* */
    {	"rcsi_less(;1, ;*, ;0)",
				1,	1,	0,	1,	21	},
    {	"rcs_last(;1, ;*, ;0)",	1,	1,	0,	1,	20	},
	0,		X1f4_E4_SLIP,	0,
	X1f4_PROGRESS_BACKWARD | X1f4_PROGRESS_POSITION,	2,
	position_3x
/* *INDENT-ON* */
}, progress_3f = {
/* *INDENT-OFF* */
    {	"rcsi_greater(;1, ;*, ;0)",
				1,	1,	0,	1,	24	},
    {	"rcs_first(;1, ;*, ;0)",
				1,	1,	0,	1,	21	},
	0,		X1f4_E4_SLIP,	0,
	X1f4_PROGRESS_POSITION,		2,
	position_3x
/* *INDENT-ON* */
};

int
x1f4_pass_lxline(void *lxline, struct x1f4_progress_type **progress,
		 char **copy)
{
    int status;

    status = _libx1f4i0_lxrail_pass_type
	(__types_rail(lxline), progress, copy, &progress_0f, &progress_0b,
	 &progress_1f, &progress_1b, "rs_g_", "rs_l_", 5, "(;1, ;*, ;0, ;0)",
	 16);
    if (status) {
    } else {
	int data;

	data = ((struct lxline_type *) lxline)->link_i.data;
	if (data ^ -1) {
	    struct x1f4_progress_type progress_cb, progress_cf, progress_db,
		progress_df;

	    progress_cb = progress_2b;
	    progress_cf = progress_2f;
	    progress_db = progress_3b;
	    progress_df = progress_3f;

	    progress_cb.arg1 = data;
	    progress_cf.arg1 = data;
	    progress_db.arg1 = data;
	    progress_df.arg1 = data;

	    status = _libx1f4i0_lxrail_pass_type
		(__types_rail(lxline), progress, copy, &progress_cf,
		 &progress_cb, &progress_df, &progress_db, "rcsi_g_",
		 "rcsi_l_", 7, "(;1, ;*, ;0)", 12);
	}
    }

    return status;
}

