/*
 * lxline-d.d.c
 * Copyright (C) 2008-2014, 2017, 2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxline-config.h>

#include <stddef.h>

#include <copy.h>
#include <e4.h>
#include <fd.h>
#include <l_note.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxline-close.h>
#include <lxline-drive.h>
#include <lxline-inter.h>
#include <lxline-types.h>
#include <nt.h>

int
_libx1f4i0_lxline_d_derived(void *context, void *output, void **input)
{
    int status, type;
    struct dxline_type *dxline_data;
    struct lxline_type *lxline_data;
    struct lxtext_type *lxtext_data, *lxtext_link;
    unsigned miss;
    void *trap;

    lxline_data = context;

    trap = input[2];

    type = ((struct x1f4_function_type *)
	    ((void **) input[-1])[X1f4_E4_RENDITION])->args[2];
    if (type == X1f4_E4_CASE) {
	struct x1f4_dxcast_type *dxcast_data;

	dxcast_data = I_USER(trap);
	trap = &dxcast_data->data;
	type = dxcast_data->type;
    }

    lxtext_link = lxline_data->link_f.text;

    lxtext_data = lxtext_link;

    close_frame(miss, lxtext_data);

    dxline_data = I_LINE(input[0]);

    do {
	int (*copy)(void *, void *, void **, const void *,
		    const struct x1f4_nodelink_type *),
	    (*node)(void *, void **, void *,
		    const struct x1f4_nodelink_type *);
	void *data, *side;

	if (miss) {
	    if (lxtext_data->lxtype.flags & X1f4_LX_LINK_ACCESS) {
		node = lxtext_data->lxtype.node;
		copy = NULL;
	    } else {
		node = NULL;
		copy = lxtext_data->lxtype.copy;
		if (copy) {
		    status = _libx1f4i0_find_sfnote
			(dxline_data->line, I_TEXT(input[1]), &data);
		    if (status) {
			status = 0;
			if (1) {
			    break;
			}
		    } else {
			status = copy
			    (lxtext_data->lxtype.context, NULL, &side,
			     I_USER(trap), lock_excase(dxline_data));
			if (status) {
			    break;
			}
		    }
		}
	    }
	} else {
	    copy = NULL;
	    node = NULL;
	}

	if (0) {
	} else {
	    status = _libx1f4i0_post_sfnote
		(dxline_data->line, I_TEXT(input[1]),
		 effect_size, &data);
	    if (status) {
		if (status == X1f4_SFNOTE_EVER_MATCH) {
		    status = 0;
		} else {
		    status = _libx1f4i0_lxline_stat_text
			(lxline_data, status, I_TEXT(input[1]));
		}

		if (copy) {
		    int excess;

		    excess = lxtext_data->lxtype.free
			(lxtext_data->lxtype.context, side, NULL,
			 lock_excase(dxline_data));
		    if (excess) {
			status = _libx1f4i0_lxcall_land_slip(excess, status);
		    }
		}
	    } else {
		unsigned link;

		type_effect(data, type);

		link = miss ? lxtext_data - lxtext_link : ~0;

		if (node) {
		    status = node
			(lxtext_data->lxtype.context,
			 (void *) (effect_data(data)), I_USER(trap),
			 lock_excase(dxline_data));
		    if (status) {
			link = ~0;
		    }
		} else {
		    if (copy) {
			l_USER(side, effect_data(data));
		    } else {
			copy_miss(effect_data(data), type, trap);
		    }
		}

		link_effect(data, link);

		dxline_data->size++;
	    }
	}
    } while (0);

    return status;
}
