/*
 * lxline-i.i.c
 * Copyright (C) 2011-2017, 2021, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <fd.h>
#include <it.h>
#include <l_note.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxline-drive.h>
#include <lxline-inter.h>
#include <lxline-types.h>

#define rule_type			either_type

#define dxline(list) \
    ((struct dxline_type *) (list))
#define lxline(list) \
    ((struct lxline_type *) (list))

static int rule_line(void *, void *);

static int
rule_line(void *line, void *data)
{
    int link, status;

    do {
	struct direct_type *direct_data;
	struct rule_type *rule_data;

	rule_data = line;

	direct_data = rule_data->either;

	direct_data->link = effect_link(data);
	direct_data->type = effect_type(data);

	direct_data->fxdata = *(struct fxdata_type *) effect_data(data);

	link = direct_data->link;
	if (~link) {
	    struct lxtext_type *lxtext_data;

	    lxtext_data = rule_data->lxtext_data + link;

	    if (lxtext_data->lxtype.flags & X1f4_LX_LINK_ACCESS) {
		status = lxtext_data->lxtype.node
		    (lxtext_data->lxtype.context,
		     (void *) &direct_data->fxdata,
		     I_USER(&direct_data->fxdata), rule_data->excase_data);
		if (status) {
		    break;
		} else {
		    rule_data->roll = 1;
		}
	    } else {
		int (*copy)(void *, void *, void **, const void *,
			    const struct x1f4_nodelink_type *);

		copy = lxtext_data->lxtype.copy;
		if (copy) {
		    if (lxtext_data->lxtype.code == X1f4_E4_TEXT) {
			char *down;

			direct_data->link = -1;

			down = I_TEXT(&direct_data->fxdata);
			status = _libx1f4i0_lxline_rule_text
			    (&direct_data->fxdata, down, strlen(down),
			     rule_data);
			if (status) {
			    break;
			} else {
			}
		    } else {
			status = copy
			    (lxtext_data->lxtype.context, NULL,
			     (void *) &direct_data->fxdata,
			     I_USER(&direct_data->fxdata),
			     rule_data->excase_data);
			if (status) {
			    break;
			} else {
			    rule_data->roll = 1;
			}
		    }
		} else {
		    status = 0;
		}
	    }
	} else {
	    status = 0;
	}

	rule_data->either = direct_data + 1;
    } while (0);

    return status;
}


int
_libx1f4i0_lxline_ever_line(struct lxrail_type *lxrail_data,
			    struct direct_type *direct_data, void *dxline,
			    void *fxroll, unsigned *miss)
{
    int status;
    struct rule_type rule;

    rule.lxline_data = lxline(lxrail_data);
    rule.excase_data = lock_excase(dxline);
    rule.lxtext_data = lxline(lxrail_data)->link_f.text;

    rule.either = direct_data;

    rule.roll = 0;
    rule.tune = 0;

    status = _libx1f4i0_lime_sfnote(dxline(dxline)->line, &rule, rule_line);

    if (rule.roll) {
	*miss = (struct direct_type *) rule.either - direct_data;
    }

    return status;
}
