/*
 * lxline-i.k.c
 * Copyright (C) 2011-2017, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <fd.h>
#include <it.h>
#include <l_note.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxline-drive.h>
#include <lxline-inter.h>
#include <lxline-types.h>

#define rule_type			either_type

#define dxline(list) \
    ((struct dxline_type *) (list))
#define lxline(list) \
    ((struct lxline_type *) (list))

extern const void *const x1f4_a1_walk_link;

static int rule_line(void *, void *, const char *, unsigned);

static int
rule_line(void *line, void *data, const char *text, unsigned size)
{
    int link, status;

    do {
	struct affect_type *affect_data;
	struct rule_type *rule_data;

	rule_data = line;

	affect_data = rule_data->either;

	status = _libx1f4i0_lxline_rule_text
	    (&affect_data->fxtext, text, size, rule_data);
	if (status) {
	    break;
	}

	affect_data->link = effect_link(data);
	affect_data->type = effect_type(data);

	affect_data->fxdata = *(struct fxdata_type *) (effect_data(data));

	link = affect_data->link;
	if (~link) {
	    struct lxtext_type *lxtext_data;

	    lxtext_data = rule_data->lxtext_data + link;

	    if (lxtext_data->lxtype.flags & X1f4_LX_LINK_ACCESS) {
		status = lxtext_data->lxtype.node
		    (lxtext_data->lxtype.context,
		     (void *) &affect_data->fxdata,
		     I_USER(&affect_data->fxdata), rule_data->excase_data);
		if (status) {
		    break;
		} else {
		    rule_data->roll = 1;
		}
	    } else {
		int (*copy)(void *, void *, void **, const void *,
			    const struct x1f4_nodelink_type *);

		copy = lxtext_data->lxtype.copy;
		if (copy) {
		    if (lxtext_data->lxtype.code == X1f4_E4_TEXT) {
			char *down;

			affect_data->link = -1;

			down = I_TEXT(&affect_data->fxdata);
			status = _libx1f4i0_lxline_rule_text
			    (&affect_data->fxdata, down, strlen(down),
			     rule_data);
			if (status) {
			    break;
			} else {
			}
		    } else {
			status = copy
			    (lxtext_data->lxtype.context, NULL,
			     (void *) &affect_data->fxdata,
			     I_USER(&affect_data->fxdata),
			     rule_data->excase_data);
			if (status) {
			    break;
			} else {
			    rule_data->roll = 1;
			}
		    }
		} else {
		}
	    }
	} else {
	}

	rule_data->either = affect_data + 1;
    } while (0);

    return status;
}


int
_libx1f4i0_lxline_will_line(struct lxrail_type *lxline_data,
			    struct affect_type *affect_data, void *dxline,
			    unsigned *miss)
{
    int status;
    struct rule_type rule;

    rule.excase_data = lock_excase(dxline);
    rule.lxline_data = (void *) lxline_data;
    rule.lxtext_data = lxline(lxline_data)->link_f.text;

    rule.either = affect_data;

    rule.roll = 0;
    rule.tune = 0;

    status = _libx1f4i0_span_sfnote(dxline(dxline)->line, &rule, rule_line);

    if (rule.roll) {
	*miss = (struct affect_type *) rule.either - affect_data;
    }

    return status;
}
