/*
 * lxline-i.r.c
 * Copyright (C) 2011-2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include <lxline-drive.h>
#include <lxline-inter.h>
#include <lxline-types.h>

extern const char *const x1f4_c1_empty_string;

int
_libx1f4i0_lxline_rule_text(void *land, const char *text, unsigned size,
			    struct either_type *either_data)
{
    int status;

    if (size) {
	unsigned tune;

	size++;

	tune = either_data->tune;
	if (tune < size || 512 < size) {
	    struct lxline_type *lxline_data;

	    if (512 < size) {
		lxline_data = either_data->lxline_data;

		status = lxline_data->link_v.link
		    (lxline_data->link_v.data, land, size);
		if (status) {
		    status = _libx1f4i0_lxline_stat_link(lxline_data);
		} else {
		    memcpy(I_TEXT(land), text, size);
		}
	    } else {
		lxline_data = either_data->lxline_data;

		status = lxline_data->link_v.link
		    (lxline_data->link_v.data, &either_data->pass, 4096);
		if (status) {
		    status = _libx1f4i0_lxline_stat_link(lxline_data);
		} else {
		    char *pass;

		    pass = either_data->pass;

		    memcpy(pass, text, size);

		    either_data->pass = pass + size;
		    either_data->tune = 4096 - size;

		    l_TEXT(pass, land);
		}
	    }
	} else {
	    char *pass;

	    status = 0;

	    pass = either_data->pass;

	    memcpy(pass, text, size);

	    either_data->pass = pass + size;
	    either_data->tune = tune - size;

	    l_TEXT(pass, land);
	}
    } else {
	status = 0;

	l_TEXT((void *) x1f4_c1_empty_string, land);
    }

    return status;
}
