/*
 * lxline-l.j.c
 * Copyright (C) 2008-2011, 2013, 2014, 2016-2018, 2024, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include <e4.h>
#include <l_note.h>
#include <lxcall.h>
#include <lxline-drive.h>
#include <lxline-inter.h>
#include <lxline-types.h>

int
_libx1f4i0_lxline_j_logique(void *context, void *output, void **input)
{
    int status;
    struct dxline_type *dxline_data, *dxline_text;

    dxline_data = I_LINE(input[0]);
    dxline_text = I_LINE(input[1]);

    do {
	const char *fold, *hike;
	unsigned mind, slip;
	void *ever, *lead, *way_red[4];

	if (dxline_data == dxline_text) {
	    status = 0;
	    break;
	}
	status = _libx1f4i0_hike_sfnote
	    (dxline_text->line, &lead, &hike, &mind);
	if (status) {
	    if (status == X1f4_SFNOTE_DECK_ERROR) {
	    } else {
		status = _libx1f4i0_lxline_stat_line(context, 0, status);
		break;
	    }
	} else {
	    status = _libx1f4i0_fold_sfnote
		(dxline_data->line, &ever, &fold, &slip);
	    if (status) {
		if (status == X1f4_SFNOTE_DECK_ERROR) {
		} else {
		    status = _libx1f4i0_lxline_stat_line(context, 0, status);
		    break;
		}
	    } else {
		int join;

		join = memcmp(fold, hike, mind < slip ? mind : slip);
		if (join < 0 || !join && slip < mind) {
		} else {
		    status = _libx1f4i0_lxline_stat_join(context);
		    break;
		}
	    }
	}

	way_red[1] = context;
	way_red[2] = lock_excase(dxline_text);
	way_red[3] = lock_excase(dxline_data);

	status = _libx1f4i0_lime_sfnote
	    (dxline_text->line, way_red, _libx1f4i0_lxrail_move_rail);
	if (status) {
	} else {
	    status = _libx1f4i0_join_sfnote
		(dxline_data->line, dxline_text->line);
	    if (status) {
	    } else {
		dxline_data->size += dxline_text->size;
		dxline_text->size = 0;
	    }
	}
    } while (0);

    if (status) {
    } else {
	l_LINE(dxline_data, output);
    }

    return status;
}
