/*
 * lxline-l.r.c
 * Copyright (C) 2008-2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <l_note.h>
#include <lxline-drive.h>
#include <lxline-inter.h>
#include <lxline-types.h>

#if __HAVE_SHUT_SFNOTE__
static int shut_note(void *, void *);
#endif				/* __HAVE_SHUT_SFNOTE__ */

#if __HAVE_SHUT_SFNOTE__
static int
shut_note(void *text, void *data)
{
    int status = 0;
    unsigned link;

    link = effect_link(data);
    if (~link) {
	int (*free)(void *, void *, struct excase_type **,
		    const struct excase_type *);
	struct lxtext_type *lxtext_link;

	lxtext_link =
	    ((struct lxtext_type *)
	     ((struct lxline_type *) *(void **) text)->link_f.text) + link;

	link_effect(data, ~0);

	free = lxtext_link->lxtype.free;
	if (free) {
	    status = free
		(lxtext_link->lxtype.context, E_USER(data), NULL,
		 lock_excase(*((void **) text + 1)));
	}
    }

    return status;
}
#endif				/* __HAVE_SHUT_SFNOTE__ */


int
_libx1f4i0_lxline_r_logique(void *context, void *output, void **input)
{
    X1f4_E4_C_TEXT fail;
    int status;
    struct dxline_type *dxline_data;
#if __HAVE_SHUT_SFNOTE__
#else
    void *data;
#endif				/* __HAVE_SHUT_SFNOTE__ */
#if __HAVE_SHUT_SFNOTE__
    void *text[2];
#endif				/* __HAVE_SHUT_SFNOTE__ */

    dxline_data = I_LINE(input[0]);

    fail = I_TEXT(input[1]);

#if __HAVE_SHUT_SFNOTE__
    text[0] = context;
    text[1] = dxline_data;
#endif				/* __HAVE_SHUT_SFNOTE__ */

#if __HAVE_SHUT_SFNOTE__
    status = _libx1f4i0_shut_sfnote(dxline_data->line, fail, text, shut_note);
    if (status) {
	if (status == X1f4_SFNOTE_MISS_CLASS) {
	    status = 0;

	    l_LINE(dxline_data, output);
	} else {
# if 0
	    if (status == X1f4_SFNOTE_CALL_ERROR) {
		status = X1f4_EX_CRITICAL;
	    } else {
# endif				/* 0 */
		status = _libx1f4i0_lxline_stat_line(context, 0, status);
# if 0
	    }
# endif				/* 0 */
	}
    } else {
	dxline_data->size--;

	l_LINE(dxline_data, output);
    }
#else
    status = _libx1f4i0_find_sfnote(dxline_data->line, fail, &data);
    if (status) {
	do {
	    struct lxline_type *lxline_data;
	    unsigned link;

	    status = 0;

	    lxline_data = context;

	    link = effect_link(data);
	    if (~link) {
		int (*free)(void *, void *, struct excase_type **,
			    const struct excase_type *);
		struct lxtext_type *lxtext_link;

		lxtext_link =
		    ((struct lxtext_type *) lxline_data->link_f.text) + link;

		link_effect(data, ~0);

		free = lxtext_link->lxtype.free;
		if (free) {
		    status = free
			(lxtext_link->lxtype.context, E_USER(data),
			 NULL, lock_excase(dxline_data));
		    if (status) {
			break;
		    }
		}
	    }

	    status = _libx1f4i0_pass_sfnote(dxline_data->line, fail);
	    if (status) {
		status = _libx1f4i0_lxline_stat_line(lxline_data, 0, status);
	    } else {
		dxline_data->size--;

		l_LINE(dxline_data, output);
	    }
	} while (0);
    } else {
	l_LINE(dxline_data, output);
    }
#endif				/* __HAVE_SHUT_SFNOTE__ */

    return status;
}
