/*
 * lxline-t.e-c.c
 * Copyright (C) 2008-2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxline-config.h>

#include <e4.h>
#include <l_note.h>
#include <lxline-drive.h>
#include <lxline-inter.h>
#include <lxline-types.h>

#if __HAVE_SHUT_SFNOTE__
static int shut_note(void *, void *);
#else
# error not fixed for -c
#endif				/* __HAVE_SHUT_SFNOTE__ */

#if __HAVE_SHUT_SFNOTE__
static int
shut_note(void *text, void *data)
{
    int status, type;
    struct lxtext_type *lxtext_data;

    lxtext_data = *(void **) text;

    type = effect_type(data);
    status = _libx1f4i0_lxrail_yank_text
	(__types_rail(lxtext_data->text), *((void **) text + 4), lxtext_data,
	 &type, &effect_link(data), effect_data(data), *((void **) text + 1),
	 **((void ***) text + 2));
    if (status) {
	int *delete;

	delete = *((void **) text + 3);
	if (type != X1f4_E4_TEXT) {
	    status = _libx1f4i0_lxline_stat_type
		(lxtext_data->text, I_TEXT((*((void ***) text + 2))[2]), type,
		 X1f4_E4_TEXT);
	}

	*delete = status;
    }

    return status;
}
#endif				/* __HAVE_SHUT_SFNOTE__ */


int
_libx1f4i0_lxline_e_press_t(void *context, void *output, void **input)
{
    X1f4_E4_C_TEXT fail;
#if __HAVE_SHUT_SFNOTE__
    int excess;
#endif				/* __HAVE_SHUT_SFNOTE__ */
    int status;
    struct dxline_type *dxline_data;
    struct lxtext_type *lxtext_data;
#if __HAVE_SHUT_SFNOTE__
#else
    void *data;
#endif				/* __HAVE_SHUT_SFNOTE__ */
#if __HAVE_SHUT_SFNOTE__
    void *text[5];
#endif				/* __HAVE_SHUT_SFNOTE__ */

    lxtext_data = context;

    dxline_data = I_LINE(input[1]);

    fail = I_TEXT(input[2]);

#if __HAVE_SHUT_SFNOTE__
    text[0] = lxtext_data;
    text[1] = output;
    text[2] = input;
    text[3] = &excess;
    text[4] = dxline_data;
#endif				/* __HAVE_SHUT_SFNOTE__ */

#if __HAVE_SHUT_SFNOTE__
    status = _libx1f4i0_shut_sfnote(dxline_data->line, fail, text, shut_note);
    if (status) {
	if (status == X1f4_SFNOTE_MISS_CLASS) {
	    status = _libx1f4i0_lxline_stat_miss(lxtext_data->text, fail);
	} else {
	    if (status == X1f4_SFNOTE_CALL_ERROR) {
		status = excess;
	    } else {
		status = _libx1f4i0_lxline_stat_line
		    (lxtext_data->text, 0, status);
	    }
	}
    } else {
	dxline_data->size--;
    }
#else
    status = _libx1f4i0_find_sfnote(dxline_data->line, fail, &data);
    if (status) {
	unsigned type;

	type = effect_type(data);
	if (type != X1f4_E4_TEXT) {
	    status = _libx1f4i0_lxline_stat_type
		(lxtext_data->text, fail, type, X1f4_E4_TEXT);
	} else {
	    status = lxtext_data->lxtype.push
		(lxtext_data->lxtype.context, input[0],
		 E_TEXT(data), lock_excase(dxline_data));
	    if (status) {
	    } else {
		link_effect(data, ~0);

		status = _libx1f4i0_pass_sfnote(dxline_data->line, fail);
		if (status) {
		    status = _libx1f4i0_lxline_stat_line
			(lxtext_data->text, 0, status);
		} else {
		    dxline_data->size--;

		    l_TEXT(I_TEXT(*input), output);
		}
	    }
	}
    } else {
	status = _libx1f4i0_lxline_stat_miss(lxtext_data->text, fail);
    }
#endif				/* __HAVE_SHUT_SFNOTE__ */

    return status;
}
