/*
 * lxlist-c.r.c
 * Copyright (C) 2013-2015, 2017-2019, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>

#define SCALAR_BITS \
    ((1 << X1f4_E4_BILL) | (1 << X1f4_E4_MODE) | (1 << X1f4_E4_REAL))

int
_libx1f4i0_lxlist_r_centric(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE mode;
    int status;
    struct dxlist_type *dxlist_data;
    struct lxlist_type *lxlist_data;
    struct x1f4_function_type *function_data;
    unsigned count, size;
    void **strip;

    lxlist_data = context;

    dxlist_data = I_LIST(input[0]);

    mode = I_MODE(input[1]);

    strip = input[-1];

    function_data = strip[X1f4_E4_RENDITION];

    count = function_data->count - 2;

    size = dxlist_data->size + 1;
    if (mode < 0
	? (X1f4_E4_C_MODE) (mode - count) < -(X1f4_E4_C_MODE) (int) size
	: !((X1f4_E4_C_MODE) (mode + count) < (X1f4_E4_C_MODE) (int) size)) {
	if (count) {
	    status = _libx1f4i0_lxlist_stat_reap
		(lxlist_data, size, mode, count);
	} else {
	    status = _libx1f4i0_lxlist_stat_deck(lxlist_data, size, mode);
	}
    } else {
	status = 0;

	if (count) {
	    const int *just;
	    struct x1f4_function_type function;
	    void *slip[4];

	    slip[0] = strip;
	    slip[2] = *input;
	    slip[3] = &mode;

	    input++;

	    /*
	     * pretty error reporting:
	     */
	    function.length = 6;
	    function.name = "l_reap";
	    function.type = X1f4_E4_VOID;

	    function.args = just = function_data->args + 2;

	    just += count + 2;

	    strip[X1f4_E4_RENDITION] = &function;

	    for (; count; count--) {
		int type;

		type = *function.args;
		if (type < 16 && (1 << type) & SCALAR_BITS
		    || *just & X1f4_E4_POST_XSET) {
		    slip[1] = *++input;
		    status = _libx1f4i0_lxlist_y_derived
			(lxlist_data, output, slip + 1);
		    if (status) {
			break;
		    }

		    just++;
		    function.args++;
		} else {
		    status = _libx1f4i0_lxlist_stat_rest
			(lxlist_data, function_data->count - count);
		    if (1) {
			break;
		    }
		}
	    }
	}

	if (status) {
	} else {
	    l_LIST(dxlist_data, output);
	}
    }

    return status;
}
