/*
 * lxlist-d.p.c
 * Copyright (C) 2008-2011, 2013-2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <l_list.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>
#include <lxrail.h>
#include <retain.h>

#if __HAVE_PULL_L4LIST__
# define clip(none) \
    ((struct clip_type *) (none))
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
typedef struct clip_type {
    int status;
    struct dxlist_type *dxlist_data;
    struct lxlist_type *lxlist_data;
    void *output;
} clip_type;
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
static int clip_miss(void *, void *);
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
static int
clip_miss(void *clip, void *data)
{
    int status;
    struct lxlist_type *lxlist_data;
    unsigned link;

    lxlist_data = clip(clip)->lxlist_data;

    status = _libx1f4i0_lxrail_post_data
	(lxlist_data, clip(clip)->output, effect_link(data), effect_type(data),
	 effect_data(data));
    if (status) {
	clip(clip)->status = status;
    } else {
	link = effect_link(data);
	if (~link) {
	    struct lxtext_type *lxtext_data;

	    link_effect(data, ~0);

	    lxtext_data = lxlist_data->link_f.text;
	    lxtext_data += link;
	    if (lxtext_data->lxtype.free) {
		status = lxtext_data->lxtype.free
		    (lxtext_data->lxtype.context, E_USER(data), NULL,
		     lock_excase(clip(clip)->dxlist_data));
		if (status) {
		    clip(clip)->status = status;
		}
	    }
	}
    }

    return status;
}
#endif				/* __HAVE_PULL_L4LIST__ */


int
_libx1f4i0_lxlist_p_derived(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE mode;
    int status;
    struct dxlist_type *dxlist_data;
    struct lxlist_type *lxlist_data;
    unsigned size;

    dxlist_data = I_LIST(input[0]);

    lxlist_data = context;

    mode = I_MODE(input[1]);

    size = dxlist_data->size;
    if (mode < -(X1f4_E4_C_MODE) (int) size
	|| !(mode < (X1f4_E4_C_MODE) (int) size)) {
	status = _libx1f4i0_lxlist_stat_deck(lxlist_data, size, mode);
    } else {
#if __HAVE_PULL_L4LIST__
	struct clip_type clip;
#else
	void *data, *list;
#endif				/* __HAVE_PULL_L4LIST__ */

	if (mode < 0) {
	    mode += size;
	}

#if __HAVE_PULL_L4LIST__
	clip.dxlist_data = dxlist_data;
	clip.lxlist_data = context;
	clip.output = output;
#else
	list = dxlist_data->list;
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
	status = _libx1f4i0_pull_l4list
	    (dxlist_data->list, mode, &clip, clip_miss);
	if (status) {
	    status = _libx1f4i0_lxlist_stat_pull(context, clip.status, status);
	} else {
	    dxlist_data->size = size - 1;
	}
#else
	status = _libx1f4i0_peek_l4list(list, mode, &data);
	if (status) {
	    status = _libx1f4i0_lxlist_stat_list(context, 0, status);
	} else {
	    status = _libx1f4i0_lxrail_post_data
		(lxlist_data, output, effect_link(data), effect_type(data),
		 effect_data(data));
	    if (status) {
	    } else {
# if __RETAIN_OBJECT_DATA__
#  error not considered
# else
		unsigned link;

		link = effect_link(data);
		if (~link) {
		    struct lxtext_type *lxtext_data;

		    link_effect(data, ~0);

		    lxtext_data = lxlist_data->link_f.text;
		    lxtext_data += link;
		    if (lxtext_data->lxtype.lead) {
			status = lxtext_data->lxtype.free
			    (lxtext_data->lxtype.context, E_USER(data),
			     NULL, lock_excase(dxlist_data));
		    } else {
		    }
		}

		if (status) {
		} else {
		    status = _libx1f4i0_miss_l4list(list, mode);
		    if (status) {
			status = _libx1f4i0_lxlist_stat_list
			    (lxlist_data, 0, status);
		    } else {
			dxlist_data->size = size - 1;
		    }
		}
# endif				/* __RETAIN_OBJECT_DATA__ */
	    }
	}
#endif				/* __HAVE_PULL_L4LIST__ */
    }

    return status;
}
