/*
 * lxlist-d.s.c
 * Copyright (C) 2008-2011, 2013, 2014, 2016-2018, 2024, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <l_list.h>
#include <lxcall.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

int
_libx1f4i0_lxlist_s_derived(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE mode;
    int status;
    struct dxlist_type *dxlist_data, *dxlist_text;
    unsigned size;

    dxlist_data = I_LIST(input[0]);
    dxlist_text = I_LIST(input[2]);

    mode = I_MODE(input[1]);

    size = dxlist_data->size;

    if (mode < -(X1f4_E4_C_MODE) (int) (size + 1)
	|| (X1f4_E4_C_MODE) (int) size < mode) {
	status = _libx1f4i0_lxlist_stat_rend
	    (context, size, mode, "split point");
    } else {
	void *dxtext;

	if (mode < 0) {
	    mode += size + 1;
	}

	status = _libx1f4i0_lxlist_line_list(context, &dxtext);
	if (status) {
	} else {
	    status = _libx1f4i0_tear_l4list
		(dxlist_data->list, mode,
		 ((struct dxlist_type *) dxtext)->list);
	    if (status) {
	    } else {
		void *way_red[4];

		way_red[1] = context;
		way_red[2] = lock_excase(dxlist_data);
		way_red[3] = lock_excase(dxlist_text);

		status = _libx1f4i0_lime_l4list
		    (((struct dxlist_type *) dxtext)->list, way_red,
		     _libx1f4i0_lxrail_move_rail);
		if (status) {
		} else {
		    dxlist_data->size = mode;
		    ((struct dxlist_type *) dxtext)->size = size - mode;

		    status = _libx1f4i0_lxlist_swap_list(dxtext, dxlist_text);
		}
	    }

	    if (1) {
		int excess;

		excess = _libx1f4i0_lxlist_flat_list(context, &dxtext);
		if (excess) {
		    status = _libx1f4i0_lxcall_land_slip(excess, status);
		}
	    }
	}
    }

    if (status) {
    } else {
	l_LIST(dxlist_data, output);
    }

    return status;
}
