/*
 * lxlist-i.i.c
 * Copyright (C) 2011-2019, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <fd.h>
#include <it.h>
#include <l_list.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

#define dxlist(list) \
    ((struct dxlist_type *) (list))
#define fxroll(roll) \
    ((struct fxroll_type *) (roll))
#define lxlist(list) \
    ((struct lxlist_type *) (list))

typedef struct rule_type {
    int roll;
    struct excase_type *excase_data;
    struct direct_type *direct_data;
    struct lxtext_type *lxtext_data;
} rule_type;

extern const void *const x1f4_a1_walk_link;

static int near_list(void *, void *);
static int rule_list(void *, void *);

static int
near_list(void *line, void *data)
{
    struct direct_type *direct_data;
    struct rule_type *rule_data;

    rule_data = line;

    direct_data = rule_data->direct_data;

    direct_data->type = effect_type(data);
    direct_data->fxdata = *(struct fxdata_type *) effect_data(data);

    rule_data->direct_data = direct_data + 1;

    return 0;
}


static int
rule_list(void *line, void *data)
{
    int link, status;
    struct direct_type *direct_data;
    struct rule_type *rule_data;

    rule_data = line;

    direct_data = rule_data->direct_data;

    link = effect_link(data);

    direct_data->type = effect_type(data);
    direct_data->link = link;

    if (~link) {
	struct lxtext_type *lxtext_data;

	lxtext_data = rule_data->lxtext_data + link;

	if (lxtext_data->lxtype.flags & X1f4_LX_LINK_ACCESS) {
	    status = lxtext_data->lxtype.node
		(lxtext_data->lxtype.context, (void *) &direct_data->fxdata,
		 E_USER(data), rule_data->excase_data);
	    if (status) {
	    } else {
		rule_data->roll = 1;
		rule_data->direct_data = direct_data + 1;
	    }
	} else {
	    int (*copy)(void *, void *, void **, const void *,
			const struct x1f4_nodelink_type *);

	    copy = lxtext_data->lxtype.copy;
	    if (copy) {
		status = copy
		    (lxtext_data->lxtype.context, NULL,
		     (void *) &direct_data->fxdata, E_USER(data),
		     rule_data->excase_data);
		if (status) {
		} else {
		    rule_data->roll = 1;
		    rule_data->direct_data = direct_data + 1;
		}
	    } else {
		status = 0;

		direct_data->fxdata =
		    *(struct fxdata_type *) effect_data(data);
		rule_data->direct_data = direct_data + 1;
	    }
	}
    } else {
	status = 0;

	direct_data->fxdata = *(struct fxdata_type *) effect_data(data);
	rule_data->direct_data = direct_data + 1;
    }

    return status;
}


int
_libx1f4i0_lxlist_ever_list(struct lxrail_type *lxlist_data,
			    struct direct_type *direct_data, void *dxlist,
			    void *fxroll, unsigned *miss)
{
    int head, (*pick)(void *, void *), status, tail;
    struct rule_type rule;

    rule.excase_data = lock_excase(dxlist);
    rule.direct_data = direct_data;
    rule.lxtext_data = lxlist(lxlist_data)->link_f.text;

    rule.roll = 0;

    head = fxroll(fxroll)->head;
    tail = fxroll(fxroll)->tail;

    pick = fxroll(fxroll)->copy ? rule_list : near_list;

    if (tail < head + fxroll(fxroll)->infer) {
	unsigned size;

	size = dxlist(dxlist)->size;
	if (head < size) {
	    status = _libx1f4i0_over_l4list
		(dxlist(dxlist)->list, head, size - 1, &rule, pick);
	} else {
	    status = 0;
	}
	if (status) {
	} else {
	    if (tail) {
		status = _libx1f4i0_over_l4list
		    (dxlist(dxlist)->list, 0, tail - 1, &rule, pick);
	    }
	}
    } else {
	if (head ^ tail) {
	    status = _libx1f4i0_over_l4list
		(dxlist(dxlist)->list, head, tail - 1, &rule, pick);
	} else {
	    status = 0;
	}
    }

    if (rule.roll) {
	*miss = rule.direct_data - direct_data;
    }

    return status;
}
