/*
 * lxlist-u.o.c
 * Copyright (C) 2008-2011, 2013, 2015-2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxlist-config.h>

#include <e4.h>
#include <l_list.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

#if __HAVE_PULL_L4LIST__
# define clip(none) \
    ((struct clip_type *) (none))
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
typedef struct clip_type {
    int type;
    void *effect;
} clip_type;
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
static int clip_miss(void *, void *);
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
static int
clip_miss(void *clip, void *data)
{
    int status;
    unsigned type;

    type = effect_type(data);
    if (type ^ clip(clip)->type) {
	status = 1;
    } else {
	status = 0;

	l_USER(E_USER(data), clip(clip)->effect);
    }

    return status;
}
#endif				/* __HAVE_PULL_L4LIST__ */


int
_libx1f4i0_lxlist_o_class_u(void *context, void *output, void **input)
{
    int status;
    struct dxlist_type *dxlist_data;
    struct lxtext_type *lxtext_data;

    lxtext_data = context;

    dxlist_data = I_LIST(input[1]);
    if (0) {
    } else {
	X1f4_E4_C_MODE mode;
	unsigned size;

	mode = I_MODE(input[2]);

	size = dxlist_data->size;
	if (mode < -(X1f4_E4_C_MODE) (int) size
	    || !(mode < (X1f4_E4_C_MODE) (int) size)) {
	    l_MODE(0, output);

	    status = 0;
	} else {
#if __HAVE_PULL_L4LIST__
	    struct clip_type clip;
#else
	    void *data;
#endif				/* __HAVE_PULL_L4LIST__ */

	    if (mode < 0) {
		mode += size;
	    }

#if __HAVE_PULL_L4LIST__
	    clip.effect = input[0];
	    clip.type = lxtext_data->lxtype.code;
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
	    status = _libx1f4i0_pull_l4list
		(dxlist_data->list, mode, &clip, clip_miss);
	    if (status) {
		if (status == X1f4_L4LIST_CALL_ERROR) {
		    l_MODE(0, output);

		    status = 0;
		} else {
		    status = _libx1f4i0_lxlist_stat_list
			(lxtext_data->text, mode, status);
		}
	    } else {
		l_MODE(1, output);

		dxlist_data->size = size - 1;
	    }
#else
	    status = _libx1f4i0_peek_l4list(dxlist_data->list, mode, &data);
	    if (status) {
		status =
		    _libx1f4i0_lxlist_stat_list(lxtext_data->text, 0, status);
	    } else {
		unsigned type;

		type = effect_type(data);
		if (type != lxtext_data->lxtype.code) {
		    l_MODE(0, output);

		    status = 0;
		} else {
		    X1f4_E4_C_USER fast;

		    fast = E_USER(data);

		    status = _libx1f4i0_miss_l4list(dxlist_data->list, mode);
		    if (status) {
			status = _libx1f4i0_lxlist_stat_list
			    (lxtext_data->text, 0, status);
		    } else {
			dxlist_data->size = size - 1;

			l_USER(fast, input[0]);

			l_MODE(1, output);
		    }
		}
	    }
#endif				/* __HAVE_PULL_L4LIST__ */
	}
    }

    return status;
}
