/*
 * lxrail-types.h
 * Copyright (C) 2016-2019, 2021, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxrail_types_H__
#define __LIBx1f4_lxrail_types_H__

#include <e4.h>
#include <fd.h>
#include <it.h>
#include <e4.h>
#include <lxplus.h>
#include <nt.h>

#define lxtype_type                     x1f4_nodetype_type

#define excase_type			x1f4_nodelink_type

#define cxlist_type			x1f4_leadnode_type

#define here_NULL			((void *) 0)

#define lock_excase(dxlist_text) \
    ((struct excase_type *) &lock_cxlist((dxlist_text))->link)

#define lock_cxlist(dxlist_text) \
    ((struct cxlist_type *)						      \
     ((char *) (dxlist_text) - sizeof(struct cxlist_type)))

typedef struct affect_type {
    int link, type;
    struct fxdata_type fxdata, fxtext;
} affect_type;

typedef struct cxroll_type {
    struct direct_type *direct_data;
    unsigned *miss;
} cxroll_type;

typedef struct depart_type {
    struct x1f4_fpdetail_type *fpdetail_alaw, *fpdetail_blaw, *fpdetail_claw,
	*fpdetail_dlaw, *fpdetail_flaw, *fpdetail_glaw, *fpdetail_llaw,
	*fpdetail_plaw, *fpdetail_qlaw, *fpdetail_rlaw, *fpdetail_slot,
	*fpdetail_xlaw, *fpdetail_ylaw;
    struct x1f4_ftaa_type *ftaa_qlaw, *ftaa_xlaw;
    struct x1f4_ftca_type *ftca_qlaw, *ftca_xlaw;
    struct x1f4_ftsl_type *ftsl_alaw, *ftsl_blaw, *ftsl_claw, *ftsl_dlaw,
	*ftsl_flaw, *ftsl_glaw, *ftsl_llaw, *ftsl_plaw, *ftsl_rlaw, *ftsl_ylaw;
    const void *screen_data;
} depart_type;

typedef struct direct_type {
    int link, type;
    struct fxdata_type fxdata;
} direct_type;

typedef struct fydata_type {
    union {
	X1f4_E4_C_MODE mode;
	X1f4_E4_C_TEXT text;
    } data;
} fydata_type;

typedef struct lxmiss_type {
    union {
	unsigned miss;
	void *data;
    } data;
} lxmiss_type;

typedef struct lxtext_type {
    struct lxtype_type lxtype;
    void *text;
} lxtext_type;

typedef struct affine_type {
    struct x1f4_nodelink_type *nodelink_data;
    struct lxtext_type *lxtext_data;
    unsigned miss, size;
} affine_type;

typedef struct lxrail_type {
    struct lxplus_type ever_p;
    struct x1f4_iturn_type link_p;
    struct x1f4_composer_type link_q;
    struct x1f4_scope_type link_x;
} lxrail_type;

typedef struct patent_type {
    const char *name;
    unsigned size;
} patent_type;

#endif				/* __LIBx1f4_lxrail_types_H__ */
