/*
 * lxroll-d.c.c
 * Copyright (C) 2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <netdb.h>
#include <stddef.h>
#include <unistd.h>

#include <netinet/tcp.h>

#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>

#include <e4.h>
#include <lxroll-defs.h>
#include <lxroll-drive.h>
#include <lxroll-inter.h>
#include <lxroll-types.h>

static int case_unix(void *, const char *, int);

static int
case_unix(void *lxroll, const char *path, int connection)
{
    int status;
    struct sockaddr_un address;

    memset(&address, 0, sizeof(struct sockaddr_un));
    address.sun_family = AF_UNIX;
    strcpy(address.sun_path, path);

    if (1) {
	status = connect
	    (connection, (struct sockaddr *) &address, sizeof(address));
	if (status) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll, "connect");
	}
    }

    return status;
}


int
_libx1f4i0_lxroll_c_derived(void *context, void *output, void **input)
{
    int fd = -1, ld = -1, status;

    do {
	const char *path;
        struct lxroll_type *lxroll_data;
        struct x1f4_dxfile_type *dxfile_data, *dxfile_text;
	unsigned bits;

        lxroll_data = context;

	path = I_TEXT(input[2]);

	if (strlen(path) < sizeof(((struct sockaddr_un *) 0)->sun_path)) {
	} else {
	    status = _libx1f4i0_lxroll_stat_fill(lxroll_data);
	    if (1) {
		break;
	    }
	}

        dxfile_data = I_FILE(input[0]);

        dxfile_text = I_FILE(input[1]);

	if (dxfile_data == dxfile_text) {
	    status = _libx1f4i0_lxroll_stat_back(lxroll_data);
	    if (1) {
		break;
	    }
	}

	/*
	 * connect(2) before closing any files.
	 */

	fd = socket(AF_UNIX, SOCK_STREAM, 0);
	if (fd == -1) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll_data, "socket");
            if (1) {
                break;
            }
	}

	bits = I_MODE(input[3]);

	/*
	 * possibly making no sense:
	 */
	if (bits & TCP_NO_DELAY) {
	    int part = 1;

	    status = setsockopt
		(fd, IPPROTO_TCP, TCP_NODELAY, &part, sizeof(int));
	    if (status) {
		status = _libx1f4i0_lxroll_stat_part
		    (lxroll_data, TCP_NO_DELAY);
		if (1) {
		    break;
		}
	    }
	}

	status = case_unix(lxroll_data, path, fd);
	if (status) {
	    break;
	}

	ld = dup(fd);
	if (ld == -1) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll_data, "dup");
            if (1) {
                break;
            }
	}

	status = _libx1f4i0_lxroll_land_roll
	    (lxroll_data, bits, dxfile_data, &fd, dxfile_text, &ld);
    } while (0);

    if (fd ^ -1) {
	close(fd);
    }

    if (ld ^ -1) {
	close(ld);
    }

    return status;
}
