/*
 * lxtrap-a.e.c
 * Copyright (C) 2011-2015, 2017-2020, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <er.h>
#include <exerrors.h>
#include <lxtrap-inter.h>
#include <lxtrap-types.h>

#define lxtrap(lxtrap) \
    ((struct lxtrap_type *) (lxtrap))

int
_libx1f4i0_lxtrap_stat_call(void *lxtrap, int rank)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("non function pointer ", 21);
    PUSH_MODE(rank + 1);
    PUSH_RANK(rank);
    PUSH_DATA(" argument", 9);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_cast(void *lxtrap)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("will not set data of the `object' type", 38);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_even(void *lxtrap)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("even number of parameters", 25);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_fill(void *lxtrap, int pull)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_TEXT(pull < 0 ? "negative" : "null");
    PUSH_DATA(" switch count (", 15);
    PUSH_MODE(pull);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_hull(void *lxtrap, unsigned pull, unsigned line)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("missing insert positions, found ", 32);
    PUSH_BILL(line);
    PUSH_DATA(", expecting ", 12);
    PUSH_BILL(pull);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_iran(void *lxtrap, int rank)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("bad ", 4);
    PUSH_MODE(rank + 1);
    PUSH_RANK(rank);
    PUSH_DATA(" argument type, integer expected", 32);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_line(void *lxtrap, int fine, int land)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_TEXT(fine < 0 ? "negative fixed" : "too big");
    PUSH_DATA(" arguments count (", 18);
    PUSH_MODE(fine);
    PUSH_DATA(" in ", 4);
    PUSH_BILL(land);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_miss(void *lxtrap, int first, int second)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("unmatched output types (store is ", 33);
    PUSH_TRAP(first, lxtrap(lxtrap));
    PUSH_DATA(", result is ", 12);
    PUSH_TRAP(second, lxtrap(lxtrap));
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_null(void *lxtrap)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("function takes no parameters", 28);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_odds(void *lxtrap)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("unpaired last argument", 22);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_over(void *lxtrap)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("too many arguments", 18);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_post(void *lxtrap, int first, int second)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("unmatched output types (store is ", 33);
    PUSH_TRAP(first, lxtrap(lxtrap));
    PUSH_DATA(", function is ", 14);
    PUSH_TRAP(second, lxtrap(lxtrap));
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_pull(void *lxtrap, unsigned pull, unsigned line)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("bad number of switched (", 24);
    PUSH_BILL(pull);
    PUSH_DATA(") arguments (", 13);
    PUSH_BILL(line);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_rise(void *lxtrap)
{
    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_sell(void *lxtrap, int sell)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("too large the number of applied arguments (", 43);
    PUSH_MODE(sell);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_sink(void *lxtrap)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("zero or less repeat", 19);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_slip(void *lxtrap, unsigned size, long mode)
{
    if (size) {
	_libx1f4i0_lxtrap_stat_time(lxtrap, size, mode);
    } else {
	_libx1f4i0_lxtrap_stat_null(lxtrap);
    }

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_spin(void *lxtrap, int spin)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("non function pointer ", 21);
    PUSH_BILL(spin + 1);
    PUSH_RANK(spin);
    PUSH_DATA(" argument", 9);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxtrap_stat_time(void *lxtrap, unsigned size, long mode)
{
    LINE_EEER(lxtrap(lxtrap)->link_e);

    PUSH_DATA("out of valid range (-", 21);
    PUSH_BILL(size);
    PUSH_DATA(" .. ", 4);
    PUSH_BILL(size - 1);
    PUSH_DATA(") index (", 9);
    PUSH_LONG(mode);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}

