/*
 * lxtrap-f.c.c
 * Copyright (C) 2011-2015, 2017, 2019, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include <e4.h>
#include <e4less.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxtrap-drive.h>
#include <lxtrap-inter.h>
#include <lxtrap-types.h>

#define POST_BITS			X1f4_LXCALL_POST_WAYS

typedef struct slip_type {
    int (*call)(void *, int, void *, struct x1f4_dispatch_type **);
    struct covert_type covert;
    struct x1f4_dispatch_type *dispatch_data;
    unsigned loop;
    void *back, *rendition, **used;
} slip_type;

extern const void *const x1f4_a1_walk_link;

static int part_call(void *, int, void *, struct x1f4_dispatch_type **);
static int pick_call(struct x1f4_linetext_type *, unsigned, void *, void **,
		     struct lxtrap_type *);
static int slip_call(struct x1f4_linetext_type *, unsigned, void **, void *,
		     void **, struct lxtrap_type *);

static int
part_call(void *back, int status, void *output,
	  struct x1f4_dispatch_type **dispatch)
{
    struct slip_type *call_data;

    call_data = back;

    do {
	unsigned slip;

	if (status) {
	    break;
	}

	slip = call_data->loop;
	if (slip) {
	    struct x1f4_dispatch_type *dispatch_data;

	    dispatch_data = call_data->dispatch_data;

	    call_data->loop = slip - 1;

	    if (call_data->covert.swap) {
		status = call_data->covert.scope_data->deck
		    (&call_data->covert.frame_data->data);
		if (status) {
		    break;
		}
	    }

	    dispatch_data->back = call_data;
	    dispatch_data->call = part_call;

	    *dispatch = dispatch_data;

	    call_data->used[X1f4_E4_RENDITION] = call_data->rendition;

	    dispatch_data->transfer.input[-1] = call_data->used;
	} else {
	    status = call_data->call(call_data->back, 0, output, dispatch);
	}
    } while (0);

    if (status || !*dispatch) {
	int excess;

	excess = _libx1f4i0_lxcall_near_call(&call_data->covert);
	if (excess) {
	    if (status) {
	    } else {
		status = excess;
	    }
	}
    }

    return status;
}


/*
 * likely, used[X1f4_E4_RENDITION] is set twice for the first call and
 * uselessly for error.
 */
static int
pick_call(struct x1f4_linetext_type *linetext_data, unsigned loop,
	  void *output, void **input, struct lxtrap_type *lxtrap_data)
{
    int status;

    do {
	int (*function)(void *, void *, void **);
	struct covert_type covert;
	struct x1f4_dispatch_type *dispatch_data;
	struct x1f4_function_type *function_data;
	struct x1f4_transfer_type *transfer_data;
	void *clear[3], *context, *rendition, **used;

	clear[0] = output;
	clear[1] = input;

	used = input[-1];

	function_data = used[X1f4_E4_RENDITION];

	if (function_data->flags & X1f4_E4_POST_TYPE) {
	    status = _libx1f4i0_lxtrap_suit_line
		(lxtrap_data, &linetext_data->function, function_data, 2,
		 &covert, input);
	    if (status) {
		break;
	    }
	}

	status = _libx1f4i0_lxcall_post_call
	    (linetext_data, function_data,
	     (void *) x1f4_a1_walk_link, 2,
	     clear, POST_BITS, &lxtrap_data->link_v, &lxtrap_data->link_p,
	     &lxtrap_data->link_q, &lxtrap_data->link_e,
	     lxtrap_data->trap_e.eelookup_data);
	if (status) {
	    break;
	}

	dispatch_data = clear[2];

	transfer_data = &dispatch_data->transfer;

	rendition = used[X1f4_E4_RENDITION];

	input = transfer_data->input;
	output = transfer_data->output;
	context = transfer_data->context;

	covert.frame_data = &lxtrap_data->link_v;
	covert.scope_data = &lxtrap_data->link_x;
	covert.swap = NULL;

	function = dispatch_data->transfer.function_data->function;

	if (covert.scope_data->deck) {
	    status =
		_libx1f4i0_lxcall_mint_call(&covert, &lxtrap_data->link_e);
	    if (status) {
		break;
	    }
	}

	for (; loop; loop--) {
	    used[X1f4_E4_RENDITION] = rendition;

	    input[-1] = used;

	    status = function(context, output, input);
	    if (status) {
		break;
	    }

	    if (covert.swap) {
		status = covert.scope_data->deck(&covert.frame_data->data);
		if (status) {
		    break;
		}
	    }
	}
	{
	    int excess;

	    excess = _libx1f4i0_lxcall_near_call(&covert);
	    if (excess) {
		if (status) {
		} else {
		    status = excess;
		}
	    }
	}

	used[X1f4_E4_RENDITION] = rendition;

	if (status) {
	} else {
	    status = dispatch_data->call
		(dispatch_data->back, 0, output, &dispatch_data);
	}
    } while (0);

    return status;
}


static int
slip_call(struct x1f4_linetext_type *linetext_data, unsigned loop,
	  void **track, void *output, void **input,
	  struct lxtrap_type *lxtrap_data)
{
    int status;

    do {
	struct x1f4_function_type *function_data;
	void **used;

	used = track[-1];

	function_data = used[X1f4_E4_RENDITION];

	if (function_data->flags & X1f4_E4_POST_TYPE) {
	    struct covert_type covert;

	    status = _libx1f4i0_lxtrap_suit_line
		(lxtrap_data, &linetext_data->function, function_data, 2,
		 &covert, track);
	    if (status) {
		break;
	    }
	}

	status = _libx1f4i0_lxcall_post_call
	    (linetext_data, function_data, output, 2, input, POST_BITS,
	     &lxtrap_data->link_v, &lxtrap_data->link_p, &lxtrap_data->link_q,
	     &lxtrap_data->link_e, lxtrap_data->trap_e.eelookup_data);
	if (status) {
	    break;
	}

	loop--;
	if (loop) {
	    void *call;

	    status = lxtrap_data->link_v.link
		(lxtrap_data->link_v.data, &call, sizeof(struct slip_type));
	    if (status) {
		status = _libx1f4i0_lxtrap_stat_link(lxtrap_data);
	    } else {
		struct slip_type *call_data;
		struct x1f4_dispatch_type *dispatch_data;

		dispatch_data = input[2];

		call_data = call;

		call_data->back = dispatch_data->back;
		call_data->call = dispatch_data->call;
		call_data->loop = loop;

		call_data->used = used;
		call_data->rendition = used[X1f4_E4_RENDITION];

		dispatch_data->back = call_data;
		dispatch_data->call = part_call;

		call_data->dispatch_data = dispatch_data;

		call_data->covert.frame_data = &lxtrap_data->link_v;
		call_data->covert.scope_data = &lxtrap_data->link_x;
		call_data->covert.swap = NULL;

		if (call_data->covert.scope_data->deck) {
		    status = _libx1f4i0_lxcall_mint_call
			(&call_data->covert, &lxtrap_data->link_e);
		    if (status) {
			break;
		    }
		}
	    }
	}
    } while (0);

    return status;
}


int
_libx1f4i0_lxtrap_c_forward(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE mode;
    int status;
    struct lxtrap_type *lxtrap_data;
    struct x1f4_linetext_type *linetext_data;
    void **track;

    lxtrap_data = context;

    if (output == x1f4_a1_walk_link) {
	track = input[1];
    } else {
	track = input;
    }

    linetext_data = I_USER(&I_CASE(track[1])->data);

    mode = I_MODE(track[0]);

    if (0 < mode) {
	if (output == x1f4_a1_walk_link) {
	    status = slip_call
		(linetext_data, mode, track, output, input, lxtrap_data);
	} else {
	    status = pick_call
		(linetext_data, mode, output, input, lxtrap_data);
	}
    } else {
#if 1
	status = _libx1f4i0_lxtrap_stat_sink(lxtrap_data);
#else
	status = 0;

	if (output == x1f4_a1_walk_link) {
	    input[2] = NULL;
	}
#endif				/* 0 */
    }

    return status;
}
