/*
 * lxtrap-s.g.c
 * Copyright (C) 2011-2015, 2017-2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <copy.h>
#include <e4.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxtrap-drive.h>
#include <lxtrap-inter.h>
#include <lxtrap-types.h>

typedef struct miss_type {
    int type;
    struct x1f4_dispatch_type *dispatch_data;
    void *data, *output;
} miss_type;

extern const void *const x1f4_a1_walk_link;

static int miss_call(void *, int, void *, struct x1f4_dispatch_type **);
static int slip_call(struct lxtrap_type *, void **, int, void *);

static int
miss_call(void *text, int success, void *output,
	  struct x1f4_dispatch_type **dispatch)
{
    struct miss_type *miss_data;
    struct x1f4_dispatch_type *dispatch_data;

    miss_data = text;

    dispatch_data = miss_data->dispatch_data;

    success = dispatch_data->call
	(dispatch_data->back, success, output, dispatch);
    if (success) {
    } else {
	if (*dispatch) {
	} else {
	    int type;
	    struct x1f4_dxcast_type *dxcast_data;

	    dxcast_data = I_USER(miss_data->output);

	    type = miss_data->type;

	    dxcast_data->type = type;

	    copy_miss(&dxcast_data->data, type, miss_data->data);
	}
    }

    return success;
}


static int
slip_call(struct lxtrap_type *lxtrap_data, void **input, int type, void *data)
{
    int status;
    void *miss;

    status = lxtrap_data->link_v.link
	(lxtrap_data->link_v.data, &miss,
	 sizeof(struct miss_type) + sizeof(struct x1f4_dispatch_type));
    if (status) {
	status = _libx1f4i0_lxtrap_stat_link(lxtrap_data);
    } else {
	struct miss_type *miss_data;
	struct x1f4_dispatch_type *dispatch_data, *dispatch_slip;

	miss_data = miss;

	dispatch_data = (void *) (miss_data + 1);
	dispatch_slip = input[2];

	*dispatch_data = *dispatch_slip;

	dispatch_data->back = miss_data;
	dispatch_data->call = miss_call;

	miss_data->data = data;
	miss_data->type = type;

	miss_data->output = input[0];

	miss_data->dispatch_data = dispatch_slip;

	input[2] = dispatch_data;
    }

    return status;
}


int
_libx1f4i0_lxtrap_g_sequent(void *context, void *output, void **input)
{
    int status;

    do {
	X1f4_E4_C_MODE mode;
	struct lxtrap_type *lxtrap_data;
	struct x1f4_dxcast_type *dxcast_data;
	const struct x1f4_function_type *function_data;
	struct x1f4_linetext_type *linetext_data;
	unsigned bits, size;
	void **track;

	lxtrap_data = context;

	if (output == x1f4_a1_walk_link) {
	    bits = X1f4_LXCALL_POST_WAYS;
	    track = input[1];
	} else {
	    track = input;
	}

	linetext_data = I_USER(&I_CASE(track[0])->data);

	function_data = ((void **) track[-1])[X1f4_E4_RENDITION];

	mode = I_MODE(track[1]);

	size = function_data->count - 2;

	if (mode < 0) {
	    if ((X1f4_E4_C_MODE) size < -mode) {
		status = _libx1f4i0_lxtrap_stat_time(context, size, mode);
		if (1) {
		    break;
		}
	    } else {
		mode = size + mode;
	    }
	} else {
	    if (mode < size) {
	    } else {
		status = _libx1f4i0_lxtrap_stat_time(context, size, mode);
		if (1) {
		    break;
		}
	    }
	}

	/*
	 * abusing _post_call for an -output- only needed for the
	 * -object- casing.
	 */
	status = _libx1f4i0_lxcall_post_call
	    (linetext_data, function_data, output, 2, input, bits,
	     &lxtrap_data->link_v, &lxtrap_data->link_p, &lxtrap_data->link_q,
	     &lxtrap_data->link_e, lxtrap_data->trap_e.eelookup_data);
	if (status) {
	} else {
	    int type;
	    void *data;

	    mode += 2;

	    type = function_data->args[mode];

	    data = track[mode];

	    if (type == X1f4_E4_CASE) {
		dxcast_data = I_USER(data);

		data = &dxcast_data->data;

		type = dxcast_data->type;
	    }

	    if (output == x1f4_a1_walk_link) {
		status = slip_call(lxtrap_data, input, type, data);
	    } else {
		dxcast_data = I_USER(output);

		dxcast_data->type = type;

		copy_miss(&dxcast_data->data, type, data);
	    }
	}
    } while (0);

    return status;
}

