/*
 * lxvery-p.e.c
 * Copyright (C) 2008-2018, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxvery-config.h>

#include <e4.h>
#include <d_fset.h>
#include <lxvery-drive.h>
#include <lxvery-inter.h>
#include <lxvery-types.h>

extern const struct x1f4_bqnorm_type _libx1f4i0_lxvery_v_norm;

int
_libx1f4i0_lxvery_e_pending(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE head, tail;
    int status;
    struct dxvery_type *dxvery_data;
    struct lxvery_type *lxvery_data;
    struct x1f4_bqfset_type bqfset;
    void *dash, *part;

    dxvery_data = I_LINE(input[0]);

    lxvery_data = context;

    bqfset.fare = _libx1f4i0_lxvery_v_norm.fare;
    bqfset.look = _libx1f4i0_lxvery_v_norm.look;
    bqfset.move = _libx1f4i0_lxvery_v_norm.move;
    bqfset.pick = _libx1f4i0_lxvery_v_norm.pick;

    bqfset.trans = &lxvery_data->link_m;

    bqfset.size = sizeof(integral_q);

    dash = dxvery_data->dash;

    head = I_MODE(input[1]);
    tail = I_MODE(input[2]);

    status = _libx1f4i0_init_lxfset
	(&part, X1f4_BQFSET_SIZE_FRAME | X1f4_BQFSET_TRANS_MASK, &bqfset);
    if (status) {
    } else {
	int excess;

	if (tail < head) {
#if __VERY_PSET__
	    head += ~(X1f4_E4_C_BILL) 0 >> 1;
#endif				/* __VERY_PSET__ */

	    status = _libx1f4i0_tear_lxfset(dash, &head, 0, part);
	    if (status) {
	    } else {
		_libx1f4i0_high_lxfset(part);

#if __VERY_PSET__
		tail += ~(X1f4_E4_C_BILL) 0 >> 1;
#endif				/* __VERY_PSET__ */

		status = _libx1f4i0_tear_lxfset(dash, &tail, 1, part);
		if (status) {
		} else {
		    _libx1f4i0_swap_lxfset(part, dash);

		    _libx1f4i0_size_lxfset(dash, &dxvery_data->size);
		}
	    }
	} else {
	    void *half;

	    status = _libx1f4i0_init_lxfset
		(&half, X1f4_BQFSET_SIZE_FRAME | X1f4_BQFSET_TRANS_MASK,
		 &bqfset);
	    if (status) {
	    } else {
#if __VERY_PSET__
		tail += ~(X1f4_E4_C_BILL) 0 >> 1;
#endif				/* __VERY_PSET__ */

		status = _libx1f4i0_tear_lxfset(dash, &tail, 1, part);
		if (status) {
		} else {
#if __VERY_PSET__
		    head += ~(X1f4_E4_C_BILL) 0 >> 1;
#endif				/* __VERY_PSET__ */

		    status = _libx1f4i0_tear_lxfset(dash, &head, 0, half);
		    if (status) {
		    } else {
			status = _libx1f4i0_join_lxfset(dash, part);
			if (status) {
			} else {
			    _libx1f4i0_size_lxfset(dash, &dxvery_data->size);
			}
		    }
		}

		excess = x1f4_fini_lxfset(&half);
		if (excess) {
		    if (status) {
		    } else {
			status = excess;
		    }
		}
	    }
	}

	excess = x1f4_fini_lxfset(&part);
	if (excess) {
	    if (status) {
	    } else {
		status = excess;
	    }
	}
    }

    if (status) {
	status = _libx1f4i0_lxvery_stat_line(lxvery_data, 0, status);
    } else {
	l_LINE(dxvery_data, output);
    }

    return status;
}
